@@ -1,6 +1,6 @@
 /*** daisy.c -- guts for daisy dates
  *
- * Copyright (C) 2010-2012 Sebastian Freundt
+ * Copyright (C) 2010-2020 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -38,13 +38,17 @@
 
 #include "nifty.h"
 
+#if !defined DEFUN
+# define DEFUN
+#endif	/* !DEFUN */
+
 
 #if !defined DAISY_ASPECT_HELPERS_
 #define DAISY_ASPECT_HELPERS_
 #define TO_BASE(x)	((x) - DT_DAISY_BASE_YEAR)
 #define TO_YEAR(x)	((x) + DT_DAISY_BASE_YEAR)
 
-static inline __attribute__((pure)) dt_daisy_t
+static inline __attribute__((const)) dt_daisy_t
 __jan00_daisy(unsigned int year)
 {
 /* daisy's base year is both 1 mod 4 and starts on a monday, so ... */
@@ -71,80 +75,34 @@ __jan00_daisy(unsigned int year)
 	return by;
 #endif	/* WITH_FAST_ARITH */
 }
-
-static unsigned int
-__get_cumwk(unsigned int year)
-{
-/* return the number of weeks elapsed since 1917 for the first week of YEAR
- * we follow the 28y cycle */
-	unsigned int by = TO_BASE(year);
-	unsigned int add = 0U;
-
-	switch (by % 28U) {
-	case 27/*1944*/:
-	case 26/*1943*/:
-		add++;
-	case 25/*1942*/:
-	case 24/*1941*/:
-	case 23/*1940*/:
-	case 22/*1939*/:
-	case 21/*1938*/:
-	case 20/*1937*/:
-		add++;
-	case 19/*1936*/:
-	case 18/*1935*/:
-	case 17/*1934*/:
-	case 16/*1933*/:
-	case 15/*1932*/:
-		add++;
-	case 14/*1931*/:
-	case 13/*1930*/:
-	case 12/*1929*/:
-	case 11/*1928*/:
-	case 10/*1927*/:
-	case 9/*1926*/:
-		add++;
-	case 8/*1925*/:
-	case 7/*1924*/:
-	case 6/*1923*/:
-	case 5/*1922*/:
-		add++;
-	case 4/*1921*/:
-	case 3/*1920*/:
-	case 2/*1919*/:
-	case 1/*1918*/:
-	case 0/*1917*/:
-	default:
-		break;
-	}
-	return 52U * by + 5U * (by / 28U) + add;
-}
 #endif	/* DAISY_ASPECT_HELPERS_ */
 
 
 #if defined ASPECT_GETTERS && !defined DAISY_ASPECT_GETTERS_
 #define DAISY_ASPECT_GETTERS_
 
-static dt_dow_t
+static __attribute__((const)) dt_dow_t
 __daisy_get_wday(dt_daisy_t d)
 {
 /* daisy wdays are simple because the base year is chosen so that day 0
  * in the daisy calendar is a sunday */
-	return (dt_dow_t)(d % GREG_DAYS_P_WEEK);
+	return (dt_dow_t)((d % GREG_DAYS_P_WEEK) ?: DT_SUNDAY);
 }
 
-static unsigned int
+static __attribute__((const)) unsigned int
 __daisy_get_year(dt_daisy_t d)
 {
 /* given days since 1917-01-01 (Mon), compute a year */
 	unsigned int by;
 
 	if (UNLIKELY(d == 0)) {
-		return 0;
+		return 0U;
 	}
 	/* get an estimate for the year and readjust */
 	by = d / 365U;
-	if (UNLIKELY(__jan00_daisy(TO_YEAR(by)) >= d)) {
+	if (UNLIKELY(TO_YEAR(by) > DT_MAX_YEAR)) {
+		return 0U;
+	} else if (UNLIKELY(__jan00_daisy(TO_YEAR(by)) >= d)) {
 		by--;
 #if !defined WITH_FAST_ARITH
 		if (UNLIKELY(__jan00_daisy(TO_YEAR(by)) >= d)) {
@@ -155,7 +113,7 @@ __daisy_get_year(dt_daisy_t d)
 	return TO_YEAR(by);
 }
 
-static unsigned int
+static __attribute__((const)) unsigned int
 __daisy_get_yday(dt_daisy_t d)
 {
 	dt_daisy_t j00;
@@ -177,29 +135,38 @@ __daisy_get_yday(dt_daisy_t d)
 #if DT_DAISY_BASE_YEAR == 1917
 # define DT_LDN_BASE	(122068U/*lilian's 1917-01-00*/)
 # define DT_JDN_BASE	(2421228.5f/*julian's 1917-01-00*/)
+# define DT_MDN_BASE	(700170/*matlab's 1917-01-00*/)
 #elif DT_DAISY_BASE_YEAR == 1753
 # define DT_LDN_BASE	(62169U/*lilian's 1753-01-00*/)
 # define DT_JDN_BASE	(2361329.5f/*julian's 1753-01-00*/)
+# define DT_MDN_BASE	(640271/*matlab's 1753-01-00*/)
 #elif DT_DAISY_BASE_YEAR == 1601
 # define DT_LDN_BASE	(6652U/*lilian's 1601-01-00*/)
 # define DT_JDN_BASE	(2305812.5f/*julian's 1601-01-00*/)
+# define DT_MDN_BASE	(584754/*matlab's 1601-01-00*/)
 #else
 # error cannot convert to ldn, unknown base year
 #endif
 
-static dt_ldn_t
+static __attribute__((const)) dt_ldn_t
 __daisy_to_ldn(dt_daisy_t d)
 {
 	return d + DT_LDN_BASE;
 }
 
-static dt_jdn_t
+static __attribute__((const)) dt_jdn_t
 __daisy_to_jdn(dt_daisy_t d)
 {
 	return (dt_jdn_t)d + DT_JDN_BASE;
 }
 
-static __attribute__((unused)) dt_daisy_t
+static __attribute__((const)) dt_mdn_t
+__daisy_to_mdn(dt_daisy_t d)
+{
+	return (dt_mdn_t)d + DT_MDN_BASE;
+}
+
+static __attribute__((const)) dt_daisy_t
 __ldn_to_daisy(dt_ldn_t d)
 {
 	dt_sdaisy_t tmp;
@@ -210,7 +177,7 @@ __ldn_to_daisy(dt_ldn_t d)
 	return 0U;
 }
 
-static __attribute__((unused)) dt_daisy_t
+static __attribute__((const)) dt_daisy_t
 __jdn_to_daisy(dt_jdn_t d)
 {
 	float tmp;
@@ -220,7 +187,18 @@ __jdn_to_daisy(dt_jdn_t d)
 	return 0U;
 }
 
-static dt_ymd_t
+static __attribute__((const)) dt_daisy_t
+__mdn_to_daisy(dt_mdn_t d)
+{
+	dt_sdaisy_t tmp;
+
+	if ((tmp = d - DT_MDN_BASE) > 0) {
+		return (dt_daisy_t)tmp;
+	}
+	return 0U;
+}
+
+DEFUN __attribute__((const)) dt_ymd_t
 __daisy_to_ymd(dt_daisy_t that)
 {
 	dt_daisy_t j00;
@@ -230,8 +208,9 @@ __daisy_to_ymd(dt_daisy_t that)
 
 	if (UNLIKELY(that == 0)) {
 		return (dt_ymd_t){.u = 0};
+	} else if (UNLIKELY(!(y = __daisy_get_year(that)))) {
+		return (dt_ymd_t){.u = 0};
 	}
-	y = __daisy_get_year(that);
 	j00 = __jan00_daisy(y);
 	doy = that - j00;
 	md = __yday_get_md(y, doy);
@@ -248,7 +227,7 @@ __daisy_to_ymd(dt_daisy_t that)
 #endif	/* HAVE_ANON_STRUCTS_INIT */
 }
 
-static dt_ymcw_t
+static __attribute__((const)) dt_ymcw_t
 __daisy_to_ymcw(dt_daisy_t that)
 {
 	dt_ymd_t tmp;
@@ -275,35 +254,18 @@ __daisy_to_ymcw(dt_daisy_t that)
 #endif
 }
 
-static dt_ywd_t
+static __attribute__((const)) dt_ywd_t
 __daisy_to_ywd(dt_daisy_t that)
 {
-	unsigned int wk = (that + 6) / 7;
-	unsigned int wd = (that + 6) % 7;
-	unsigned int y;
-	unsigned int yw;
-	dt_ywd_t res;
+	const unsigned int wd = (that + GREG_DAYS_P_WEEK - 1) % GREG_DAYS_P_WEEK;
+	dt_dow_t dow = (dt_dow_t)(wd + 1U);
+	unsigned int y = __daisy_get_year(that);
+	int yd = that - __jan00_daisy(y);
 
-	/* get an estimate for the year and readjust */
-	y = __daisy_get_year(that);
-	/* get the cumulative week count */
-	if (UNLIKELY((yw = wk - __get_cumwk(y)) == 0)) {
-		y--;
-		yw = __get_isowk(y);
-	} else if (UNLIKELY(yw > __get_isowk(y))) {
-		/* hanging over into the new year */
-		yw = 1U;
-		y++;
-	}
-
-	/* final assignment */
-	res.y = y;
-	res.c = yw;
-	res.w = (wd + 1U) % 7U;
-	return res;
+	return __make_ywd_yd_dow(y, yd, dow);
 }
 
-static dt_yd_t
+static __attribute__((const)) dt_yd_t
 __daisy_to_yd(dt_daisy_t d)
 {
 	int yd = __daisy_get_yday(d);
@@ -327,7 +289,7 @@ __daisy_to_yd(dt_daisy_t d)
 #include "daisy.c"
 #undef ASPECT_GETTERS
 
-static dt_daisy_t
+static __attribute__((const)) dt_daisy_t
 __daisy_add_d(dt_daisy_t d, int n)
 {
 /* add N days to D */
@@ -335,7 +297,7 @@ __daisy_add_d(dt_daisy_t d, int n)
 	return d;
 }
 
-static dt_daisy_t
+static __attribute__((const)) dt_daisy_t
 __daisy_add_b(dt_daisy_t d, int n)
 {
 /* add N business days to D */
@@ -345,7 +307,7 @@ __daisy_add_b(dt_daisy_t d, int n)
 	return d;
 }
 
-static dt_daisy_t
+static __attribute__((const)) dt_daisy_t
 __daisy_add_w(dt_daisy_t d, int n)
 {
 /* add N weeks to D */
@@ -356,15 +318,12 @@ __daisy_add_w(dt_daisy_t d, int n)
 
 #if defined ASPECT_DIFF && !defined DAISY_ASPECT_DIFF_
 #define DAISY_ASPECT_DIFF_
-static struct dt_d_s
+static __attribute__((const)) struct dt_ddur_s
 __daisy_diff(dt_daisy_t d1, dt_daisy_t d2)
 {
 /* compute d2 - d1 */
-	struct dt_d_s res = {.typ = DT_DAISY, .dur = 1};
 	int32_t diff = d2 - d1;
-
-	res.daisydur = diff;
-	return res;
+	return dt_make_ddur(DT_DURD, diff);
 }
 #endif	/* ASPECT_DIFF */
 
@@ -372,7 +331,7 @@ __daisy_diff(dt_daisy_t d1, dt_daisy_t d2)
 #if defined ASPECT_STRF && !defined DAISY_ASPECT_STRF_
 #define DAISY_ASPECT_STRF_
 
-static void
+DEFUN void
 __prep_strfd_daisy(struct strpd_s *tgt, dt_daisy_t d)
 {
 	dt_ymd_t tmp = __daisy_to_ymd(d);
