@@ -1,6 +1,6 @@
 /*** date-core-strpf.c -- parser and formatter funs for date-core
  *
- * Copyright (C) 2011-2012 Sebastian Freundt
+ * Copyright (C) 2011-2020 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -45,6 +45,7 @@
 #include "token.h"
 #include "date-core.h"
 #include "date-core-strpf.h"
+#include "dt-locale.h"
 
 #if defined __INTEL_COMPILER
 /* we MUST return a char* */
@@ -53,168 +54,96 @@
 # pragma GCC diagnostic ignored "-Wcast-qual"
 #endif	/* __INTEL_COMPILER */
 
-static const char *__long_wday[] = {
-	"Sunday",
-	"Monday",
-	"Tuesday",
-	"Wednesday",
-	"Thursday",
-	"Friday",
-	"Saturday",
-	"Miracleday",
-};
-DEFVAR const char **dut_long_wday = __long_wday;
-DEFVAR const ssize_t dut_nlong_wday = countof(__long_wday);
-
-static const char *__abbr_wday[] = {
-	"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Mir",
-};
-DEFVAR const char **dut_abbr_wday = __abbr_wday;
-DEFVAR const ssize_t dut_nabbr_wday = countof(__abbr_wday);
-
-static const char __abab_wday[] = "SMTWRFAX";
-DEFVAR const char *dut_abab_wday = __abab_wday;
-DEFVAR const ssize_t dut_nabab_wday = countof(__abab_wday);
-
-static const char *__long_mon[] = {
-	"Miraculary",
-	"January",
-	"February",
-	"March",
-	"April",
-	"May",
-	"June",
-	"July",
-	"August",
-	"September",
-	"October",
-	"November",
-	"December",
-};
-DEFVAR const char **dut_long_mon = __long_mon;
-DEFVAR const ssize_t dut_nlong_mon = countof(__long_mon);
-
-static const char *__abbr_mon[] = {
-	"Mir",
-	"Jan",
-	"Feb",
-	"Mar",
-	"Apr",
-	"May",
-	"Jun",
-	"Jul",
-	"Aug",
-	"Sep",
-	"Oct",
-	"Nov",
-	"Dec",
-};
-DEFVAR const char **dut_abbr_mon = __abbr_mon;
-DEFVAR const ssize_t dut_nabbr_mon = countof(__abbr_mon);
-
-/* futures expiry codes, how convenient */
-static const char __abab_mon[] = "_FGHJKMNQUVXZ";
-DEFVAR const char *dut_abab_mon = __abab_mon;
-DEFVAR const ssize_t dut_nabab_mon = countof(__abab_mon);
+#if !defined DEFUN
+# define DEFUN
+#endif	/* !DEFUN */
+#if !defined DEFVAR
+# define DEFVAR
+#endif	/* !DEFVAR */
 
 
-DEFUN inline void
-__fill_strpdi(struct strpdi_s *tgt, struct dt_d_s dur)
-{
-	switch (dur.typ) {
-	case DT_YMD:
-		tgt->y = dur.ymd.y;
-		tgt->m = dur.ymd.m;
-		tgt->d = dur.ymd.d;
-		break;
-	case DT_DAISY:
-		tgt->d = dur.daisydur;
-		/* we don't need the negation, so return here */
-		return;
-	case DT_BIZSI:
-		tgt->b = dur.bizsidur;
-		/* we don't need the negation, so return here */
-		return;
-	case DT_BIZDA:
-		tgt->y = dur.bizda.y;
-		tgt->m = dur.bizda.m;
-		tgt->b = dur.bizda.bd;
-		break;
-	case DT_YMCW:
-		tgt->y = dur.ymcw.y;
-		tgt->m = dur.ymcw.m;
-		tgt->w = dur.ymcw.c;
-		tgt->d = dur.ymcw.w;
-		break;
-	case DT_MD:
-		tgt->m = dur.md.m;
-		tgt->d = dur.md.d;
-		break;
-	default:
-		break;
-	}
-
-	if (UNLIKELY(dur.neg)) {
-		tgt->y = -tgt->y;
-		tgt->m = -tgt->m;
-		tgt->d = -tgt->d;
-		tgt->w = -tgt->w;
-		tgt->b = -tgt->b;
-	}
-	return;
-}
-
 DEFUN struct dt_d_s
 __strpd_std(const char *str, char **ep)
 {
 /* code dupe, see __strpdt_std() */
-	struct dt_d_s res;
-	struct strpd_s d;
+	struct dt_d_s res = {DT_DUNK};
+	struct strpd_s d = {0};
 	const char *sp;
 
 	if ((sp = str) == NULL) {
 		goto out;
 	}
 
-	d = strpd_initialiser();
+	d.c = -1;
 	/* read the year */
-	if ((d.y = strtoi_lim(sp, &sp, DT_MIN_YEAR, DT_MAX_YEAR)) < 0 ||
-	    *sp++ != '-') {
+	d.y = strtoi32(sp, &sp);
+	if (d.y < DT_MIN_YEAR || d.y > DT_MAX_YEAR || *sp++ != '-') {
 		goto fucked;
 	}
 	/* check for ywd dates */
 	if (UNLIKELY(*sp == 'W')) {
 		/* brilliant */
-		if ((sp++, d.c = strtoi_lim(sp, &sp, 0, 53)) < 0 ||
-		    *sp++ != '-') {
+		sp++, d.c = strtoi32(sp, &sp);
+		if (d.c < 0 || d.c > 53) {
 			goto fucked;
 		}
 		d.flags.c_wcnt_p = 1;
 		d.flags.wk_cnt = YWD_ISOWK_CNT;
-		goto dow;
-	}
-	/* read the month */
-	if ((d.m = strtoi_lim(sp, &sp, 0, GREG_MONTHS_P_YEAR)) < 0 ||
-	    *sp++ != '-') {
+		if (*sp == '-') {
+			goto dow;
+		} else if ((unsigned char)*sp <= ' ') {
+			/* unspecified week-date */
+			goto guess;
+		}
 		goto fucked;
 	}
-	/* read the day or the count */
-	if ((d.d = strtoi_lim(sp, &sp, 0, 31)) < 0) {
-		/* didn't work, fuck off */
-		goto fucked;
+	/* read the month, then day count */
+	with (const char *tmp) {
+		d.m = strtoi32(sp, &tmp);
+		if (d.m < 0 || d.m > 366) {
+			goto fucked;
+		} else if (UNLIKELY(*tmp != '-')) {
+			/* oh, could be an ordinal date */
+			if (tmp - sp >= 3U) {
+				d.d = d.m;
+				d.m = 0U;
+				d.flags.d_dcnt_p = 1U;
+			} else if ((unsigned int)d.m <= GREG_MONTHS_P_YEAR) {
+				;
+			} else {
+				goto fucked;
+			}
+			switch (*(sp = tmp)) {
+			case 'B':
+				/* it's a bizda/YMDU before ultimo date */
+				d.flags.ab = BIZDA_BEFORE;
+			case 'b':
+				/* it's a bizda/YMDU after ultimo date */
+				d.flags.bizda = 1;
+				sp++;
+				/* we currently don't support
+				 * business day of year calendars */
+				goto fucked;
+			}
+			goto guess;
+		} else if (d.d = strtoi32(++tmp, &sp), d.d < 0 || d.d > 31) {
+			/* didn't work, fuck off */
+			goto fucked;
+		}
 	}
 	/* check the date type */
 	switch (*sp) {
 	case '-':
 		/* it is a YMCW date */
-		if ((d.c = d.d) > 5) {
+		if (d.d > 5) {
 			/* nope, it was bollocks */
-			break;
+			goto fucked;
 		}
-		d.d = 0;
-		sp++;
+		d.c = d.d, d.d = 0;
 	dow:
-		if ((d.w = strtoi_lim(sp, &sp, 0, GREG_DAYS_P_WEEK)) < 0) {
+		sp++;
+		if (d.w = strtoi32(sp, &sp),
+		    d.w < 0 || (unsigned int)d.w > GREG_DAYS_P_WEEK) {
 			/* didn't work, fuck off */
 			goto fucked;
 		}
@@ -233,6 +162,7 @@ __strpd_std(const char *str, char **ep)
 		/* we don't care */
 		break;
 	}
+guess:
 	/* guess what we're doing */
 	res = __guess_dtyp(d);
 out:
@@ -244,7 +174,7 @@ fucked:
 	if (ep != NULL) {
 		*ep = (char*)str;
 	}
-	return dt_d_initialiser();
+	return (struct dt_d_s){DT_DUNK};
 }
 
 DEFUN int
@@ -266,15 +196,57 @@ __strpd_card(struct strpd_s *d, const char *sp, struct dt_spec_s s, char **ep)
 		res = 0 - (d->y < 0 || d->m < 0 || d->d < 0);
 		break;
 	case DT_SPFL_N_YEAR:
-		if (s.abbr == DT_SPMOD_NORM) {
+		switch (s.abbr) {
+		case DT_SPMOD_LONG:
 			d->y = strtoi_lim(sp, &sp, DT_MIN_YEAR, DT_MAX_YEAR);
-		} else if (s.abbr == DT_SPMOD_ABBR) {
+			break;
+		case DT_SPMOD_NORM:
 			d->y = strtoi_lim(sp, &sp, 0, 99);
 			if (UNLIKELY(d->y < 0)) {
-				;
-			} else if ((d->y += 2000) > 2068) {
+				break;
+			}
+#if defined LIBDUT
+			with (struct dt_d_s b = dt_get_dbase()) {
+				unsigned int by = b.ymd.y - 50U;
+				int c = by / 100U;
+				int y = by % 100U;
+
+				if (d->y < y) {
+					c++;
+				}
+				d->y += c * 100;
+			}
+#else  /* !LIBDUT */
+			/* hardcode base as 2015-01-01 */
+			if ((d->y += 2000) > 2065) {
 				d->y -= 100;
 			}
+#endif	/* LIBDUT */
+			break;
+		case DT_SPMOD_ABBR:
+			if (UNLIKELY((unsigned char)(*sp ^ '0') >= 10)) {
+				d->y = -1;
+				break;
+			}
+			d->y = (*sp++ ^ '0');
+#if defined LIBDUT
+			with (struct dt_d_s b = dt_get_dbase()) {
+				unsigned int by = b.ymd.y;
+				int c = by / 10U;
+				int y = by % 10U;
+
+				if (d->y < y) {
+					c++;
+				}
+				d->y += c * 10;
+			}
+#else  /* !LIBDUT */
+			/* hardcode base as 2015-01-01 */
+			if ((d->y += 2010) < 2015) {
+				d->y += 10;
+			}
+#endif	/* LIBDUT */
+			break;
 		}
 		res = 0 - (d->y < 0);
 		break;
@@ -285,7 +257,7 @@ __strpd_card(struct strpd_s *d, const char *sp, struct dt_spec_s s, char **ep)
 	case DT_SPFL_N_DCNT_MON:
 		/* ymd mode? */
 		if (LIKELY(!s.bizda)) {
-			d->d = strtoi_lim(sp, &sp, 0, 31);
+			d->d = padstrtoi_lim(sp, &sp, 0, 31);
 			res = 0 - (d->d < 0);
 		} else {
 			d->b = strtoi_lim(sp, &sp, 0, 23);
@@ -295,6 +267,7 @@ __strpd_card(struct strpd_s *d, const char *sp, struct dt_spec_s s, char **ep)
 	case DT_SPFL_N_DCNT_WEEK:
 		/* ymcw mode? */
 		d->w = strtoi_lim(sp, &sp, 0, GREG_DAYS_P_WEEK);
+		/* fix up d->w right away */
 		res = 0 - (d->w < 0);
 		break;
 	case DT_SPFL_N_WCNT_MON:
@@ -420,16 +393,27 @@ __strpd_rom(struct strpd_s *d, const char *sp, struct dt_spec_s s, char **ep)
 		break;
 
 	case DT_SPFL_N_YEAR:
-		if (s.abbr == DT_SPMOD_NORM) {
+		switch (s.abbr) {
+		case DT_SPMOD_LONG:
 			d->y = romstrtoi_lim(
 				sp, &sp, DT_MIN_YEAR, DT_MAX_YEAR);
-		} else if (s.abbr == DT_SPMOD_ABBR) {
+			break;
+		case DT_SPMOD_NORM:
 			d->y = romstrtoi_lim(sp, &sp, 0, 99);
 			if (UNLIKELY(d->y < 0)) {
 				;
 			} else if ((d->y += 2000) > 2068) {
 				d->y -= 100;
 			}
+			break;
+		case DT_SPMOD_ABBR:
+			d->y = romstrtoi_lim(sp, &sp, 0, 9);
+			if (UNLIKELY(d->y < 0)) {
+				;
+			} else {
+				d->y += 2010;
+			}
+			break;
 		}
 		res = 0 - (d->y < 0);
 		break;
@@ -495,51 +479,66 @@ __strfd_card(
 	case DT_SPFL_UNK:
 		break;
 	case DT_SPFL_N_DSTD:
-		if (UNLIKELY(!d->m && !d->d)) {
+		if (UNLIKELY(!d->m && (!d->d || d->flags.d_dcnt_p))) {
 			__strfd_get_md(d, that);
 		} else if (UNLIKELY(!d->d)) {
 			__strfd_get_d(d, that);
 		}
 		if (LIKELY(bsz >= 10)) {
-			ui32tostr(buf + 0, bsz, d->y, 4);
+			ui9999topstr(buf + 0, bsz, d->y, 4, '0');
 			buf[4] = '-';
-			res = ui32tostr(buf + 5, bsz, d->m, 2);
+			ui99topstr(buf + 5, bsz, d->m, 2, '0');
 			buf[7] = '-';
-			ui32tostr(buf + 8, bsz, d->d, 2);
+			ui99topstr(buf + 8, bsz, d->d, 2, '0');
 			res = 10;
 		}
 		break;
 	case DT_SPFL_N_YEAR: {
 		unsigned int y = d->y;
-		int prec = 4;
+		int prec;
 
 		if (UNLIKELY(s.tai && d->flags.real_y_in_q)) {
 			y = d->q;
+		} else if (s.tai) {
+			/* oh we want ISO week dates */
+			y = dt_dconv(DT_YWD, that).ywd.y;
 		}
-		if (UNLIKELY(s.abbr == DT_SPMOD_ABBR)) {
+		switch (s.abbr) {
+		case DT_SPMOD_LONG:
+			prec = 4;
+			break;
+		case DT_SPMOD_NORM:
 			prec = 2;
+			break;
+		case DT_SPMOD_ABBR:
+			prec = 1;
+			break;
+		default:
+			/* it's just bollocks */
+			return 0U;
 		}
-		res = ui32tostr(buf, bsz, y, prec);
+		res = ui9999topstr(buf, prec, y, 4U, padchar(s));
 		break;
 	}
 	case DT_SPFL_N_MON:
-		if (UNLIKELY(!d->m && !d->d)) {
+		if (UNLIKELY(!d->m && (!d->d || d->flags.d_dcnt_p))) {
 			__strfd_get_md(d, that);
 		} else if (UNLIKELY(!d->m)) {
 			__strfd_get_m(d, that);
 		}
-		res = ui32tostr(buf, bsz, d->m, 2);
+		res = ui99topstr(
+			buf, bsz, d->m,
+			2 - (s.pad == DT_SPPAD_OMIT), padchar(s));
 		break;
 	case DT_SPFL_N_DCNT_MON: {
 		/* ymd mode check? */
 		unsigned int pd;
 
 		if (LIKELY(!s.bizda)) {
-			if (UNLIKELY(!d->m && !d->d)) {
+			if (UNLIKELY(!d->m && (!d->d || d->flags.d_dcnt_p))) {
 				__strfd_get_md(d, that);
 			} else if (UNLIKELY(!d->d)) {
 				__strfd_get_d(d, that);
-				pd = d->d;
 			}
 			pd = d->d;
 		} else {
@@ -547,21 +546,17 @@ __strfd_card(
 			pd = dt_get_bday_q(
 				that, __make_bizda_param(s.ab, BIZDA_ULTIMO));
 		}
-		res = ui32tostr(buf, bsz, pd, 2);
+		res = ui99topstr(
+			buf, bsz, pd, 2 - (s.pad == DT_SPPAD_OMIT), padchar(s));
 		break;
 	}
 	case DT_SPFL_N_DCNT_WEEK:
 		/* ymcw mode check */
-		if (!d->w) {
-			d->w = dt_get_wday(that);
+		with (unsigned int w = (unsigned)d->w ?: dt_get_wday(that)) {
+			const unsigned int ymcwp = s.wk_cnt != YWD_MONWK_CNT;
+
+			res = ui99topstr(buf, bsz, w, 1 + ymcwp, padchar(s));
 		}
-		if (UNLIKELY(d->w == 0)) {
-			if (s.wk_cnt == YWD_MONWK_CNT) {
-				/* turn Sun 00 to Sun 07 */
-				d->w = 7;
-			}
-		}
-		res = ui32tostr(buf, bsz, d->w, 2);
 		break;
 	case DT_SPFL_N_WCNT_MON: {
 		unsigned int c = d->c;
@@ -571,7 +566,8 @@ __strfd_card(
 			/* don't store it */
 			c = (unsigned int)dt_get_wcnt_mon(that);
 		}
-		res = ui32tostr(buf, bsz, c, 2);
+		res = ui99topstr(
+			buf, bsz, c, 2 - (s.pad == DT_SPPAD_OMIT), padchar(s));
 		break;
 	}
 	case DT_SPFL_S_WDAY:
@@ -581,12 +577,12 @@ __strfd_card(
 		case DT_SPMOD_NORM:
 			res = arritostr(
 				buf, bsz, d->w,
-				dut_abbr_wday, dut_nabbr_wday);
+				duf_abbr_wday, dut_nabbr_wday);
 			break;
 		case DT_SPMOD_LONG:
 			res = arritostr(
 				buf, bsz, d->w,
-				dut_long_wday, dut_nlong_wday);
+				duf_long_wday, dut_nlong_wday);
 			break;
 		case DT_SPMOD_ABBR:
 			/* super abbrev'd wday */
@@ -604,12 +600,12 @@ __strfd_card(
 		case DT_SPMOD_NORM:
 			res = arritostr(
 				buf, bsz, d->m,
-				dut_abbr_mon, dut_nabbr_mon);
+				duf_abbr_mon, dut_nabbr_mon);
 			break;
 		case DT_SPMOD_LONG:
 			res = arritostr(
 				buf, bsz, d->m,
-				dut_long_mon, dut_nlong_mon);
+				duf_long_mon, dut_nlong_mon);
 			break;
 		case DT_SPMOD_ABBR:
 			/* super abbrev'd month */
@@ -657,7 +653,10 @@ __strfd_card(
 					that.bizda, __get_bizda_param(that));
 			}
 			if (yd >= 0) {
-				res = ui32tostr(buf, bsz, yd, 3);
+				res = ui999topstr(
+					buf, bsz, yd,
+					3 - (s.pad == DT_SPPAD_OMIT) << 1U,
+					padchar(s));
 			} else {
 				buf[res++] = '0';
 				buf[res++] = '0';
@@ -665,6 +664,11 @@ __strfd_card(
 			}
 			break;
 		}
+		case DT_YD:
+			res = ui999topstr(
+				buf, bsz, d->d,
+				3 - (s.pad == DT_SPPAD_OMIT) << 1U, padchar(s));
+			break;
 		case DT_LDN:
 			res = snprintf(buf, bsz, "%u", that.ldn);
 			break;
@@ -677,7 +681,8 @@ __strfd_card(
 		break;
 	case DT_SPFL_N_WCNT_YEAR: {
 		int yw = dt_get_wcnt_year(that, s.wk_cnt);
-		res = ui32tostr(buf, bsz, yw, 2);
+		res = ui99topstr(
+			buf, bsz, yw, 2 - (s.pad == DT_SPPAD_OMIT), padchar(s));
 		break;
 	}
 	}
@@ -701,12 +706,20 @@ __strfd_rom(
 	case DT_SPFL_UNK:
 		break;
 	case DT_SPFL_N_YEAR:
-		if (s.abbr == DT_SPMOD_NORM) {
+		switch (s.abbr) {
+		case DT_SPMOD_LONG:
 			res = ui32tostrrom(buf, bsz, d->y);
 			break;
-		} else if (s.abbr == DT_SPMOD_ABBR) {
+		case DT_SPMOD_NORM:
 			res = ui32tostrrom(buf, bsz, d->y % 100);
 			break;
+		case DT_SPMOD_ABBR:
+			res = ui32tostrrom(buf, bsz, d->y % 10);
+			break;
+		case DT_SPMOD_ILL:
+		default:
+			/* should be impossible */
+			break;
 		}
 		break;
 	case DT_SPFL_N_MON:
@@ -732,7 +745,7 @@ __strfd_rom(
 DEFUN size_t
 __strfd_dur(
 	char *buf, size_t bsz, struct dt_spec_s s,
-	struct strpd_s *d, struct dt_d_s UNUSED(that))
+	struct strpd_s *d, struct dt_ddur_s UNUSED(that))
 {
 	size_t res = 0;
 
