@@ -1,6 +1,6 @@
 /*** date-core.c -- our universe of dates
  *
- * Copyright (C) 2011-2012 Sebastian Freundt
+ * Copyright (C) 2011-2020 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -46,7 +46,11 @@
 #include <stdbool.h>
 #include <sys/time.h>
 #include <time.h>
+#include <assert.h>
+#include <errno.h>
+#include <limits.h>
 #include "date-core.h"
+#include "date-core-private.h"
 #include "strops.h"
 #include "token.h"
 #include "nifty.h"
@@ -60,6 +64,13 @@
 # pragma GCC diagnostic ignored "-Wcast-qual"
 #endif	/* __INTEL_COMPILER */
 
+#if !defined DEFUN
+# define DEFUN
+#endif	/* !DEFUN */
+#if !defined DEFVAR
+# define DEFVAR
+#endif	/* !DEFVAR */
+
 #if !defined assert
 # define assert(x)
 #endif	/* !assert */
@@ -89,95 +100,11 @@ struct __md_s {
 
 
 /* helpers */
-#if 1
-static uint16_t __mon_yday[] = {
-/* this is \sum ml, first element is a bit set of leap days to add */
-	0xfff8, 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365
-};
-#endif
+#include "gmtime.h"
 
 /* bizda definitions, reference dates */
 static __attribute__((unused)) const char *bizda_ult[] = {"ultimo", "ult"};
 
-static inline bool
-__leapp(unsigned int y)
-{
-#if defined WITH_FAST_ARITH
-	return y % 4 == 0;
-#else  /* !WITH_FAST_ARITH */
-	return y % 4 == 0 && (y % 100 != 0 || y % 400 == 0);
-#endif	/* WITH_FAST_ARITH */
-}
-
-/* UTC has a constant day length */
-#define UTC_SECS_PER_DAY	(86400)
-
-static void
-ffff_gmtime(struct tm *tm, const time_t t)
-{
-	register int days;
-	register unsigned int yy;
-	const uint16_t *ip;
-
-	/* just go to day computation */
-	days = (int)(t / UTC_SECS_PER_DAY);
-	/* week day computation, that one's easy, 1 jan '70 was Thu */
-	tm->tm_wday = (days + 4) % GREG_DAYS_P_WEEK;
-
-	/* gotta do the date now */
-	yy = 1970;
-	/* stolen from libc */
-#define DIV(a, b)		((a) / (b))
-/* we only care about 1901 to 2099 and there are no bullshit leap years */
-#define LEAPS_TILL(y)		(DIV(y, 4))
-	while (days < 0 || days >= (!__leapp(yy) ? 365 : 366)) {
-		/* Guess a corrected year, assuming 365 days per year. */
-		register unsigned int yg = yy + days / 365 - (days % 365 < 0);
-
-		/* Adjust DAYS and Y to match the guessed year.  */
-		days -= (yg - yy) * 365 +
-			LEAPS_TILL(yg - 1) - LEAPS_TILL(yy - 1);
-		yy = yg;
-	}
-	/* set the year */
-	tm->tm_year = (int)yy;
-
-	ip = __mon_yday;
-	/* unrolled */
-	yy = 13;
-	if (days < ip[--yy] &&
-	    days < ip[--yy] &&
-	    days < ip[--yy] &&
-	    days < ip[--yy] &&
-	    days < ip[--yy] &&
-	    days < ip[--yy] &&
-	    days < ip[--yy] &&
-	    days < ip[--yy] &&
-	    days < ip[--yy] &&
-	    days < ip[--yy] &&
-	    days < ip[--yy]) {
-		yy = 1;
-	}
-	/* set the rest of the tm structure */
-	tm->tm_mday = days - ip[yy] + 1;
-	tm->tm_yday = days;
-	tm->tm_mon = (int)yy;
-	/* fix up leap years */
-	if (UNLIKELY(__leapp(tm->tm_year))) {
-		if ((ip[0] >> (yy)) & 1) {
-			if (UNLIKELY(tm->tm_yday == 59)) {
-				tm->tm_mon = 2;
-				tm->tm_mday = 29;
-			} else if (UNLIKELY(tm->tm_yday == ip[yy])) {
-				tm->tm_mday = tm->tm_yday - ip[--tm->tm_mon];
-			} else {
-				tm->tm_mday--;
-			}
-		}
-	}
-	return;
-}
-
 /* arithmetics helpers */
 static inline unsigned int
 __uimod(signed int x, signed int m)
@@ -194,6 +121,12 @@ __uidiv(signed int x, signed int m)
 	return x >= 0 ? res : x % m ? res - 1 : res;
 }
 
+static inline __attribute__((const, pure)) int
+__sgn(signed int x)
+{
+	return (x > 0) - (x < 0);
+}
+
 
 /* helpers from the calendar files, don't define any aspect, so only
  * the helpers should get included */
@@ -223,47 +156,6 @@ __uidiv(signed int x, signed int m)
 #include "daisy.c"
 #undef ASPECT_CONV
 
-static int
-__ymcw_cmp(dt_ymcw_t d1, dt_ymcw_t d2)
-{
-	if (d1.y < d2.y) {
-		return -1;
-	} else if (d1.y > d2.y) {
-		return 1;
-	} else if (d1.m < d2.m) {
-		return -1;
-	} else if (d1.m > d2.m) {
-		return 1;
-	}
-
-	/* we're down to counts, however, the last W of a month is always
-	 * count 5, even though counting forward it would be 4 */
-	if (d1.c < d2.c) {
-		return -1;
-	} else if (d1.c > d2.c) {
-		return 1;
-	}
-	/* now it's up to the first of the month */
-	{
-		dt_dow_t wd01;
-		unsigned int off1;
-		unsigned int off2;
-
-		wd01 = __get_m01_wday(d1.y, d1.m);
-		/* represent cw as C-th WD01 + OFF */
-		off1 = __uimod(d1.w - wd01, GREG_DAYS_P_WEEK);
-		off2 = __uimod(d2.w - wd01, GREG_DAYS_P_WEEK);
-
-		if (off1 < off2) {
-			return -1;
-		} else if (off1 > off2) {
-			return 1;
-		} else {
-			return 0;
-		}
-	}
-}
-
 
 /* converting accessors */
 DEFUN int
@@ -358,13 +250,15 @@ dt_get_md(struct dt_d_s that)
 		return (struct __md_s){.m = that.ymcw.m, .d = d};
 	}
 	case DT_YWD:
-		/* should have come throught the GETTERS aspect */
+		/* should have come through the GETTERS aspect */
 		return __ywd_get_md(that.ywd);
+	case DT_YD:
+		return __yd_get_md(that.yd);
 	}
 }
 
-/* too exotic to be public */
-static int
+/* too exotic to be public, nope bug #81 needs it */
+int
 dt_get_wcnt_mon(struct dt_d_s that)
 {
 	if (LIKELY(that.typ == DT_YMCW)) {
@@ -390,7 +284,7 @@ dt_get_wcnt_mon(struct dt_d_s that)
 /* forward decl */
 static dt_yd_t dt_conv_to_yd(struct dt_d_s this);
 
-static int
+DEFUN int
 dt_get_wcnt_year(struct dt_d_s this, unsigned int wkcnt_convention)
 {
 	int res;
@@ -413,7 +307,20 @@ dt_get_wcnt_year(struct dt_d_s this, unsigned int wkcnt_convention)
 		case YWD_SUNWK_CNT: {
 			/* using monwk_cnt is a minor trick
 			 * from = 1 = Mon or 0 = Sun */
-			int from = wkcnt_convention == YWD_MONWK_CNT;
+			dt_dow_t from;
+
+			switch (wkcnt_convention) {
+			case YWD_MONWK_CNT:
+				from = DT_MONDAY;
+				break;
+			case YWD_SUNWK_CNT:
+				from = DT_SUNDAY;
+				break;
+			default:
+				/* huh? */
+				from = DT_MIRACLEDAY;
+				break;
+			}
 			res = __yd_get_wcnt(yd, from);
 			break;
 		}
@@ -537,7 +444,7 @@ dt_get_quarter(struct dt_d_s that)
 
 
 /* converters */
-static dt_daisy_t
+DEFUN dt_daisy_t
 dt_conv_to_daisy(struct dt_d_s that)
 {
 	switch (that.typ) {
@@ -551,6 +458,14 @@ dt_conv_to_daisy(struct dt_d_s that)
 		return __ywd_to_daisy(that.ywd);
 	case DT_BIZDA:
 		return __bizda_to_daisy(that.bizda, __get_bizda_param(that));
+	case DT_LDN:
+		return __ldn_to_daisy(that.ldn);
+	case DT_JDN:
+		return __jdn_to_daisy(that.jdn);
+	case DT_MDN:
+		return __mdn_to_daisy(that.mdn);
+	case DT_YD:
+		return __yd_to_daisy(that.yd);
 	case DT_DUNK:
 	default:
 		break;
@@ -566,12 +481,24 @@ dt_conv_to_ymd(struct dt_d_s that)
 		return that.ymd;
 	case DT_YMCW:
 		return __ymcw_to_ymd(that.ymcw);
+	case DT_JDN:
+		that.daisy = __jdn_to_daisy(that.jdn);
+		goto daisy;
+	case DT_LDN:
+		that.daisy = __ldn_to_daisy(that.ldn);
+		goto daisy;
+	case DT_MDN:
+		that.daisy = __mdn_to_daisy(that.mdn);
+		goto daisy;
 	case DT_DAISY:
+	daisy:
 		return __daisy_to_ymd(that.daisy);
 	case DT_BIZDA:
 		return __bizda_to_ymd(that.bizda);
 	case DT_YWD:
 		return __ywd_to_ymd(that.ywd);
+	case DT_YD:
+		return __yd_to_ymd(that.yd);
 	case DT_DUNK:
 	default:
 		break;
@@ -587,12 +514,24 @@ dt_conv_to_ymcw(struct dt_d_s that)
 		return __ymd_to_ymcw(that.ymd);
 	case DT_YMCW:
 		return that.ymcw;
+	case DT_JDN:
+		that.daisy = __jdn_to_daisy(that.jdn);
+		goto daisy;
+	case DT_LDN:
+		that.daisy = __ldn_to_daisy(that.ldn);
+		goto daisy;
+	case DT_MDN:
+		that.daisy = __mdn_to_daisy(that.mdn);
+		goto daisy;
 	case DT_DAISY:
+	daisy:
 		return __daisy_to_ymcw(that.daisy);
 	case DT_BIZDA:
 		return __bizda_to_ymcw(that.bizda, __get_bizda_param(that));
 	case DT_YWD:
 		return __ywd_to_ymcw(that.ywd);
+	case DT_YD:
+		return __yd_to_ymcw(that.yd);
 	case DT_DUNK:
 	default:
 		break;
@@ -615,6 +554,8 @@ dt_conv_to_bizda(struct dt_d_s that)
 		break;
 	case DT_DAISY:
 		break;
+	case DT_YD:
+		break;
 	case DT_DUNK:
 	default:
 		break;
@@ -633,10 +574,22 @@ dt_conv_to_ywd(struct dt_d_s this)
 		return __ymd_to_ywd(this.ymd);
 	case DT_YMCW:
 		return __ymcw_to_ywd(this.ymcw);
+	case DT_JDN:
+		this.daisy = __jdn_to_daisy(this.jdn);
+		goto daisy;
+	case DT_LDN:
+		this.daisy = __ldn_to_daisy(this.ldn);
+		goto daisy;
+	case DT_MDN:
+		this.daisy = __mdn_to_daisy(this.mdn);
+		goto daisy;
 	case DT_DAISY:
+	daisy:
 		return __daisy_to_ywd(this.daisy);
 	case DT_BIZDA:
 		return __bizda_to_ywd(this.bizda, __get_bizda_param(this));
+	case DT_YD:
+		return __yd_to_ywd(this.yd);
 	case DT_DUNK:
 	default:
 		break;
@@ -653,7 +606,17 @@ dt_conv_to_yd(struct dt_d_s this)
 		return this.yd;
 	case DT_YMD:
 		return __ymd_to_yd(this.ymd);
+	case DT_JDN:
+		this.daisy = __jdn_to_daisy(this.jdn);
+		goto daisy;
+	case DT_LDN:
+		this.daisy = __ldn_to_daisy(this.ldn);
+		goto daisy;
+	case DT_MDN:
+		this.daisy = __mdn_to_daisy(this.mdn);
+		goto daisy;
 	case DT_DAISY:
+	daisy:
 		return __daisy_to_yd(this.daisy);
 	case DT_YMCW:
 		return __ymcw_to_yd(this.ymcw);
@@ -685,33 +648,42 @@ dt_conv_to_yd(struct dt_d_s this)
 #include "daisy.c"
 #undef ASPECT_DIFF
 
+#define ASPECT_CMP
+#include "yd.c"
+#include "ymd.c"
+#include "ymcw.c"
+#include "ywd.c"
+#include "bizda.c"
+#include "daisy.c"
+#undef ASPECT_CMP
+
 
 /* guessing parsers */
-#include "token.c"
-#include "strops.c"
+#include "strops.h"
+#include "token.h"
 #include "date-core-strpf.c"
 #if !defined SKIP_LEAP_ARITH
 /* we assume this file is in the dist, it's gen'd from fmt-special.gperf */
 # include "fmt-special.c"
 #endif	/* SKIP_LEAP_ARITH */
 
-static const char ymd_dflt[] = "%F";
-static const char ymcw_dflt[] = "%Y-%m-%c-%w";
-static const char ywd_dflt[] = "%rY-W%V-%u";
-static const char yd_dflt[] = "%Y-%d";
-static const char daisy_dflt[] = "%d";
-static const char bizsi_dflt[] = "%db";
-static const char bizda_dflt[] = "%Y-%m-%db";
+DEFVAR const char ymd_dflt[] = "%F";
+DEFVAR const char ymcw_dflt[] = "%Y-%m-%c-%w";
+DEFVAR const char ywd_dflt[] = "%G-W%V-%u";
+DEFVAR const char yd_dflt[] = "%Y-%D";
+DEFVAR const char daisy_dflt[] = "%d";
+DEFVAR const char bizsi_dflt[] = "%db";
+DEFVAR const char bizda_dflt[] = "%Y-%m-%db";
 
-static const char ymddur_dflt[] = "%Y-%0m-%0d";
-static const char ymcwdur_dflt[] = "%Y-%0m-%0w-%0d";
-static const char ywddur_dflt[] = "%rY-W%0w-%0d";
-static const char yddur_dflt[] = "%Y-%0d";
-static const char daisydur_dflt[] = "%d";
-static const char bizsidur_dflt[] = "%db";
-static const char bizdadur_dflt[] = "%Y-%0m-%0db";
+DEFVAR const char ymddur_dflt[] = "%Y-%0m-%0d";
+DEFVAR const char ymcwdur_dflt[] = "%Y-%0m-%0w-%0d";
+DEFVAR const char ywddur_dflt[] = "%G-W%0w-%d";
+DEFVAR const char yddur_dflt[] = "%Y-%0d";
+DEFVAR const char daisydur_dflt[] = "%d";
+DEFVAR const char bizsidur_dflt[] = "%db";
+DEFVAR const char bizdadur_dflt[] = "%Y-%0m-%0db";
 
-static dt_dtyp_t
+DEFUN dt_dtyp_t
 __trans_dfmt_special(const char *fmt)
 {
 #if !defined SKIP_LEAP_ARITH
@@ -727,7 +699,7 @@ __trans_dfmt_special(const char *fmt)
 	return DT_DUNK;
 }
 
-static void
+DEFUN dt_dtyp_t
 __trans_dfmt(const char **fmt)
 {
 	if (UNLIKELY(*fmt == NULL)) {
@@ -737,7 +709,9 @@ __trans_dfmt(const char **fmt)
 		/* don't worry about it */
 		;
 	} else {
-		switch (__trans_dfmt_special(*fmt)) {
+		const dt_dtyp_t tmp = __trans_dfmt_special(*fmt);
+
+		switch (tmp) {
 		default:
 			break;
 		case DT_YMD:
@@ -762,11 +736,13 @@ __trans_dfmt(const char **fmt)
 			*fmt = bizsi_dflt;
 			break;
 		}
+
+		return tmp;
 	}
-	return;
+	return DT_DUNK;
 }
 
-static void
+DEFUN dt_durtyp_t
 __trans_ddurfmt(const char **fmt)
 {
 	if (UNLIKELY(*fmt == NULL)) {
@@ -776,47 +752,57 @@ __trans_ddurfmt(const char **fmt)
 		/* don't worry about it */
 		;
 	} else {
-		switch (__trans_dfmt_special(*fmt)) {
+		unsigned int tmp = __trans_dfmt_special(*fmt);
+
+		switch (tmp) {
 		default:
 			break;
 		case DT_YMD:
 			*fmt = ymddur_dflt;
+			tmp = DT_DURYMD;
 			break;
 		case DT_YMCW:
 			*fmt = ymcwdur_dflt;
+			tmp = DT_DURYMCW;
 			break;
 		case DT_YWD:
 			*fmt = ywddur_dflt;
+			tmp = DT_DURYWD;
 			break;
 		case DT_YD:
 			*fmt = yddur_dflt;
+			tmp = DT_DURYD;
 			break;
 		case DT_BIZDA:
 			*fmt = bizdadur_dflt;
+			tmp = DT_DURBIZDA;
 			break;
 		case DT_DAISY:
 			*fmt = daisydur_dflt;
+			tmp = DT_DURD;
 			break;
 		case DT_BIZSI:
 			*fmt = bizsidur_dflt;
+			tmp = DT_DURBD;
 			break;
 		}
+
+		return (dt_durtyp_t)tmp;
 	}
-	return;
+	return DT_DURUNK;
 }
 
 /* strpf glue */
 DEFUN struct dt_d_s
 __guess_dtyp(struct strpd_s d)
 {
-	struct dt_d_s res = dt_d_initialiser();
+	struct dt_d_s res = {DT_DUNK};
 
 	if (LIKELY(d.y > 0 && d.c <= 0 && !d.flags.c_wcnt_p && !d.flags.bizda)) {
 		/* nearly all goes to ymd */
 		res.typ = DT_YMD;
 		res.ymd.y = d.y;
 		if (LIKELY(!d.flags.d_dcnt_p)) {
-			res.ymd.m = d.m;
 #if defined WITH_FAST_ARITH
 			res.ymd.d = d.d;
 #else  /* !WITH_FAST_ARITH */
@@ -824,17 +810,28 @@ __guess_dtyp(struct strpd_s d)
 			/* check for illegal dates, like 31st of April */
 			if ((res.ymd.d = d.d) > md) {
 				res.ymd.d = md;
+				res.fix = 1U;
 			}
-		} else {
-			/* convert dcnt to m + d */
-			struct __md_s r = __yday_get_md(d.y, d.d);
-			res.ymd.m = r.m;
-			res.ymd.d = r.d;
-		}
 #endif	/* !WITH_FAST_ARITH */
+			/* month is always pertained */
+			res.ymd.m = d.m;
+		} else {
+			/* produce yd dates */
+			res.typ = DT_YD;
+			res.yd.y = d.y;
+#if defined WITH_FAST_ARITH
+			res.yd.d = d.d;
+#else  /* !WITH_FAST_ARITH */
+			with (int maxd = __get_ydays(d.y)) {
+				if (UNLIKELY((res.yd.d = d.d) > maxd)) {
+					res.yd.d = maxd;
+				}
+			}
+#endif	/* WITH_FAST_ARITH */
+		}
 	} else if (d.y > 0 && d.m <= 0 && !d.flags.bizda) {
 		res.typ = DT_YWD;
-		res.ywd = __make_ywd_c(d.y, d.c, d.w, d.flags.wk_cnt);
+		res.ywd = __make_ywd_c(d.y, d.c, (dt_dow_t)d.w, d.flags.wk_cnt);
 	} else if (d.y > 0 && !d.flags.bizda) {
 		/* its legit for d.w to be naught */
 		res.typ = DT_YMCW;
@@ -843,7 +840,7 @@ __guess_dtyp(struct strpd_s d)
 #if defined WITH_FAST_ARITH
 		res.ymcw.c = d.c;
 #else  /* !WITH_FAST_ARITH */
-		if ((res.ymcw.c = d.c) >= 5) {
+		if (UNLIKELY((res.ymcw.c = d.c) >= 5)) {
 			/* the user meant the LAST wday actually */
 			res.ymcw.c = __get_mcnt(d.y, d.m, (dt_dow_t)d.w);
 		}
@@ -856,7 +853,15 @@ __guess_dtyp(struct strpd_s d)
 		res.typ = DT_BIZDA;
 		res.bizda.y = d.y;
 		res.bizda.m = d.m;
+#if defined WITH_FAST_ARITH
 		res.bizda.bd = d.b;
+#else  /* !WITH_FAST_ARITH */
+		unsigned int bd = __get_bdays(d.y, d.m);
+		if ((res.bizda.bd = d.b) > bd) {
+			res.bizda.bd = bd;
+			res.fix = 1U;
+		}
+#endif	/* WITH_FAST_ARITH */
 	} else {
 		/* anything else is bollocks for now */
 		;
@@ -876,10 +881,10 @@ __guess_dtyp(struct strpd_s d)
 DEFUN struct dt_d_s
 dt_strpd(const char *str, const char *fmt, char **ep)
 {
-	struct dt_d_s res = dt_d_initialiser();
-	struct strpd_s d = strpd_initialiser();
+	struct dt_d_s res = {DT_DUNK};
+	struct strpd_s d = {0};
 	const char *sp = str;
-	const char *fp = fmt;
+	const char *fp;
 
 	if (UNLIKELY(fmt == NULL)) {
 		return __strpd_std(str, ep);
@@ -887,6 +892,7 @@ dt_strpd(const char *str, const char *fmt, char **ep)
 	/* translate high-level format names */
 	__trans_dfmt(&fmt);
 
+	fp = fmt;
 	while (*fp && *sp) {
 		const char *fp_sav = fp;
 		struct dt_spec_s spec = __tok_spec(fp_sav, &fp);
@@ -949,7 +955,7 @@ out:
 DEFUN size_t
 dt_strfd(char *restrict buf, size_t bsz, const char *fmt, struct dt_d_s that)
 {
-	struct strpd_s d = strpd_initialiser();
+	struct strpd_s d = {0};
 	const char *fp;
 	char *bp;
 	dt_dtyp_t tgttyp;
@@ -981,6 +987,9 @@ dt_strfd(char *restrict buf, size_t bsz, const char *fmt, struct dt_d_s that)
 		case DT_YWD:
 			fmt = ywd_dflt;
 			break;
+		case DT_YD:
+			fmt = yd_dflt;
+			break;
 		case DT_DAISY:
 			/* subject to change */
 			fmt = ymd_dflt;
@@ -1007,7 +1016,25 @@ dt_strfd(char *restrict buf, size_t bsz, const char *fmt, struct dt_d_s that)
 		d.c = that.ymcw.c;
 		d.w = that.ymcw.w;
 		break;
+	case DT_YD:
+		d.y = that.yd.y;
+		d.d = that.yd.d;
+		d.flags.d_dcnt_p = 1U;
+		break;
+	case DT_JDN:
+		that.typ = DT_DAISY;
+		that.daisy = __jdn_to_daisy(that.jdn);
+		goto daisy_prep;
+	case DT_LDN:
+		that.typ = DT_DAISY;
+		that.daisy = __ldn_to_daisy(that.ldn);
+		goto daisy_prep;
+	case DT_MDN:
+		that.typ = DT_DAISY;
+		that.daisy = __mdn_to_daisy(that.mdn);
+		goto daisy_prep;
 	case DT_DAISY:
+	daisy_prep:
 		__prep_strfd_daisy(&d, that.daisy);
 		break;
 	case DT_BIZDA:
@@ -1055,80 +1082,68 @@ dt_strfd(char *restrict buf, size_t bsz, const char *fmt, struct dt_d_s that)
 	return bp - buf;
 }
 
-DEFUN struct dt_d_s
+DEFUN struct dt_ddur_s
 dt_strpddur(const char *str, char **ep)
 {
 /* at the moment we allow only one format */
-	struct dt_d_s res = dt_d_initialiser();
-	struct strpd_s d = strpd_initialiser();
+	struct dt_ddur_s res = dt_make_ddur(DT_DURUNK, 0);
 	const char *sp = str;
-	int tmp;
+	long int tmp;
 
 	if (str == NULL) {
 		goto out;
 	}
-	/* read just one component */
-	tmp = strtol(sp, (char**)&sp, 10);
+	/* read off co-class indicator */
+	if (*sp == '/') {
+		res.cocl = 1U;
+		sp++;
+	}
+	/* read just one component, use rudi's errno trick */
+	errno = 0;
+	if ((tmp = strtol(str, (char**)&sp, 10)) == 0 && str == sp) {
+		/* didn't work aye? */
+		goto out;
+	} else if (tmp > INT_MAX || errno) {
+		errno = ERANGE;
+		goto out;
+	}
+
 	switch (*sp++) {
 	case '\0':
 		/* must have been day then */
-		d.d = tmp;
+		res.durtyp = DT_DURD;
 		sp--;
 		break;
 	case 'd':
 	case 'D':
-		d.d = tmp;
+		res.durtyp = DT_DURD;
 		break;
 	case 'y':
 	case 'Y':
-		d.y = tmp;
+		res.durtyp = DT_DURYR;
 		break;
 	case 'm':
 	case 'M':
-		d.m = tmp;
+		res.durtyp = DT_DURMO;
 		break;
 	case 'w':
 	case 'W':
-		d.w = tmp;
+		res.durtyp = DT_DURWK;
 		break;
 	case 'b':
 	case 'B':
-		d.b = tmp;
+		res.durtyp = DT_DURBD;
 		break;
 	case 'q':
 	case 'Q':
-		d.q = tmp;
+		res.durtyp = DT_DURQU;
 		break;
 	default:
 		sp = str;
 		goto out;
 	}
-	/* assess */
-	if (d.b && (d.y || d.m >= 1)) {
-		res.typ = DT_BIZDA;
-		res.bizda.y = d.y;
-		res.bizda.m = d.q * 3 + d.m;
-		res.bizda.bd = d.b + d.w * DUWW_BDAYS_P_WEEK;
-	} else if (LIKELY(d.y || d.m >= 1)) {
-		res.typ = DT_YMD;
-		res.ymd.y = d.y;
-		res.ymd.m = d.q * 3 + d.m;
-		res.ymd.d = d.d + d.w * GREG_DAYS_P_WEEK;
-	} else if (d.d) {
-		res.typ = DT_DAISY;
-		res.daisydur = d.w * GREG_DAYS_P_WEEK + d.d;
-	} else if (d.b) {
-		res.typ = DT_BIZSI;
-		res.bizsidur = d.w * DUWW_BDAYS_P_WEEK + d.b;
-	} else {
-		/* we leave out YMCW diffs simply because YMD diffs
-		 * cover them better
-		 * anything that doesn't fit shall be mapped to MD durs
-		 * using the definitions of MONTHS_P_YEAR and DAYS_P_WEEK */
-		res.typ = DT_MD;
-		res.md.d = d.w * GREG_DAYS_P_WEEK + d.d;
-		res.md.m = d.y * GREG_MONTHS_P_YEAR + d.m;
-	}
+	/* no further checks on TMP */
+	res.dv = tmp;
 out:
 	if (ep != NULL) {
 		*ep = (char*)sp;
@@ -1137,18 +1152,18 @@ out:
 }
 
 DEFUN size_t
-dt_strfddur(char *restrict buf, size_t bsz, const char *fmt, struct dt_d_s that)
+dt_strfddur(char *restrict buf, size_t bsz, const char *fmt, struct dt_ddur_s that)
 {
-	struct strpd_s d = strpd_initialiser();
+	struct strpd_s d = {0};
 	const char *fp;
 	char *bp;
 
-	if (UNLIKELY(buf == NULL || bsz == 0 || !that.dur)) {
+	if (UNLIKELY(buf == NULL || bsz == 0)) {
 		return 0;
 	}
 
-	switch (that.typ) {
-	case DT_YMD:
+	switch (that.durtyp) {
+	case DT_DURYMD:
 		d.y = that.ymd.y;
 		d.m = that.ymd.m;
 		d.d = that.ymd.d;
@@ -1156,7 +1171,7 @@ dt_strfddur(char *restrict buf, size_t bsz, const char *fmt, struct dt_d_s that)
 			fmt = ymddur_dflt;
 		}
 		break;
-	case DT_YMCW:
+	case DT_DURYMCW:
 		d.y = that.ymcw.y;
 		d.m = that.ymcw.m;
 		d.c = that.ymcw.c;
@@ -1165,7 +1180,7 @@ dt_strfddur(char *restrict buf, size_t bsz, const char *fmt, struct dt_d_s that)
 			fmt = ymcwdur_dflt;
 		}
 		break;
-	case DT_YWD:
+	case DT_DURYWD:
 		d.y = that.ywd.y;
 		d.c = that.ywd.c;
 		d.d = that.ywd.w;
@@ -1173,41 +1188,17 @@ dt_strfddur(char *restrict buf, size_t bsz, const char *fmt, struct dt_d_s that)
 			fmt = ywddur_dflt;
 		}
 		break;
-	case DT_YD:
+	case DT_DURYD:
 		d.y = that.yd.y;
 		d.d = that.yd.d;
 		if (fmt == NULL) {
 			fmt = yddur_dflt;
 		}
 		break;
-	case DT_DAISY:
-		if (that.daisydur >= 0) {
-			d.d = that.daisydur;
-		} else {
-			d.d = -that.daisydur;
-			/* make sure the neg bit doesn't bite us */
-			that.neg = 1;
-		}
-		if (fmt == NULL) {
-			/* subject to change */
-			fmt = daisydur_dflt;
-		}
-		break;
-	case DT_BIZSI:
-		if (that.bizsidur >= 0) {
-			d.d = that.bizsidur;
-		} else {
-			d.d = -that.bizsidur;
-			/* make sure the neg bit doesn't bite us */
-			that.neg = 1;
-		}
-		if (fmt == NULL) {
-			/* subject to change */
-			fmt = bizsidur_dflt;
-		}
-		break;
-	case DT_BIZDA: {
-		dt_bizda_param_t bparam = __get_bizda_param(that);
+	case DT_DURBIZDA:;
+		dt_bizda_param_t bparam;
+
+		bparam.bs = that.param;
 		d.y = that.bizda.y;
 		d.m = that.bizda.m;
 		d.b = that.bizda.bd;
@@ -1221,13 +1212,41 @@ dt_strfddur(char *restrict buf, size_t bsz, const char *fmt, struct dt_d_s that)
 			fmt = bizdadur_dflt;
 		}
 		break;
-	}
-	case DT_MD:
-		d.y = __uidiv(that.md.m, GREG_MONTHS_P_YEAR);
-		d.m = __uimod(that.md.m, GREG_MONTHS_P_YEAR);
-		d.w = __uidiv(that.md.d, GREG_DAYS_P_WEEK);
-		d.d = __uimod(that.md.d, GREG_DAYS_P_WEEK);
+
+	case DT_DURD:
+	case DT_DURBD:
+	case DT_DURWK:
+	case DT_DURMO:
+	case DT_DURQU:
+	case DT_DURYR:
+		if (that.dv >= 0) {
+			/* make sure the neg bit doesn't bite us */
+			that.dv = -that.dv;
+			that.neg = 1U;
+		}
+
+		switch (that.durtyp) {
+		case DT_DURD:
+			d.d = that.dv;
+			break;
+		case DT_DURBD:
+			d.b = that.dv;
+			break;
+		case DT_DURWK:
+			d.d = that.dv * GREG_DAYS_P_WEEK;
+			break;
+		case DT_DURMO:
+			d.m = that.dv;
+			break;
+		case DT_DURQU:
+			d.m = that.dv * 3U;
+			break;
+		case DT_DURYR:
+			d.y = that.dv;
+			break;
+		}
 		break;
+
 	default:
 	case DT_DUNK:
 		bp = buf;
@@ -1285,7 +1304,9 @@ dt_date(dt_dtyp_t outtyp)
 
 	switch ((res.typ = outtyp)) {
 	case DT_YMD:
-	case DT_YMCW: {
+	case DT_YMCW:
+	case DT_YWD:
+	case DT_YD: {
 		struct tm tm;
 		ffff_gmtime(&tm, t);
 		switch (res.typ) {
@@ -1313,18 +1334,32 @@ dt_date(dt_dtyp_t outtyp)
 			res.ymcw.w = tm.tm_wday;
 			break;
 		}
+		case DT_YD:
+			res.yd.y = tm.tm_year;
+			res.yd.d = tm.tm_yday;
+			break;
+		case DT_YWD:
+			/* use ordinary conversion to ywd */
+			res.typ = DT_YMD;
+			res.ymd.y = tm.tm_year;
+			res.ymd.m = tm.tm_mon;
+			res.ymd.d = tm.tm_mday;
+			res = dt_dconv(DT_YWD, res);
+			break;
+
 		default:
 			break;
 		}
 		break;
 	}
+
 	case DT_DAISY:
 		/* time_t's base is 1970-01-01, which is daisy 19359 */
 		res.daisy = t / 86400 + 19359;
 		break;
+
+		/* the rest doesn't make sense I say */
 	default:
-	case DT_MD:
-		/* doesn't make sense */
 	case DT_DUNK:
 		res.u = 0;
 	}
@@ -1334,7 +1369,10 @@ dt_date(dt_dtyp_t outtyp)
 DEFUN struct dt_d_s
 dt_dconv(dt_dtyp_t tgttyp, struct dt_d_s d)
 {
-	struct dt_d_s res = dt_d_initialiser();
+	struct dt_d_s res = {DT_DUNK};
+
+	/* fix up before conversion */
+	d = dt_dfixup(d);
 
 	switch ((res.typ = tgttyp)) {
 	case DT_YMD:
@@ -1345,7 +1383,8 @@ dt_dconv(dt_dtyp_t tgttyp, struct dt_d_s d)
 		break;
 	case DT_DAISY:
 	case DT_JDN:
-	case DT_LDN: {
+	case DT_LDN:
+	case DT_MDN: {
 		dt_daisy_t tmp = dt_conv_to_daisy(d);
 		switch (tgttyp) {
 		case DT_DAISY:
@@ -1357,6 +1396,9 @@ dt_dconv(dt_dtyp_t tgttyp, struct dt_d_s d)
 		case DT_JDN:
 			res.jdn = __daisy_to_jdn(tmp);
 			break;
+		case DT_MDN:
+			res.mdn = __daisy_to_mdn(tmp);
+			break;
 		default:
 			/* nice one gcc */
 			;
@@ -1370,6 +1412,9 @@ dt_dconv(dt_dtyp_t tgttyp, struct dt_d_s d)
 	case DT_YWD:
 		res.ywd = dt_conv_to_ywd(d);
 		break;
+	case DT_YD:
+		res.yd = dt_conv_to_yd(d);
+		break;
 	case DT_DUNK:
 	default:
 		res.typ = DT_DUNK;
@@ -1382,9 +1427,42 @@ DEFUN struct dt_d_s
 dt_dadd_d(struct dt_d_s d, int n)
 {
 /* add N (gregorian) days to D */
+	if (UNLIKELY(!n)) {
+		/* cacn't use short-cut return here, it'd upset the IPO/LTO */
+		goto out;
+	}
 	switch (d.typ) {
+	case DT_JDN:
+		d.daisy = __jdn_to_daisy(d.jdn);
+		goto daisy_add_d;
+
+	case DT_LDN:
+		d.daisy = __ldn_to_daisy(d.ldn);
+		goto daisy_add_d;
+
+	case DT_MDN:
+		d.daisy = __mdn_to_daisy(d.mdn);
+		goto daisy_add_d;
+
 	case DT_DAISY:
+	daisy_add_d:
 		d.daisy = __daisy_add_d(d.daisy, n);
+
+		/* transform back (maybe) */
+		switch (d.typ) {
+		case DT_DAISY:
+		default:
+			break;
+		case DT_LDN:
+			d.ldn = __daisy_to_ldn(d.daisy);
+			break;
+		case DT_JDN:
+			d.jdn = __daisy_to_jdn(d.daisy);
+			break;
+		case DT_MDN:
+			d.mdn = __daisy_to_mdn(d.daisy);
+			break;
+		}
 		break;
 
 	case DT_YMD:
@@ -1403,12 +1481,17 @@ dt_dadd_d(struct dt_d_s d, int n)
 		d.ywd = __ywd_add_d(d.ywd, n);
 		break;
 
+	case DT_YD:
+		d.yd = __yd_add_d(d.yd, n);
+		break;
+
 	case DT_DUNK:
 	default:
 		d.typ = DT_DUNK;
 		d.u = 0;
 		break;
 	}
+out:
 	return d;
 }
 
@@ -1416,9 +1499,42 @@ DEFUN struct dt_d_s
 dt_dadd_b(struct dt_d_s d, int n)
 {
 /* add N business days to D */
+	if (UNLIKELY(!n)) {
+		/* cacn't use short-cut return here, it'd upset the IPO/LTO */
+		goto out;
+	}
 	switch (d.typ) {
+	case DT_JDN:
+		d.daisy = __jdn_to_daisy(d.jdn);
+		goto daisy_add_b;
+
+	case DT_LDN:
+		d.daisy = __ldn_to_daisy(d.ldn);
+		goto daisy_add_b;
+
+	case DT_MDN:
+		d.daisy = __mdn_to_daisy(d.mdn);
+		goto daisy_add_b;
+
 	case DT_DAISY:
+	daisy_add_b:
 		d.daisy = __daisy_add_b(d.daisy, n);
+
+		/* transform back (maybe) */
+		switch (d.typ) {
+		case DT_DAISY:
+		default:
+			break;
+		case DT_LDN:
+			d.ldn = __daisy_to_ldn(d.daisy);
+			break;
+		case DT_JDN:
+			d.jdn = __daisy_to_jdn(d.daisy);
+			break;
+		case DT_MDN:
+			d.mdn = __daisy_to_mdn(d.daisy);
+			break;
+		}
 		break;
 
 	case DT_YMD:
@@ -1437,12 +1553,17 @@ dt_dadd_b(struct dt_d_s d, int n)
 		d.ywd = __ywd_add_b(d.ywd, n);
 		break;
 
+	case DT_YD:
+		d.yd = __yd_add_b(d.yd, n);
+		break;
+
 	case DT_DUNK:
 	default:
 		d.typ = DT_DUNK;
 		d.u = 0;
 		break;
 	}
+out:
 	return d;
 }
 
@@ -1450,9 +1571,42 @@ DEFUN struct dt_d_s
 dt_dadd_w(struct dt_d_s d, int n)
 {
 /* add N weeks to D */
+	if (UNLIKELY(!n)) {
+		/* cacn't use short-cut return here, it'd upset the IPO/LTO */
+		goto out;
+	}
 	switch (d.typ) {
+	case DT_JDN:
+		d.daisy = __jdn_to_daisy(d.jdn);
+		goto daisy_add_w;
+
+	case DT_LDN:
+		d.daisy = __ldn_to_daisy(d.ldn);
+		goto daisy_add_w;
+
+	case DT_MDN:
+		d.daisy = __mdn_to_daisy(d.mdn);
+		goto daisy_add_w;
+
 	case DT_DAISY:
+	daisy_add_w:
 		d.daisy = __daisy_add_w(d.daisy, n);
+
+		/* transform back (maybe) */
+		switch (d.typ) {
+		case DT_DAISY:
+		default:
+			break;
+		case DT_LDN:
+			d.ldn = __daisy_to_ldn(d.daisy);
+			break;
+		case DT_JDN:
+			d.jdn = __daisy_to_jdn(d.daisy);
+			break;
+		case DT_MDN:
+			d.mdn = __daisy_to_mdn(d.daisy);
+			break;
+		}
 		break;
 
 	case DT_YMD:
@@ -1471,12 +1625,17 @@ dt_dadd_w(struct dt_d_s d, int n)
 		d.ywd = __ywd_add_w(d.ywd, n);
 		break;
 
+	case DT_YD:
+		d.yd = __yd_add_w(d.yd, n);
+		break;
+
 	case DT_DUNK:
 	default:
 		d.typ = DT_DUNK;
 		d.u = 0;
 		break;
 	}
+out:
 	return d;
 }
 
@@ -1484,7 +1643,13 @@ DEFUN struct dt_d_s
 dt_dadd_m(struct dt_d_s d, int n)
 {
 /* add N months to D */
+	if (UNLIKELY(!n)) {
+		goto out;
+	}
 	switch (d.typ) {
+	case DT_LDN:
+	case DT_JDN:
+	case DT_MDN:
 	case DT_DAISY:
 		/* daisy objects have no notion of months */
 		break;
@@ -1505,12 +1670,17 @@ dt_dadd_m(struct dt_d_s d, int n)
 		/* ywd have no notion of months */
 		break;
 
+	case DT_YD:
+		/* yd have no notion of months */
+		break;
+
 	case DT_DUNK:
 	default:
 		d.typ = DT_DUNK;
 		d.u = 0;
 		break;
 	}
+out:
 	return d;
 }
 
@@ -1518,7 +1688,14 @@ DEFUN struct dt_d_s
 dt_dadd_y(struct dt_d_s d, int n)
 {
 /* add N years to D */
+	if (UNLIKELY(!n)) {
+		/* cacn't use short-cut return here, it'd upset the IPO/LTO */
+		goto out;
+	}
 	switch (d.typ) {
+	case DT_LDN:
+	case DT_JDN:
+	case DT_MDN:
 	case DT_DAISY:
 		/* daisy objects have no notion of years */
 		break;
@@ -1539,50 +1716,69 @@ dt_dadd_y(struct dt_d_s d, int n)
 		d.ywd = __ywd_add_y(d.ywd, n);
 		break;
 
+	case DT_YD:
+		d.yd = __yd_add_y(d.yd, n);
+		break;
+
 	case DT_DUNK:
 	default:
 		d.typ = DT_DUNK;
 		d.u = 0;
 		break;
 	}
+out:
 	return d;
 }
 
 DEFUN struct dt_d_s
-dt_dadd(struct dt_d_s d, struct dt_d_s dur)
+dt_dadd(struct dt_d_s d, struct dt_ddur_s dur)
 {
-	struct strpdi_s durcch = strpdi_initialiser();
-
-	__fill_strpdi(&durcch, dur);
-
-	if (durcch.d) {
-		d = dt_dadd_d(d, durcch.d);
-	} else if (durcch.b) {
-		d = dt_dadd_b(d, durcch.b);
+	if (UNLIKELY(!dur.dv)) {
+		/* cacn't use short-cut return here, it'd upset the IPO/LTO */
+		goto out;
 	}
-	if (durcch.w) {
-		d = dt_dadd_w(d, durcch.w);
-	}
-	if (durcch.m) {
-		d = dt_dadd_m(d, durcch.m);
-	}
-	if (durcch.y) {
-		d = dt_dadd_y(d, durcch.y);
+	switch (dur.durtyp) {
+	case DT_DURD:
+		d = dt_dadd_d(d, dur.dv);
+		break;
+	case DT_DURBD:
+		d = dt_dadd_b(d, dur.dv);
+		break;
+	case DT_DURWK:
+		d = dt_dadd_w(d, dur.dv);
+		break;
+	case DT_DURQU:
+		/* just use the month adder */
+		dur.dv *= 3U;
+	case DT_DURMO:
+		d = dt_dadd_m(d, dur.dv);
+		break;
+	case DT_DURYR:
+		d = dt_dadd_y(d, dur.dv);
+		break;
+	default:
+	case DT_DURUNK:
+		/* huh? */
+		break;
 	}
+out:
 	return d;
 }
 
-DEFUN struct dt_d_s
-dt_neg_dur(struct dt_d_s dur)
+DEFUN struct dt_ddur_s
+dt_neg_dur(struct dt_ddur_s dur)
 {
 	dur.neg = (uint16_t)(~dur.neg & 0x01);
-	switch (dur.typ) {
-	case DT_DAISY:
-		dur.daisydur = -dur.daisydur;
-		break;
-	case DT_BIZSI:
-		dur.bizsidur = -dur.bizsidur;
+	switch (dur.durtyp) {
+	case DT_DURD:
+	case DT_DURBD:
+	case DT_DURWK:
+	case DT_DURMO:
+	case DT_DURQU:
+	case DT_DURYR:
+		dur.dv = -dur.dv;
 		break;
+
 	default:
 		break;
 	}
@@ -1590,109 +1786,109 @@ dt_neg_dur(struct dt_d_s dur)
 }
 
 DEFUN int
-dt_dur_neg_p(struct dt_d_s dur)
+dt_dur_neg_p(struct dt_ddur_s dur)
 {
-	switch (dur.typ) {
-	case DT_DAISY:
-		return dur.daisydur < 0;
-	case DT_BIZSI:
-		return dur.bizsidur < 0;
+	switch (dur.durtyp) {
+	case DT_DURD:
+	case DT_DURBD:
+	case DT_DURWK:
+	case DT_DURMO:
+	case DT_DURQU:
+	case DT_DURYR:
+		return dur.dv < 0;
 	default:
-		return dur.neg;
+		break;
 	}
+	return dur.neg;
 }
 
-DEFUN struct dt_d_s
-dt_ddiff(dt_dtyp_t tgttyp, struct dt_d_s d1, struct dt_d_s d2)
+DEFUN struct dt_ddur_s
+dt_ddiff(dt_durtyp_t tgttyp, struct dt_d_s d1, struct dt_d_s d2, int carry)
 {
-	struct dt_d_s res = {.typ = DT_DUNK};
-	dt_dtyp_t tmptyp = tgttyp;
+/* carry of 0 doesn't change anything
+ * carry <0 adds -1 to d2 iff d1 <= d2
+ * carry >0 adds 1 to d2 iff d1 >= d2 */
+	struct dt_ddur_s res = dt_make_ddur(DT_DURUNK, 0);
 
-	if (tgttyp == DT_MD) {
-		if (d1.typ == DT_YMD || d2.typ == DT_YMD) {
-			tmptyp = DT_YMD;
-		} else if (d1.typ == DT_YMCW || d2.typ == DT_YMCW) {
-			tmptyp = DT_YMCW;
-		} else {
-			tmptyp = DT_DAISY;
-		}
-	}
-
-	switch (tmptyp) {
-	case DT_BIZSI:
-	case DT_DAISY: {
+	switch (tgttyp) {
+	case DT_DURD:
+	case DT_DURBD: {
 		dt_daisy_t tmp1 = dt_conv_to_daisy(d1);
 		dt_daisy_t tmp2 = dt_conv_to_daisy(d2);
+
 		res = __daisy_diff(tmp1, tmp2);
 
 		/* fix up result in case it's bizsi, i.e. kick weekends */
-		if (tgttyp == DT_BIZSI) {
+		if (tgttyp == DT_DURBD) {
 			dt_dow_t wdb = __daisy_get_wday(tmp2);
-			res.bizsidur = __get_nbdays(res.daisydur, wdb);
+			res.dv = __get_nbdays(res.dv, wdb);
 		}
 		break;
 	}
-	case DT_YMD: {
+	case DT_DURYMD: {
 		dt_ymd_t tmp1 = dt_conv_to_ymd(d1);
 		dt_ymd_t tmp2 = dt_conv_to_ymd(d2);
+		int fix = __sgn(carry) == -__sgn(tmp2.u - tmp1.u);
+
+		if (UNLIKELY(fix)) {
+			/* add -1 iff tmp2 > tmp1 && carry < 0
+			 * add 1 iff tmp2 < tmp1 && carry > 0, i.e.
+			 * add sgn(carry) iff sgn(tmp2-tmp1) = -sgn(carry) */
+			tmp2 = __ymd_add_d(tmp2, __sgn(carry));
+		}
 		res = __ymd_diff(tmp1, tmp2);
+		res.fix = fix;
 		break;
 	}
-	case DT_YMCW: {
+	case DT_DURYMCW: {
 		dt_ymcw_t tmp1 = dt_conv_to_ymcw(d1);
 		dt_ymcw_t tmp2 = dt_conv_to_ymcw(d2);
+		int fix = carry && __sgn(carry) == -__ymcw_cmp(tmp1, tmp2);
+
+		if (UNLIKELY(fix)) {
+			/* add -1 iff tmp2 > tmp1 && carry < 0
+			 * add 1 iff tmp2 < tmp1 && carry > 0, i.e.
+			 * add sgn(carry) iff sgn(tmp2-tmp1) = -sgn(carry) */
+			tmp2 = __ymcw_add_d(tmp2, __sgn(carry));
+		}
 		res = __ymcw_diff(tmp1, tmp2);
+		res.fix = fix;
 		break;
 	}
-	case DT_YD: {
+	case DT_DURYD: {
 		dt_yd_t tmp1 = dt_conv_to_yd(d1);
 		dt_yd_t tmp2 = dt_conv_to_yd(d2);
+		int fix = __sgn(carry) == -__sgn(tmp2.u - tmp1.u);
+
+		if (UNLIKELY(fix)) {
+			/* add -1 iff tmp2 > tmp1 && carry < 0
+			 * add 1 iff tmp2 < tmp1 && carry > 0, i.e.
+			 * add sgn(carry) iff sgn(tmp2-tmp1) = -sgn(carry) */
+			tmp2 = __yd_add_d(tmp2, __sgn(carry));
+		}
 		res = __yd_diff(tmp1, tmp2);
+		res.fix = fix;
 		break;
 	}
-	case DT_YWD: {
+	case DT_DURYWD: {
 		dt_ywd_t tmp1 = dt_conv_to_ywd(d1);
 		dt_ywd_t tmp2 = dt_conv_to_ywd(d2);
-		res = __ywd_diff(tmp1, tmp2);
-		break;
-	}
-	case DT_BIZDA:
-	case DT_DUNK:
-	default:
-		res.typ = DT_DUNK;
-		res.u = 0;
-		/* @fallthrough@ */
-	case DT_MD:
-		/* md is handled later */
-		break;
-	}
-	/* check if we had DT_MD as tgttyp */
-	if (tgttyp == DT_MD) {
-		/* convert res back to DT_MD */
-		struct dt_d_s tmp = {.typ = DT_MD};
+		int fix = __sgn(carry) == -__sgn(tmp2.u - tmp1.u);
 
-		switch (tmptyp) {
-		case DT_YMD:
-			tmp.md.m = res.ymd.y * GREG_MONTHS_P_YEAR + res.ymd.m;
-			tmp.md.d = res.ymd.d;
-			break;
-		case DT_YMCW:
-			tmp.md.m = res.ymcw.y * GREG_MONTHS_P_YEAR + res.ymcw.m;
-			tmp.md.d = res.ymcw.w * GREG_DAYS_P_WEEK + res.ymcw.c;
-			break;
-		case DT_DAISY:
-			tmp.md.m = 0;
-			tmp.md.d = res.daisy;
-			break;
-
-		case DT_BIZDA:
-		case DT_BIZSI:
-		case DT_MD:
-		case DT_DUNK:
-		default:
-			break;
+		if (UNLIKELY(fix)) {
+			/* add -1 iff tmp2 > tmp1 && carry < 0
+			 * add 1 iff tmp2 < tmp1 && carry > 0, i.e.
+			 * add sgn(carry) iff sgn(tmp2-tmp1) = -sgn(carry) */
+			tmp2 = __ywd_add_d(tmp2, __sgn(carry));
 		}
-		res = tmp;
+		res = __ywd_diff(tmp1, tmp2);
+		res.fix = fix;
+		break;
+	}
+	case DT_DURBIZDA:
+	case DT_DURUNK:
+	default:
+		break;
 	}
 	return res;
 }
@@ -1712,6 +1908,8 @@ dt_dcmp(struct dt_d_s d1, struct dt_d_s d2)
 	case DT_YMD:
 	case DT_DAISY:
 	case DT_BIZDA:
+	case DT_YWD:
+	case DT_YD:
 		/* use arithmetic comparison */
 		if (d1.u == d2.u) {
 			return 0;
@@ -1730,7 +1928,50 @@ dt_dcmp(struct dt_d_s d1, struct dt_d_s d2)
 DEFUN int
 dt_d_in_range_p(struct dt_d_s d, struct dt_d_s d1, struct dt_d_s d2)
 {
-	return dt_dcmp(d, d1) >= 0 && dt_dcmp(d, d2) <= 0;
+/* use the following multiplication table
+ *
+ * |d,d2|v |d,d1|>  -2 -1  0  1
+ *      -2          -1 -1 -1 -1
+ *      -1          -1  0  1  1
+ *       0          -1  0  1  1
+ *       1          -1  0  0  0
+ *
+ * encoded in a 32bit uint */
+	static const uint32_t m = 0b10010111100101111010101111111111U;
+	const unsigned int i = (dt_dcmp(d, d1) + 2) & 0b11U;
+	const unsigned int j = (dt_dcmp(d, d2) + 2) & 0b11U;
+	return 2 - ((m >> (i * 8U + j * 2U)) & 0b11U);
+}
+
+DEFUN __attribute__((const)) struct dt_d_s
+dt_dfixup(struct dt_d_s d)
+{
+	switch (d.typ) {
+	case DT_YMD:
+		d.ymd = __ymd_fixup(d.ymd);
+		break;
+	case DT_YMCW:
+		d.ymcw = __ymcw_fixup(d.ymcw);
+		break;
+	case DT_YWD:
+		d.ywd = __ywd_fixup(d.ywd);
+		break;
+	case DT_YD:
+		d.yd = __yd_fixup(d.yd);
+		break;
+	case DT_BIZDA:
+		d.bizda = __bizda_fixup(d.bizda);
+		break;
+
+		/* these can't be buggered */
+	case DT_DAISY:
+	case DT_JDN:
+	case DT_LDN:
+	case DT_MDN:
+	default:
+		break;
+	}
+	return d;
 }
 
 #endif	/* INCLUDED_date_core_c_ */
