@@ -1,6 +1,6 @@
 /*** ltrcc.c -- leapseconds materialiser
  *
- * Copyright (C) 2012 Sebastian Freundt
+ * Copyright (C) 2012-2020 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -45,14 +45,22 @@
 #endif	/* HAVE_SYS_STDINT_H */
 #include <stdio.h>
 #include <stdarg.h>
+#include <string.h>
 
 #include "leaps.h"
 #include "date-core.h"
-#include "dt-core.h"
+#include "time-core.h"
 #include "nifty.h"
 
 #include "version.c"
 
+
+static __attribute__((const)) unsigned long int
+ntp_to_unix_epoch(unsigned long int x)
+{
+	return x - 25567U * 86400U;
+}
+
 
 #define PROLOGUE	(-1UL)
 #define EPILOGUE	(0UL)
@@ -60,20 +68,20 @@
 static int
 pr_line_corr(const char *line, size_t llen, va_list UNUSED(vap))
 {
-	static int32_t corr = 0;
-	char *ep;
+	static unsigned long int cor;
+	char *sp, *ep;
 
 	if (llen == PROLOGUE) {
 		/* prologue */
-		corr = 0;
 		fprintf(stdout, "\
 const int32_t %s[] = {\n\
-	0,\n", line);
+	10,\n", line);
 		return 0;
 	} else if (llen == EPILOGUE) {
 		fprintf(stdout, "\
-	%i\n\
-};\n", corr);
+	%ld\n\
+};\n", cor);
+		cor = 0;
 		return 0;
 	} else if (line == NULL) {
 		/* grrrr */
@@ -86,36 +94,24 @@ const int32_t %s[] = {\n\
 		return 0;
 	}
 	/* otherwise process */
-	if ((dt_strpd(line, "Leap\t%Y\t%b\t%d\t", &ep), ep) == NULL) {
+	if ((sp = memchr(line, '\t', llen)) == NULL) {
 		return -1;
-	} else if (llen - (ep - line) < 9) {
-		return -1;
-	} else if (ep[8] != '\t') {
+	} else if ((ep = NULL, cor = strtoul(++sp, &ep, 10), ep == NULL || cor == ULONG_MAX)) {
 		return -1;
 	}
 
-	switch (ep[9]) {
-	case '+':
-		++corr;
-		break;
-	case '-':
-		--corr;
-		break;
-	default:
-		/* still buggered */
-		return -1;
-	}
 	/* output the correction then */
-	fprintf(stdout, "\t%i,\n", corr);
+	fprintf(stdout, "\t%ld,\n", cor);
 	return 0;
 }
 
 static int
 pr_line_d(const char *line, size_t llen, va_list vap)
 {
-	static int32_t corr = 0;
+	static unsigned long int cor;
 	struct dt_d_s d;
 	dt_dtyp_t typ;
+	unsigned long int val;
 	int colp;
 	char *ep;
 
@@ -125,7 +121,6 @@ pr_line_d(const char *line, size_t llen, va_list vap)
 
 	if (llen == PROLOGUE) {
 		/* prologue */
-		corr = 0;
 		if (!colp) {
 			fprintf(stdout, "\
 const struct zleap_s %s[] = {\n\
@@ -140,13 +135,14 @@ const uint32_t %s[] = {\n\
 		/* epilogue */
 		if (!colp) {
 			fprintf(stdout, "\
-	{UINT32_MAX, %i}\n\
-};\n", corr);
+	{UINT32_MAX, %li}\n\
+};\n", cor);
 		} else {
 			fputs("\
 	UINT32_MAX\n\
 };\n", stdout);
 		}
+		cor = 0;
 		return 0;
 	} else if (line == NULL) {
 		/* something's fucked */
@@ -159,32 +155,21 @@ const uint32_t %s[] = {\n\
 		return 0;
 	}
 	/* otherwise process */
-	if ((d = dt_strpd(line, "Leap\t%Y\t%b\t%d\t", &ep), ep) == NULL) {
-		return -1;
-	} else if (llen - (ep - line) < 9) {
-		return -1;
-	} else if (ep[8] != '\t') {
+	if ((ep = NULL, val = strtoul(line, &ep, 10), ep == NULL || val == ULONG_MAX)) {
 		return -1;
 	}
 
-	/* convert to target type */
+	/* fix up and convert to target type */
+	d = (struct dt_d_s){DT_DAISY, .daisy = val / 86400 + 109207};
 	d = dt_dconv(typ, d);
 
 	if (!colp) {
-		switch (ep[9]) {
-		case '+':
-			++corr;
-			break;
-		case '-':
-			--corr;
-			break;
-		default:
-			/* still buggered */
+		if ((cor = strtoul(ep, &ep, 10), ep == NULL || val == ULONG_MAX)) {
 			return -1;
 		}
 		/* just output the line then */
-		fprintf(stdout, "\t{0x%xU/* %i */, %i},\n",
-			d.u, (int32_t)d.u, corr);
+		fprintf(stdout, "\t{0x%xU/* %i */, %li},\n",
+			d.u, (int32_t)d.u, cor);
 	} else {
 		fprintf(stdout, "\t0x%xU/* %i */,\n", d.u, (int32_t)d.u);
 	}
@@ -194,12 +179,11 @@ const uint32_t %s[] = {\n\
 static int
 pr_line_dt(const char *line, size_t llen, va_list vap)
 {
-	static int32_t corr = 0;
-	struct dt_dt_s d;
+	static unsigned long int cor;
 	dt_dtyp_t __attribute__((unused)) typ;
+	unsigned long int val;
 	int colp;
 	char *ep;
-	dt_ssexy_t val;
 
 	/* extract type from inner list */
 	typ = va_arg(vap, dt_dtyp_t);
@@ -207,11 +191,10 @@ pr_line_dt(const char *line, size_t llen, va_list vap)
 
 	if (llen == PROLOGUE) {
 		/* prologue */
-		corr = 0;
 		if (!colp) {
 			fprintf(stdout, "\
 const struct zleap_s %s[] = {\n\
-	{INT32_MIN, 0},\n", line);
+	{INT32_MIN, 10},\n", line);
 		} else {
 			fprintf(stdout, "\
 const int32_t %s[] = {\n\
@@ -222,13 +205,14 @@ const int32_t %s[] = {\n\
 		/* epilogue */
 		if (!colp) {
 			fprintf(stdout, "\
-	{INT32_MAX, %i}\n\
-};\n", corr);
+	{INT32_MAX, %li}\n\
+};\n", cor);
 		} else {
 			fputs("\
 	INT32_MAX\n\
 };\n", stdout);
 		}
+		cor = 0;
 		return 0;
 	} else if (line == NULL) {
 		/* buggre */
@@ -241,37 +225,24 @@ const int32_t %s[] = {\n\
 		return 0;
 	}
 	/* otherwise process */
-	if ((d = dt_strpdt(
-		     line, "Leap\t%Y\t%b\t%d\t%H:%M:%S", &ep), ep) == NULL) {
-		return -1;
-	} else if (llen - (ep - line) < 1) {
-		return -1;
-	} else if (ep[0] != '\t') {
+	if ((ep = NULL, val = strtoul(line, &ep, 10), ep == NULL || val == ULONG_MAX)) {
 		return -1;
 	}
 
 	/* fix up and convert to target type */
-	d.t.hms.s--;
-	val = dt_to_unix_epoch(d);
+	val--;
+	val = ntp_to_unix_epoch(val);
 
 	if (!colp) {
-		switch (ep[1]) {
-		case '+':
-			++corr;
-			break;
-		case '-':
-			--corr;
-			break;
-		default:
-			/* still buggered */
+		if ((cor = strtoul(ep, &ep, 10), ep == NULL || cor == ULONG_MAX)) {
 			return -1;
 		}
 		/* just output the line then */
-		fprintf(stdout, "\t{0x%xU/* %li */, %i},\n",
-			(uint32_t)val, val, corr);
+		fprintf(stdout, "\t{0x%lxU/* %li */, %li},\n",
+			val, val, cor);
 	} else {
 		/* column-oriented mode */
-		fprintf(stdout, "\t0x%xU/* %li */,\n", (uint32_t)val, val);
+		fprintf(stdout, "\t0x%lxU/* %li */,\n", val, val);
 	}
 	return 0;
 }
@@ -279,16 +250,22 @@ const int32_t %s[] = {\n\
 static int
 pr_line_t(const char *line, size_t llen, va_list vap)
 {
-	struct dt_dt_s d;
+	static unsigned long int cor;
+	struct dt_t_s t = {DT_TUNK};
 	dt_dtyp_t typ;
 	int colp;
 	char *ep;
-	uint32_t val;
+	unsigned long int val;
 
 	/* extract type from inner list */
 	typ = va_arg(vap, dt_dtyp_t);
 	colp = va_arg(vap, int);
 
+	/* column-oriented mode only */
+	if (!colp) {
+		return 0;
+	}
+
 	if (llen == PROLOGUE) {
 		/* prologue */
 		fprintf(stdout, "\
@@ -300,6 +277,7 @@ const uint32_t %s[] = {\n\
 		fputs("\
 	UINT32_MAX\n\
 };\n", stdout);
+		cor = 0;
 		return 0;
 	} else if (typ != (dt_dtyp_t)DT_HMS || !colp) {
 		return 0;
@@ -314,19 +292,27 @@ const uint32_t %s[] = {\n\
 		return 0;
 	}
 	/* otherwise process */
-	if ((d = dt_strpdt(
-		     line, "Leap\t%Y\t%b\t%d\t%H:%M:%S", &ep), ep) == NULL) {
+	if ((ep = NULL, val = strtoul(line, &ep, 10), ep == NULL || val == ULONG_MAX)) {
 		return -1;
-	} else if (llen - (ep - line) < 1) {
-		return -1;
-	} else if (ep[0] != '\t') {
+	}
+	val--;
+	t.hms.s = val % 60L;
+	val /= 60L;
+	t.hms.m = val % 60L;
+	val /= 60L;
+	t.hms.h = val % 24L;
+
+	/* read correction */
+	if ((val = strtoul(ep, &ep, 10), ep == NULL || val == ULONG_MAX)) {
 		return -1;
 	}
 
 	/* fix up and convert to target type */
-	val = d.t.hms.u24;
-	/* column-oriented mode */
-	fprintf(stdout, "\t0x%xU/* %u */,\n", val, val);
+	with (uint32_t ual = t.hms.u24) {
+		ual += val >= cor;
+		fprintf(stdout, "\t0x%xU/* %u */,\n", ual, ual);
+		cor = val;
+	}
 	return 0;
 }
 
@@ -382,6 +368,7 @@ parse_file(const char *file)
 #include <stdint.h>\n\
 #include <limits.h>\n\
 #include \"leaps.h\"\n\
+#include \"leap-seconds.h\"\n\
 \n\
 #if !defined INCLUDED_ltrcc_generated_def_\n\
 #define INCLUDED_ltrcc_generated_def_\n\
@@ -407,70 +394,42 @@ parse_file(const char *file)
 	pr_file(fp, "leaps_hms", pr_line_t, DT_HMS, col);
 
 	fputs("\
+/* exported number of leap transitions */\n\
+const size_t nleaps = countof(leaps_corr);\n\
+\n\
 #endif  /* INCLUDED_ltrcc_generated_def_ */\n", stdout);
 	return 0;
 }
 
 
-static void
-pr_version(FILE *where)
-{
-	fputs("ltrcc " VERSION "\n", where);
-	return;
-}
-
-static void
-pr_usage(FILE *where)
-{
-	pr_version(where);
-	fputs("\n\
-Usage: ltrcc LEAPS_FILE\n\
-\n\
-Compile LEAPS_FILE into C source code.\n\
-\n\
-  -h                    Print help and exit\n\
-  -V                    Print version and exit\n\
-\n\
-  -C                    Column-oriented output\n\
-", where);
-	return;
-}
+#include "ltrcc.yucc"
 
 int
 main(int argc, char *argv[])
 {
-	const char *leaps;
+	yuck_t argi[1U];
+	int rc = 0;
 
-	for (int c; (c = getopt(argc, argv, "ChVv")) != -1;) {
-		switch (c) {
-		case 'C':
-			col = 1;
-			break;
-		case 'h':
-			pr_usage(stdout);
-			return 0;
-		case 'v':
-		case 'V':
-			pr_version(stdout);
-			return 0;
-		case '?':
-			pr_usage(stderr);
-			return 1;
-		default:
-			return 2;
-		}
-	}
-	if (optind >= argc) {
-		pr_usage(stderr);
-		return 1;
+	if (yuck_parse(argi, argc, argv) < 0) {
+		rc = 1;
+		goto out;
+	} else if (!argi->nargs) {
+		fputs("LEAP-SECONDSS.LIST argument is mandatory\n", stderr);
+		rc = 1;
+		goto out;
 	}
 
-	leaps = argv[optind];
-	if (parse_file(leaps) < 0) {
+	/* assign params */
+	col = argi->column_oriented_flag;
+
+	if (parse_file(argi->args[0U]) < 0) {
 		perror("Cannot parse file");
-		return 1;
+		rc = 1;
 	}
-	return 0;
+
+out:
+	yuck_free(argi);
+	return rc;
 }
 
 /* ltrcc.c ends here */
