@@ -1,6 +1,6 @@
 /*** strops.c -- useful string operations
  *
- * Copyright (C) 2011-2012 Sebastian Freundt
+ * Copyright (C) 2011-2020 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -59,21 +59,22 @@
 # pragma GCC diagnostic ignored "-Wcast-qual"
 #endif	/* __INTEL_COMPILER */
 
+#if !defined DEFUN
+# define DEFUN
+#endif	/* DEFUN */
+
 /* stolen from Klaus Klein/David Laight's strptime() */
 DEFUN int32_t
 strtoi_lim(const char *str, const char **ep, int32_t llim, int32_t ulim)
 {
+	const char *sp = str;
 	int32_t res = 0;
-	const char *sp;
-	/* we keep track of the number of digits via rulim */
-	int32_t rulim;
 
-	for (sp = str, rulim = ulim > 10 ? ulim : 10;
-	     res * 10 <= ulim && rulim && *sp >= '0' && *sp <= '9';
-	     sp++, rulim /= 10) {
-		res *= 10;
-		res += *sp - '0';
-	}
+	/* we keep track of the number of digits via rulim */
+	for (int32_t rulim = ulim > 10 ? ulim : 10;
+	     rulim && (unsigned char)(*sp ^ '0') < 10U &&
+		     (res *= 10, res += (unsigned char)(*sp++ ^ '0')) <= ulim;
+	     rulim /= 10);
 	if (UNLIKELY(sp == str)) {
 		res = -1;
 	} else if (UNLIKELY(res < llim || res > ulim)) {
@@ -84,102 +85,72 @@ strtoi_lim(const char *str, const char **ep, int32_t llim, int32_t ulim)
 }
 
 DEFUN int32_t
-strtoi(const char *str, const char **ep)
+padstrtoi_lim(const char *str, const char **ep, int32_t llim, int32_t ulim)
+{
+	const char *sp;
+	int32_t rulim = ulim > 10 ? ulim : 10;
+	int32_t res = 0;
+
+	/* overread whitespace */
+	for (; *str == ' '; str++, rulim /= 10);
+
+	/* we keep track of the number of digits via rulim */
+	for (sp = str;
+	     rulim && (unsigned char)(*sp ^ '0') < 10U &&
+		     (res *= 10, res += (unsigned char)(*sp++ ^ '0')) <= ulim;
+	     rulim /= 10);
+	if (UNLIKELY(sp == str)) {
+		res = -1;
+	} else if (UNLIKELY(res < llim || res > ulim)) {
+		res = -2;
+	}
+	*ep = (char*)sp;
+	return res;
+}
+
+DEFUN int32_t
+strtoi32(const char *str, const char **ep)
 {
 	const char *sp = str;
 	bool negp = false;
-	int32_t res;
+	int32_t res = INT32_MIN;
 
 	if (*str == '-') {
 		negp = true;
 		sp++;
 	}
-	if ((res = strtoi_lim(sp, ep, 0, INT32_MAX)) < 0) {
-		*ep = str;
-	} else if (negp) {
+	while (res < INT32_MAX / 10 && (unsigned char)(*sp ^ '0') < 10U) {
+		res *= 10, res += (unsigned char)(*sp++ ^ '0');
+	}
+	if (negp) {
 		res = -res;
 	}
+	*ep = res > INT32_MIN ? (char*)sp : (char*)str;
 	return res;
 }
 
-DEFUN size_t
-ui32tostr(char *restrict buf, size_t bsz, uint32_t d, int width)
-{
-/* all strings should be little */
-#define C(x, d)	(char)((x) / (d) % 10 + '0')
-	size_t res;
 
-	if (UNLIKELY(d > 1000000000U)) {
-		return 0U;
+DEFUN int64_t
+strtoi64(const char *str, const char **ep)
+{
+	const char *sp = str;
+	bool negp = false;
+	int64_t res = INT64_MIN;
+
+	if (*str == '-') {
+		negp = true;
+		sp++;
 	}
-	switch ((res = (size_t)width) < bsz ? res : bsz) {
-	case 9:
-		/* for nanoseconds */
-		buf[width - 9] = C(d, 100000000);
-		buf[width - 8] = C(d, 10000000);
-		buf[width - 7] = C(d, 1000000);
-	case 6:
-		/* for microseconds */
-		buf[width - 6] = C(d, 100000);
-		buf[width - 5] = C(d, 10000);
-	case 4:
-		/* for western year numbers */
-		buf[width - 4] = C(d, 1000);
-	case 3:
-		/* for milliseconds or doy et al. numbers */
-		buf[width - 3] = C(d, 100);
-	case 2:
-		/* hours, mins, secs, doms, moys, etc. */
-		buf[width - 2] = C(d, 10);
-	case 1:
-		buf[width - 1] = C(d, 1);
-		break;
-	default:
-	case 0:
-		res = 0U;
-		break;
+	while (res < INT64_MAX / 10 && (unsigned char)(*sp ^ '0') < 10U) {
+		res *= 10, res += (unsigned char)(*sp++ ^ '0');
 	}
-#undef C
+	if (negp) {
+		res = -res;
+	}
+	*ep = res > INT64_MIN ? (char*)sp : (char*)str;
 	return res;
 }
 
-DEFUN size_t
-ui32topstr(char *restrict b, size_t z, uint32_t d, int width, char pad)
-{
-/* all strings should be little */
-#define C(x)	(char)((x) + '0')
-	char *restrict bp = b;
-	const char *const ep = b + z;
-
-	if (UNLIKELY(d > 1000000000U)) {
-		return 0U;
-	} else if (!d) {
-		*bp++ = C(0U);
-		width--;
-	}
-	/* write the mantissa right to left */
-	for (; d && bp < ep; width--) {
-		register unsigned int x = d % 10U;
-
-		d /= 10U;
-		*bp++ = C(x);
-	}
-	/* fill up with padding */
-	if (LIKELY(pad)) {
-		while (width-- > 0) {
-			*bp++ = pad;
-		}
-	}
-	/* reverse the string */
-	for (char *ip = b, *jp = bp - 1; ip < jp; ip++, jp--) {
-		register char tmp = *ip;
-		*ip = *jp;
-		*jp = tmp;
-	}
-#undef C
-	return bp - b;
-}
-
 
 /* roman numerals */
 static int32_t
@@ -397,8 +368,8 @@ DEFUN int32_t
 strtoarri(const char *buf, const char **ep, const char *const *arr, size_t narr)
 {
 /* take a string, compare it to an array of string (case-insensitively) and
- * return its index if found or -1 if not */
-	for (size_t i = 0; i < narr; i++) {
+ * return its index if found or 0 if not */
+	for (size_t i = 1U; i < narr; i++) {
 		const char *chk = arr[i];
 		size_t len = strlen(chk);
 
@@ -515,6 +486,16 @@ xstrpbrkp(const char *src, const char *set, size_t *set_offs)
 	return (char*)p;
 }
 
+DEFUN char*
+xmempbrk(const char *src, size_t len, const char *set)
+{
+	size_t i;
+
+	set_up_table((const unsigned char*)set, false);
+	for (i = 0U; i < len && !in_current_set((unsigned char)src[i]); i++);
+	return (char*)src + i;
+}
+
 #if defined __INTEL_COMPILER
 # pragma warning (default:2203)
 #elif defined __GNUC__
