@@ -1,6 +1,6 @@
 /*** token.c -- tokeniser specs and stuff
  *
- * Copyright (C) 2011-2012 Sebastian Freundt
+ * Copyright (C) 2011-2020 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -56,7 +56,7 @@
 struct dt_spec_s
 __tok_spec(const char *fp, const char **ep)
 {
-	struct dt_spec_s res = spec_initialiser();
+	struct dt_spec_s res = {0};
 
 	if (*fp != '%') {
 		goto out;
@@ -72,9 +72,9 @@ next:
 	case 'T':
 		res.spfl = DT_SPFL_N_TSTD;
 		break;
-	case 'y':
-		res.abbr = DT_SPMOD_ABBR;
 	case 'Y':
+		res.abbr = DT_SPMOD_LONG;
+	case 'y':
 		res.spfl = DT_SPFL_N_YEAR;
 		break;
 	case 'm':
@@ -146,6 +146,10 @@ next:
 		break;
 	case 's':
 		res.spfl = DT_SPFL_N_EPOCH;
+		if (fp[1U] == '%' && fp[2U] == 'N') {
+			res.spfl = DT_SPFL_N_EPOCHNS;
+			fp += 2U;
+		}
 		break;
 	case 'Z':
 		res.spfl = DT_SPFL_N_ZDIFF;
@@ -182,12 +186,18 @@ next:
 		/* SPC padding modifier */
 		res.pad = DT_SPPAD_SPC;
 		goto next;
+	case '-':
+		/* OMIT padding modifier */
+		res.pad = DT_SPPAD_OMIT;
+		goto next;
 	case 'r':
 		/* real modifier */
 		res.tai = 1;
 		goto next;
 	case 'G':
 		/* for compatibility with posix */
+		res.abbr = DT_SPMOD_LONG;
+	case 'g':
 		res.tai = 1U;
 		res.spfl = DT_SPFL_N_YEAR;
 		break;
