@@ -1,6 +1,6 @@
 /*** yd.c -- guts for yd dates that consist of a year and a year-day
  *
- * Copyright (C) 2010-2012 Sebastian Freundt
+ * Copyright (C) 2010-2020 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -45,6 +45,10 @@
 
 #include "nifty.h"
 
+#if !defined DEFUN
+# define DEFUN
+#endif	/* !DEFUN */
+
 /* algo choices for jan01 wd determination */
 #if defined GET_JAN01_WDAY_FULL_LOOKUP
 #elif defined GET_JAN01_WDAY_28Y_LOOKUP
@@ -143,7 +147,7 @@ __get_jan01_block(unsigned int year)
 	return __jan01_wday[(year - __JAN01_WDAY_BEG) / __JAN01_Y_PER_B];
 }
 
-static inline __attribute__((pure)) dt_dow_t
+static inline __attribute__((const)) dt_dow_t
 __get_jan01_wday(unsigned int year)
 {
 /* get the weekday of jan01 in YEAR */
@@ -225,7 +229,7 @@ static const dt_dow_t __jan01_28y_wday[] = {
 # undef A
 # undef S
 
-static __attribute__((pure)) unsigned int
+static __attribute__((const)) unsigned int
 __get_28y_year_equiv(unsigned year)
 {
 /* the 28y cycle works for 1901 to 2100, for other years find an equivalent */
@@ -265,7 +269,7 @@ __get_jan01_wday(unsigned int year)
 
 #elif defined GET_JAN01_WDAY_28Y_SWITCH
 
-static inline __attribute__((pure)) dt_dow_t
+static inline __attribute__((const)) dt_dow_t
 __get_jan01_wday(unsigned int year)
 {
 /* get the weekday of jan01 in YEAR
@@ -350,14 +354,15 @@ __get_jan01_wday(unsigned int year)
 
 #elif defined GET_JAN01_WDAY_SAKAMOTO
 
-static inline __attribute__((pure)) dt_dow_t
+static inline __attribute__((const)) dt_dow_t
 __get_jan01_wday(unsigned int year)
 {
 	unsigned int res;
 
 	year--,
 		res = year + year / 4 - year / 100 + year / 400 + 1;
-	return (dt_dow_t)(res % GREG_DAYS_P_WEEK);
+	res %= GREG_DAYS_P_WEEK;
+	return (dt_dow_t)(res ?: DT_SUNDAY);
 }
 
 #endif	/* GET_JAN01_WDAY_* */
@@ -366,11 +371,18 @@ __get_jan01_wday(unsigned int year)
 static inline __attribute__((pure)) unsigned int
 __md_get_yday(unsigned int year, unsigned int mon, unsigned int dom)
 {
+	static uint16_t __mon_yday[] = {
+		/* this is \sum ml,
+		 * first element is a bit set of leap days to add */
+		0xfff8, 0,
+		31, 59, 90, 120, 151, 181,
+		212, 243, 273, 304, 334, 365
+	};
 	return __mon_yday[mon] + dom + UNLIKELY(__leapp(year) && mon >= 3);
 }
 
 #elif defined YMD_GET_YD_DIVREM
-static inline __attribute__((pure)) unsigned int
+static inline __attribute__((const)) unsigned int
 __md_get_yday(unsigned int year, unsigned int mon, unsigned int dom)
 {
 #define SL(x)	((x) * 5)
@@ -398,7 +410,7 @@ __md_get_yday(unsigned int year, unsigned int mon, unsigned int dom)
 
 #if defined GET_MD_FREUNDT
 /* Freundt's 32-adic algo */
-static struct __md_s
+static __attribute__((pure)) struct __md_s
 __yday_get_md(unsigned int year, unsigned int doy)
 {
 /* Given a year and the day of the year, return gregorian month + dom
@@ -425,7 +437,6 @@ __yday_get_md(unsigned int year, unsigned int doy)
 #define GET_REM(x)	(rem[x])
 	static const uint8_t rem[] = {
 		19, 19, 18, 14, 13, 11, 10, 8, 7, 6, 4, 3, 1, 0
-
 	};
 	unsigned int m;
 	unsigned int d;
@@ -456,7 +467,7 @@ __yday_get_md(unsigned int year, unsigned int doy)
 
 #elif defined GET_MD_TRIMESTER
 /* Trimester algo, we can't figure out which one's faster */
-static struct __md_s
+static __attribute__((const)) struct __md_s
 __yday_get_md(unsigned int year, unsigned int yday)
 {
 /* The idea here is that the year can be divided into trimesters:
@@ -506,6 +517,32 @@ __yday_get_md(unsigned int year, unsigned int yday)
 }
 #endif	/* GET_MD_* */
 
+static inline __attribute__((const)) dt_dow_t
+__get_jan01_yday_dow(unsigned int yd, dt_dow_t w)
+{
+	unsigned int res = (yd + 6U - (unsigned int)w) % GREG_DAYS_P_WEEK;
+	return (dt_dow_t)(DT_SUNDAY - res);
+}
+
+static inline __attribute__((const)) unsigned int
+__get_ydays(unsigned int y)
+{
+	return LIKELY(!__leapp(y)) ? 365U : 366U;
+}
+
+DEFUN __attribute__((pure)) dt_yd_t
+__yd_fixup(dt_yd_t d)
+{
+	int ydays;
+
+	if (LIKELY(d.d <= 365)) {
+		/* trivial */
+		;
+	} else if (d.d > (ydays = __get_ydays(d.y))) {
+		d.d = ydays;
+	}
+	return d;
+}
 #endif	/* YD_ASPECT_HELPERS_ */
 
 
@@ -517,20 +554,20 @@ __get_isowk_wd(unsigned int yd, dt_dow_t f01)
 /* given the weekday the year starts with, F01, and the year-day YD
  * return the iso week number */
 	static const int_fast8_t iso[] = {2, 1, 0, -1, -2, 4, 3, 2};
-	return (yd - iso[f01]) / GREG_DAYS_P_WEEK + 1;
+	return (GREG_DAYS_P_WEEK + yd - iso[f01]) / GREG_DAYS_P_WEEK;
 }
 
-DEFUN int
+DEFUN __attribute__((const)) int
 __yd_get_wcnt_abs(dt_yd_t d)
 {
 /* absolutely count the n-th occurrence of WD regardless what WD
  * the year started with */
 	int yd = d.d;
 	/* express yd as 7k + n relative to jan01 */
-	return (yd - 1) / 7 + 1;
+	return (GREG_DAYS_P_WEEK + yd - 1) / GREG_DAYS_P_WEEK;
 }
 
-DEFUN int
+DEFUN __attribute__((const)) int
 __yd_get_wcnt_iso(dt_yd_t d)
 {
 /* like __yd_get_wcnt() but for iso week conventions
@@ -545,7 +582,8 @@ __yd_get_wcnt_iso(dt_yd_t d)
 	int wk;
 
 	/* express yd as 7k + n relative to jan01 */
-	if (UNLIKELY((wk = __get_isowk_wd(yd, (dt_dow_t)y01)) < 1)) {
+	wk = __get_isowk_wd(yd, (dt_dow_t)y01);
+	if (UNLIKELY(wk < 1)) {
 		/* get last years y01
 		 * which is basically y01 - (365|366 % 7) */
 		if (LIKELY(!__leapp(--y))) {
@@ -557,7 +595,7 @@ __yd_get_wcnt_iso(dt_yd_t d)
 			y01 += 5;
 			yd += 366;
 		}
-		if (y01 >= GREG_DAYS_P_WEEK) {
+		if (y01 > GREG_DAYS_P_WEEK) {
 			y01 -= GREG_DAYS_P_WEEK;
 		}
 		/* same computation now */
@@ -579,8 +617,8 @@ __yd_get_wcnt_iso(dt_yd_t d)
 	return wk;
 }
 
-DEFUN int
-__yd_get_wcnt(dt_yd_t d, int wdays_from)
+DEFUN __attribute__((const)) int
+__yd_get_wcnt(dt_yd_t d, dt_dow_t _1st_wd)
 {
 /* absolutely count the n-th occurrence of WD regardless what WD
  * the year started with */
@@ -590,22 +628,174 @@ __yd_get_wcnt(dt_yd_t d, int wdays_from)
 	int wk;
 
 	/* yd of the FIRST week of the year */
-	if ((wk = 8 - (int)y01 + wdays_from) > 7) {
+	if ((wk = 8 - (int)y01 + (int)_1st_wd) > 7) {
 		wk -= 7;
 	}
 	/* and now express yd as 7k + n relative to jan01 */
 	return (yd - wk + 7) / 7;
 }
+
+static dt_dow_t
+__yd_get_wday(dt_yd_t this)
+{
+	unsigned int j01_wd = __get_jan01_wday(this.y);
+	if (LIKELY(j01_wd != DT_MIRACLEDAY && this.d)) {
+		unsigned int wd = (this.d - 1 + j01_wd) % GREG_DAYS_P_WEEK;
+		return (dt_dow_t)(wd ?: DT_SUNDAY);
+	}
+	return DT_MIRACLEDAY;
+}
+
+static struct __md_s
+__yd_get_md(dt_yd_t this)
+{
+	return __yday_get_md(this.y, this.d);
+}
 #endif	/* YD_ASPECT_GETTERS_ */
 
+
+#if defined ASPECT_CONV && !defined YD_ASPECT_CONV_
+#define YD_ASPECT_CONV_
+/* we need some getter stuff, so get it */
+#define ASPECT_GETTERS
+#include "yd.c"
+#undef ASPECT_GETTERS
+
+static dt_ymd_t
+__yd_to_ymd(dt_yd_t d)
+{
+	struct __md_s md = __yd_get_md(d);
+
+#if defined HAVE_ANON_STRUCTS_INIT
+	return (dt_ymd_t){.y = d.y, .m = md.m, .d = md.d};
+#else  /* !HAVE_ANON_STRUCTS_INIT */
+	dt_ymd_t res;
+	res.y = d.y;
+	res.m = md.m;
+	res.d = md.d;
+	return res;
+#endif	/* HAVE_ANON_STRUCTS_INIT */
+}
+
+static dt_daisy_t
+__yd_to_daisy(dt_yd_t d)
+{
+	if (UNLIKELY((signed int)TO_BASE(d.y) < 0)) {
+		return 0;
+	}
+	return __jan00_daisy(d.y) + d.d;
+}
+
+static dt_ymcw_t
+__yd_to_ymcw(dt_yd_t d)
+{
+	struct __md_s md = __yd_get_md(d);
+	unsigned int w = __yd_get_wday(d);
+	unsigned int c;
+
+	/* we obtain C from weekifying the month */
+	c = (md.d - 1U) / GREG_DAYS_P_WEEK + 1U;
+
+#if defined HAVE_ANON_STRUCTS_INIT
+	return (dt_ymcw_t){.y = d.y, .m = md.m, .c = c, .w = w};
+#else  /* !HAVE_ANON_STRUCTS_INIT */
+	dt_ymcw_t res;
+	res.y = d.y;
+	res.m = md.m;
+	res.c = c;
+	res.w = w;
+	return res;
+#endif	/* HAVE_ANON_STRUCTS_INIT */
+}
+
+static dt_ywd_t
+__yd_to_ywd(dt_yd_t d)
+{
+	dt_dow_t w = __yd_get_wday(d);
+	unsigned int c = __yd_get_wcnt_abs(d);
+	return __make_ywd_c(d.y, c, w, YWD_ABSWK_CNT);
+}
+#endif	/* ASPECT_CONV */
+
+
+#if defined ASPECT_ADD && !defined YD_ASPECT_ADD_
+#define YD_ASPECT_ADD_
+static __attribute__((pure)) dt_yd_t
+__yd_fixup_d(unsigned int y, signed int d)
+{
+	dt_yd_t res = {0};
+
+	if (LIKELY(d >= 1 && d <= 365)) {
+		/* all years in our design range have at least 365 days */
+		;
+	} else if (d < 1) {
+		int ydays;
+
+		do {
+			y--;
+			ydays = __get_ydays(y);
+			d += ydays;
+		} while (d < 1);
+
+	} else {
+		int ydays;
+
+		while (d > (ydays = __get_ydays(y))) {
+			d -= ydays;
+			y++;
+		}
+	}
+
+	res.y = y;
+	res.d = d;
+	return res;
+}
+
+static dt_yd_t
+__yd_add_d(dt_yd_t d, int n)
+{
+/* add N days to D */
+	signed int tgtd = d.d + n;
+
+	/* fixup the day */
+	return __yd_fixup_d(d.y, tgtd);
+}
+
+static dt_yd_t
+__yd_add_b(dt_yd_t d, int n)
+{
+/* add N business days to D */
+	dt_dow_t wd = __yd_get_wday(d);
+	int tgtd = d.d + __get_d_equiv(wd, n);
+
+	/* fixup the day, i.e. 2012-0367 -> 2013-001 */
+	return __yd_fixup_d(d.y, tgtd);
+}
+
+static dt_yd_t
+__yd_add_w(dt_yd_t d, int n)
+{
+/* add N weeks to D */
+	return __yd_add_d(d, GREG_DAYS_P_WEEK * n);
+}
+
+static dt_yd_t
+__yd_add_y(dt_yd_t d, int n)
+{
+/* add N years to D */
+	d.y += n;
+	return d;
+}
+#endif	/* ASPECT_ADD */
+
 
 #if defined ASPECT_DIFF && !defined YD_ASPECT_DIFF_
 #define YD_ASPECT_DIFF_
-static struct dt_d_s
+static __attribute__((const)) struct dt_ddur_s
 __yd_diff(dt_yd_t d1, dt_yd_t d2)
 {
 /* compute d2 - d1 entirely in terms of ymd but express the result as yd */
-	struct dt_d_s res = {.typ = DT_YD, .dur = 1};
+	struct dt_ddur_s res = dt_make_ddur(DT_DURYD, 0);
 	signed int tgtd;
 	signed int tgty;
 
