@@ -1,6 +1,6 @@
 /*** ymcw.c -- guts for ymcw dates
  *
- * Copyright (C) 2010-2012 Sebastian Freundt
+ * Copyright (C) 2010-2020 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -36,6 +36,10 @@
  **/
 #define ASPECT_YMCW
 
+#if !defined DEFUN
+# define DEFUN
+#endif	/* !DEFUN */
+
 
 #if !defined YMCW_ASPECT_HELPERS_
 #define YMCW_ASPECT_HELPERS_
@@ -63,7 +67,7 @@ __get_mcnt(unsigned int y, unsigned int m, dt_dow_t w)
 	}
 }
 
-static dt_ymcw_t
+DEFUN dt_ymcw_t
 __ymcw_fixup(dt_ymcw_t d)
 {
 /* given dates like 2012-02-05-01 this returns 2012-02-04-01 */
@@ -196,7 +200,8 @@ __ymcw_get_yday(dt_ymcw_t that)
 		},
 	};
 	dt_dow_t j01w = __get_jan01_wday(that.y);
-	unsigned int diff = j01w <= that.w ? that.w - j01w : that.w + 7 - j01w;
+	dt_dow_t w = (dt_dow_t)(that.w ?: DT_THURSDAY);
+	unsigned int diff = j01w <= w ? w - j01w : w + 7 - j01w;
 
 	if (UNLIKELY(__leapp(that.y))) {
 		switch (diff) {
@@ -293,7 +298,7 @@ static dt_ywd_t
 __ymcw_to_ywd(dt_ymcw_t d)
 {
 	unsigned int y = d.y;
-	unsigned int w = d.w;
+	dt_dow_t w = (dt_dow_t)d.w;
 	unsigned int c = __ymcw_get_yday(d);
 	return __make_ywd_c(y, c, w, YWD_ABSWK_CNT);
 }
@@ -396,10 +401,10 @@ __ymcw_add_d(dt_ymcw_t d, int n)
 	signed int aw = n / (signed int)GREG_DAYS_P_WEEK;
 	signed int ad = n % (signed int)GREG_DAYS_P_WEEK;
 
-	if ((ad += d.w) >= (signed int)GREG_DAYS_P_WEEK) {
+	if ((ad += d.w) > (signed int)GREG_DAYS_P_WEEK) {
 		ad -= GREG_DAYS_P_WEEK;
 		aw++;
-	} else if (ad < 0) {
+	} else if (ad <= 0) {
 		ad += GREG_DAYS_P_WEEK;
 		aw--;
 	}
@@ -471,7 +476,7 @@ __ymcw_add_m(dt_ymcw_t d, int n)
 	}
 	/* final assignment */
 	d.m = tgtm;
-	return __ymcw_fixup(d);
+	return d;
 }
 
 static dt_ymcw_t
@@ -479,7 +484,7 @@ __ymcw_add_y(dt_ymcw_t d, int n)
 {
 /* add N years to D */
 	d.y += n;
-	return __ymcw_fixup(d);
+	return d;
 }
 #endif	/* ASPECT_ADD */
 
@@ -487,11 +492,11 @@ __ymcw_add_y(dt_ymcw_t d, int n)
 #if defined ASPECT_DIFF && !defined YMCW_ASPECT_DIFF_
 #define YMCW_ASPECT_DIFF_
 
-static struct dt_d_s
+static struct dt_ddur_s
 __ymcw_diff(dt_ymcw_t d1, dt_ymcw_t d2)
 {
 /* compute d2 - d1 entirely in terms of ymd */
-	struct dt_d_s res = {.typ = DT_YMCW, .dur = 1};
+	struct dt_ddur_s res = dt_make_ddur(DT_DURYMCW, 0);
 	signed int tgtd;
 	signed int tgtm;
 	dt_dow_t wd01, wd02;
@@ -547,6 +552,51 @@ __ymcw_diff(dt_ymcw_t d1, dt_ymcw_t d2)
 }
 #endif	/* ASPECT_DIFF */
 
+
+#if defined ASPECT_CMP && !defined YMCW_ASPECT_CMP_
+#define YMCW_ASPECT_CMP_
+DEFUN int
+__ymcw_cmp(dt_ymcw_t d1, dt_ymcw_t d2)
+{
+	if (d1.y < d2.y) {
+		return -1;
+	} else if (d1.y > d2.y) {
+		return 1;
+	} else if (d1.m < d2.m) {
+		return -1;
+	} else if (d1.m > d2.m) {
+		return 1;
+	}
+
+	/* we're down to counts, however, the last W of a month is always
+	 * count 5, even though counting forward it would be 4 */
+	if (d1.c < d2.c) {
+		return -1;
+	} else if (d1.c > d2.c) {
+		return 1;
+	}
+	/* now it's up to the first of the month */
+	{
+		dt_dow_t wd01;
+		unsigned int off1;
+		unsigned int off2;
+
+		wd01 = __get_m01_wday(d1.y, d1.m);
+		/* represent cw as C-th WD01 + OFF */
+		off1 = __uimod(d1.w - wd01, GREG_DAYS_P_WEEK);
+		off2 = __uimod(d2.w - wd01, GREG_DAYS_P_WEEK);
+
+		if (off1 < off2) {
+			return -1;
+		} else if (off1 > off2) {
+			return 1;
+		} else {
+			return 0;
+		}
+	}
+}
+#endif	/* ASPECT_CMP */
+
 
 #if defined ASPECT_STRF && !defined YMCW_ASPECT_STRF_
 #define YMCW_ASPECT_STRF_
