@@ -1,6 +1,6 @@
 /*** ymd.c -- guts for ymd dates
  *
- * Copyright (C) 2010-2012 Sebastian Freundt
+ * Copyright (C) 2010-2020 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -54,6 +54,10 @@
 # define YMD_GET_WDAY_LOOKUP
 #endif
 
+#if !defined DEFUN
+# define DEFUN
+#endif	/* !DEFUN */
+
 
 #if !defined YMD_ASPECT_HELPERS_
 #define YMD_ASPECT_HELPERS_
@@ -72,7 +76,7 @@ __get_mdays(unsigned int y, unsigned int m)
 	return res - __md_get_yday(y, m, 0);
 }
 
-static __attribute__((pure)) dt_ymd_t
+DEFUN __attribute__((pure)) dt_ymd_t
 __ymd_fixup(dt_ymd_t d)
 {
 /* given dates like 2012-02-32 this returns 2012-02-29 */
@@ -96,12 +100,13 @@ __get_m01_wday(unsigned int year, unsigned int mon)
 	unsigned int off;
 	dt_dow_t cand;
 
-	if (UNLIKELY(mon < 1 && mon > GREG_MONTHS_P_YEAR)) {
+	if (UNLIKELY(mon < 1 || mon > GREG_MONTHS_P_YEAR)) {
 		return DT_MIRACLEDAY;
 	}
 	cand = __get_jan01_wday(year);
 	off = __md_get_yday(year, mon, 0);
-	return (dt_dow_t)((cand + off) % GREG_DAYS_P_WEEK);
+	off = (cand + off) % GREG_DAYS_P_WEEK;
+	return (dt_dow_t)(off ?: DT_SUNDAY);
 }
 
 #if defined YMD_GET_WDAY_LOOKUP
@@ -114,7 +119,8 @@ __get_dom_wday(unsigned int year, unsigned int mon, unsigned int dom)
 
 	if ((yd = __md_get_yday(year, mon, dom)) > 0 &&
 	    (j01_wd = __get_jan01_wday(year)) != DT_MIRACLEDAY) {
-		return (dt_dow_t)((yd - 1 + j01_wd) % GREG_DAYS_P_WEEK);
+		unsigned int wd = (yd - 1 + j01_wd) % GREG_DAYS_P_WEEK;
+		return (dt_dow_t)(wd ?: DT_SUNDAY);
 	}
 	return DT_MIRACLEDAY;
 }
@@ -129,6 +135,7 @@ __get_dom_wday(int year, int mon, int dom)
 	int w;
 	int c, x;
 	int d, y;
+	unsigned int wd;
 
 	if ((mon -= 2) <= 0) {
 		mon += 12;
@@ -141,7 +148,8 @@ __get_dom_wday(int year, int mon, int dom)
 	y = d / 4;
 
 	w = (13 * mon - 1) / 5;
-	return (dt_dow_t)((w + x + y + dom + c - 2 * d) % GREG_DAYS_P_WEEK);
+	wd = (w + x + y + dom + c - 2 * d) % GREG_DAYS_P_WEEK;
+	return (dt_dow_t)(wd ?: DT_SUNDAY);
 }
 #elif defined YMD_GET_WDAY_SAKAMOTO
 /* Sakamoto method */
@@ -150,11 +158,13 @@ __get_dom_wday(int year, int mon, int dom)
 {
 	static int t[] = {0, 3, 2, 5, 0, 3, 5, 1, 4, 6, 2, 4};
 	int res;
+	unsigned int wd;
 
 	year -= mon < 3;
 	res = year + year / 4 - year / 100 + year / 400;
 	res += t[mon - 1] + dom;
-	return (dt_dow_t)(res % GREG_DAYS_P_WEEK);
+	wd = (unsigned int)res % GREG_DAYS_P_WEEK;
+	return (dt_dow_t)(wd ?: DT_SUNDAY);
 }
 
 #elif defined YMD_GET_WDAY_7YM_ALL || defined YMD_GET_WDAY_7YM_REP
@@ -351,6 +361,7 @@ __get_dom_wday(unsigned int year, unsigned int mon, unsigned int dom)
 	unsigned int bm = mon - 1;
 	unsigned int bd = dom - 1;
 	unsigned int idx;
+	unsigned int wd;
 
 	if (bm < 2U) {
 		/* FG */
@@ -369,8 +380,8 @@ __get_dom_wday(unsigned int year, unsigned int mon, unsigned int dom)
 		idx = __get_widx_H(by);
 	}
 
-	/* implicit assumption DT_SUN == 0! */
-	return (dt_dow_t)((__get_ser(idx, bm) + bd) % GREG_DAYS_P_WEEK);
+	wd = (__get_ser(idx, bm) + bd) % GREG_DAYS_P_WEEK;
+	return (dt_dow_t)(wd ?: DT_SUNDAY);
 }
 
 #endif	/* 0 */
@@ -402,7 +413,7 @@ __ymd_get_wday(dt_ymd_t that)
 	return __get_dom_wday(that.y, that.m, that.d);
 }
 
-static unsigned int
+DEFUN unsigned int
 __ymd_get_count(dt_ymd_t that)
 {
 /* get N where N is the N-th occurrence of wday in the month of that year */
@@ -484,7 +495,7 @@ __ymd_to_ymcw(dt_ymd_t d)
 static dt_ywd_t
 __ymd_to_ywd(dt_ymd_t d)
 {
-	unsigned int w = __ymd_get_wday(d);
+	dt_dow_t w = __ymd_get_wday(d);
 	unsigned int c = __ymd_get_wcnt_abs(d);
 	return __make_ywd_c(d.y, c, w, YWD_ABSWK_CNT);
 }
@@ -541,6 +552,7 @@ __ymd_fixup_d(unsigned int y, signed int m, signed int d)
 {
 	dt_ymd_t res = {0};
 
+	m += !m;
 	if (LIKELY(d >= 1 && d <= 28)) {
 		/* all months in our design range have at least 28 days */
 		;
@@ -578,7 +590,9 @@ static dt_ymd_t
 __ymd_add_d(dt_ymd_t d, int n)
 {
 /* add N days to D */
-	signed int tgtd = d.d + n;
+	signed int tgtd;
+	d = __ymd_fixup(d);
+	tgtd = d.d + n;
 
 	/* fixup the day */
 	return __ymd_fixup_d(d.y, d.m, tgtd);
@@ -588,8 +602,12 @@ static dt_ymd_t
 __ymd_add_b(dt_ymd_t d, int n)
 {
 /* add N business days to D */
-	dt_dow_t wd = __ymd_get_wday(d);
-	int tgtd = d.d + __get_d_equiv(wd, n);
+	dt_dow_t wd;
+	signed int tgtd;
+
+	d = __ymd_fixup(d);
+	wd = __ymd_get_wday(d);
+	tgtd = d.d + __get_d_equiv(wd, n);
 
 	/* fixup the day, i.e. 2012-01-34 -> 2012-02-03 */
 	return __ymd_fixup_d(d.y, d.m, tgtd);
@@ -618,7 +636,7 @@ __ymd_add_m(dt_ymd_t d, int n)
 	}
 	/* final assignment */
 	d.m = tgtm;
-	return __ymd_fixup(d);
+	return d;
 }
 
 static dt_ymd_t
@@ -626,18 +644,18 @@ __ymd_add_y(dt_ymd_t d, int n)
 {
 /* add N years to D */
 	d.y += n;
-	return __ymd_fixup(d);
+	return d;
 }
 #endif	/* ASPECT_ADD */
 
 
 #if defined ASPECT_DIFF && !defined YMD_ASPECT_DIFF_
 #define YMD_ASPECT_DIFF_
-static struct dt_d_s
+static struct dt_ddur_s
 __ymd_diff(dt_ymd_t d1, dt_ymd_t d2)
 {
 /* compute d2 - d1 entirely in terms of ymd */
-	struct dt_d_s res = {.typ = DT_YMD, .dur = 1};
+	struct dt_ddur_s res = dt_make_ddur(DT_DURYMD, 0);
 	signed int tgtd;
 	signed int tgtm;
 
