@@ -1,6 +1,6 @@
 /*** ywd.c -- guts for ywd dates
  *
- * Copyright (C) 2012 Sebastian Freundt
+ * Copyright (C) 2012-2020 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -39,6 +39,10 @@
 
 #include "nifty.h"
 
+#if !defined DEFUN
+# define DEFUN
+#endif	/* !DEFUN */
+
 /* table of 53-week years (years % 400)
 
   {4, 9, 15, 20, 26, 32, 37, 43, 48, 54, 60, 65, 71, 76, 82, 88, 93, 99, 105,
@@ -66,8 +70,11 @@ __ywd_get_jan01_wday(dt_ywd_t d)
 /* hang of 0 means Mon, -1 Tue, -2 Wed, -3 Thu, 3 Fri, 2 Sat, 1 Sun */
 	int res;
 
-	if (UNLIKELY((res = 1 - d.hang) < 0)) {
-		return (dt_dow_t)(GREG_DAYS_P_WEEK + res);
+	/* the second portion of the assert is trivially true */
+	assert(d.hang >= -3 /*&& d.hang <= 3*/);
+	if (UNLIKELY((res = 1 - d.hang) <= 0)) {
+		res += GREG_DAYS_P_WEEK;
+		assert(res > DT_MIRACLEDAY);
 	}
 	return (dt_dow_t)res;
 }
@@ -79,7 +86,8 @@ __ywd_get_jan01_hang(dt_dow_t j01)
 	int res;
 
 	if (UNLIKELY((res = 1 - (int)j01) < -3)) {
-		return (int)(GREG_DAYS_P_WEEK + res);
+		assert(res >= -6);
+		return (int)GREG_DAYS_P_WEEK + res;
 	}
 	return res;
 }
@@ -91,13 +99,16 @@ __ywd_get_dec31_wday(dt_ywd_t d)
 	dt_dow_t res = __ywd_get_jan01_wday(d);
 
 	if (UNLIKELY(__leapp(d.y))) {
-		res = (dt_dow_t)((res + 1) % 7);
+		if (++res > GREG_DAYS_P_WEEK) {
+			/* wrap around DT_SUNDAY -> DT_MONDAY */
+			res = DT_MONDAY;
+		}
 	}
 	return res;
 }
 
 #if defined GET_ISOWK_FULL_SWITCH
-static unsigned int
+DEFUN __attribute__((const)) inline unsigned int
 __get_isowk(unsigned int y)
 {
 /* return the number of iso weeks in Y */
@@ -354,7 +365,7 @@ __get_z31wk(unsigned int y)
 }
 
 #elif defined GET_ISOWK_28Y_SWITCH
-static inline __attribute__((pure))  unsigned int
+DEFUN __attribute__((const)) inline unsigned int
 __get_isowk(unsigned int y)
 {
 	switch (y % 28U) {
@@ -382,6 +393,7 @@ __get_z31wk(unsigned int y)
  * the new year are treated as 53
  * In the 400 year cycle, there's 243 years with 53 weeks and
  * 157 years with 52 weeks. */
+#if 0
 	switch (y % 28U) {
 	default:
 		break;
@@ -401,11 +413,20 @@ __get_z31wk(unsigned int y)
 	}
 	/* more weeks with 53, so default to that */
 	return 53;
+#endif
+#define BIT(x)	(1U << (x))
+	/* redo with bitmasks */
+	static const unsigned int w =
+		BIT(1U) ^ BIT(2U) ^ BIT(3U) ^ BIT(4U) ^
+		BIT(7U) ^ BIT(8U) ^ BIT(9U) ^ BIT(12U) ^
+		BIT(13U) ^ BIT(14U) ^ BIT(15U) ^ BIT(18U) ^
+		BIT(19U) ^ BIT(20U) ^ BIT(24U) ^ BIT(25U) ^ BIT(26U);
+	return 52 + (w >> (y % 28U)) & 0b1U;
 }
 
 #endif	/* GET_ISOWK_* */
 
-static __attribute__((pure)) dt_ywd_t
+DEFUN __attribute__((pure)) dt_ywd_t
 __ywd_fixup(dt_ywd_t d)
 {
 /* given dates like 2012-W53-01 this returns 2013-W01-01 */
@@ -420,13 +441,36 @@ __ywd_fixup(dt_ywd_t d)
 	return d;
 }
 
+#define canon_yc(y, c, hang)				\
+	if (LIKELY(c >= 1 && c <= 52)) {		\
+		/* all years support this */		\
+		;					\
+	} else if (UNLIKELY(c < 1)) {			\
+		if (UNLIKELY(__leapp(--y))) {		\
+			hang++;				\
+		}					\
+		if (++hang > 3) {			\
+			hang -= GREG_DAYS_P_WEEK;	\
+		}					\
+		c += __get_isowk(y);			\
+	} else if (UNLIKELY(c > __get_isowk(y))) {	\
+		c -= __get_isowk(y);			\
+		if (UNLIKELY(__leapp(y++))) {		\
+			hang--;				\
+		}					\
+		if (--hang < -3) {			\
+			hang += GREG_DAYS_P_WEEK;	\
+		}					\
+	}
+
+
 #endif	/* !YWD_ASPECT_HELPERS_ */
 
 
 #if defined ASPECT_GETTERS && !defined YWD_ASPECT_GETTERS_
 #define YWD_ASPECT_GETTERS_
 static dt_ywd_t
-__make_ywd_c(unsigned int y, unsigned int c, unsigned int w, unsigned int cc)
+__make_ywd_c(unsigned int y, unsigned int c, dt_dow_t w, unsigned int cc)
 {
 /* build a 8601 compliant ywd object from year Y, week C and weekday W
  * where C conforms to week-count convention cc */
@@ -442,17 +486,18 @@ __make_ywd_c(unsigned int y, unsigned int c, unsigned int w, unsigned int cc)
 	default:
 	case YWD_ISOWK_CNT:
 		break;
-	case YWD_ABSWK_CNT:
-		if (hang == 1 && w >= DT_MONDAY) {
+	case YWD_ABSWK_CNT: {
+		dt_dow_t u = w ?: DT_THURSDAY;
+
+		if (hang == 1 && u < DT_SUNDAY) {
 			/* n-th W in the year is n-th week,
-			 * year starts on sunday, so w >= DT_MONDAY is
-			 * equivalent to w < DT_SUNDAY */
+			 * year starts on sunday */
 			;
-		} else if (hang > 0 && w >= DT_MONDAY && w < j01) {
+		} else if (hang > 0 && u < DT_SUNDAY && u < j01) {
 			/* n-th W in the year is n-th week,
 			 * in this case the year doesnt start on sunday */
 			;
-		} else if (hang <= 0 && (w >= j01 || w == DT_SUNDAY)) {
+		} else if (hang <= 0 && (u >= j01 || u == DT_SUNDAY)) {
 			/* n-th W in the year is n-th week */
 			;
 		} else if (hang > 0) {
@@ -462,14 +507,8 @@ __make_ywd_c(unsigned int y, unsigned int c, unsigned int w, unsigned int cc)
 			/* weekdays missing in the first week of Y */
 			c++;
 		}
-
-		if (UNLIKELY(c > __get_isowk(y))) {
-			y++;
-			c = 1;
-		} else if (UNLIKELY(c == 0)) {
-			y--;
-			c = __get_isowk(y);
-		}
+	}
+		canon_yc(y, c, hang);
 		break;
 	case YWD_SUNWK_CNT:
 		if (j01 == DT_SUNDAY) {
@@ -490,13 +529,38 @@ __make_ywd_c(unsigned int y, unsigned int c, unsigned int w, unsigned int cc)
 	/* assign and fuck off */
 	res.y = y;
 	res.c = c;
-	res.w = w < GREG_DAYS_P_WEEK ? w : 0;
+	res.w = w;
+	res.hang = hang;
+	return res;
+}
+
+static dt_ywd_t
+__make_ywd_yd_dow(unsigned int y, int yd, dt_dow_t dow)
+{
+/* build a 8601 compliant ywd object from year Y, day-of-year YD
+ * and weekday DOW */
+	dt_ywd_t res = {0};
+	dt_dow_t j01;
+	unsigned int c;
+	int hang;
+
+	/* deduce the weekday of the first, given the weekday
+	 * of the yd-th is DOW */
+	j01 = __get_jan01_yday_dow(yd, dow);
+	hang = __ywd_get_jan01_hang(j01);
+
+	/* compute weekday, decompose yd into 7p + q */
+	c = (yd + GREG_DAYS_P_WEEK - 1 - hang) / (signed int)GREG_DAYS_P_WEEK;
+
+	/* fixup c (and y) */
+	canon_yc(y, c, hang);
+
+	/* assign and fuck off */
+	res.y = y;
+	res.c = c;
+	res.w = dow;
 	res.hang = hang;
-#if defined WITH_FAST_ARITH
 	return res;
-#else  /* !WITH_FAST_ARITH */
-	return __ywd_fixup(res);
-#endif	/* WITH_FAST_ARITH */
 }
 
 static dt_ywd_t
@@ -506,7 +570,6 @@ __make_ywd_ybd(unsigned int y, int yd)
 	dt_ywd_t res = {0};
 	dt_dow_t j01;
 	unsigned int c;
-	unsigned int nwk;
 	int w;
 	int hang;
 
@@ -515,8 +578,8 @@ __make_ywd_ybd(unsigned int y, int yd)
 	hang = __ywd_get_jan01_hang(j01);
 
 	/* compute weekday, decompose yd into 7p + q */
-	c = (yd - 1) / (signed int)DUWW_BDAYS_P_WEEK;
-	w = (yd - 1) % (signed int)DUWW_BDAYS_P_WEEK;
+	c = (yd + DUWW_BDAYS_P_WEEK - 1) / (signed int)DUWW_BDAYS_P_WEEK;
+	w = (yd + DUWW_BDAYS_P_WEEK - 1) % (signed int)DUWW_BDAYS_P_WEEK;
 	if ((w += j01) > (signed int)DUWW_BDAYS_P_WEEK) {
 		w -= DUWW_BDAYS_P_WEEK;
 		c++;
@@ -526,16 +589,7 @@ __make_ywd_ybd(unsigned int y, int yd)
 	}
 
 	/* fixup c (and y) */
-	c++;
-	if (LIKELY(c >= 1 && c <= 52)) {
-		/* all years support this */
-		;
-	} else if (UNLIKELY(c < 1)) {
-		c += __get_isowk(--y);
-	} else if (UNLIKELY(c > (nwk = __get_isowk(y)))) {
-		c -= nwk;
-		y++;
-	}
+	canon_yc(y, c, hang);
 
 	/* assign and fuck off */
 	res.y = y;
@@ -545,12 +599,14 @@ __make_ywd_ybd(unsigned int y, int yd)
 	return res;
 }
 
-static unsigned int
+static __attribute__((const)) int
 __ywd_get_yday(dt_ywd_t d)
 {
 /* since everything is in ISO 8601 format, getting the doy is a matter of
- * counting how many days there are in a week. */
-	return GREG_DAYS_P_WEEK * (d.c - 1) + (d.w ? d.w : 7) + d.hang;
+ * counting how many days there are in a week.
+ * This may return negative values and values larger than the number of
+ * days in that year. */
+	return GREG_DAYS_P_WEEK * (d.c - 1) +  (dt_dow_t)(d.w ?: DT_THURSDAY) + d.hang;
 }
 
 static dt_dow_t
@@ -564,7 +620,7 @@ __ywd_get_wcnt_mon(dt_ywd_t d)
 {
 /* given a YWD with week-count within the year (ISOWK_CNT convention)
  * return the week-count within the month (ABSWK_CNT convention) */
-	unsigned int yd = __ywd_get_yday(d);
+	int yd = __ywd_get_yday(d);
 	struct __md_s x = __yday_get_md(d.y, yd);
 	return (x.d - 1) / 7 + 1;
 }
@@ -588,12 +644,12 @@ __ywd_get_wcnt_year(dt_ywd_t d, unsigned int tgtcc)
 		}
 		break;
 	case YWD_MONWK_CNT:
-		if (j01 >= DT_TUESDAY) {
+		if (j01 >= DT_TUESDAY && j01 < DT_SUNDAY) {
 			return d.c - 1;
 		}
 		break;
 	case YWD_SUNWK_CNT:
-		if (j01 >= DT_MONDAY) {
+		if (j01 < DT_SUNDAY) {
 			return d.c - 1;
 		}
 		break;
@@ -606,7 +662,7 @@ __ywd_get_wcnt_year(dt_ywd_t d, unsigned int tgtcc)
 static struct __md_s
 __ywd_get_md(dt_ywd_t d)
 {
-	unsigned int yday = __ywd_get_yday(d);
+	int yday = __ywd_get_yday(d);
 	struct __md_s res = __yday_get_md(d.y, yday);
 
 	if (UNLIKELY(res.m == 0)) {
@@ -621,9 +677,25 @@ __ywd_get_md(dt_ywd_t d)
 static unsigned int
 __ywd_get_mon(dt_ywd_t d)
 {
-	unsigned int yd = __ywd_get_yday(d);
+	int yd = __ywd_get_yday(d);
 	return __yday_get_md(d.y, yd).m;
 }
+
+static unsigned int
+__ywd_get_year(dt_ywd_t d)
+{
+/* return the true gregorian year */
+	unsigned int y = d.y;
+
+	if (d.c == 1) {
+		dt_dow_t f01 = __ywd_get_jan01_wday(d);
+		y -= d.hang <= 0 && (dt_dow_t)(d.w ?: DT_THURSDAY) < f01;
+	} else if (d.c >= __get_z31wk(y)) {
+		dt_dow_t z31 = __ywd_get_dec31_wday(d);
+		y += z31 < DT_SUNDAY && (dt_dow_t)(d.w ?: DT_THURSDAY) > z31;
+	}
+	return y;
+}
 #endif	/* ASPECT_GETTERS */
 
 
@@ -637,23 +709,9 @@ __ywd_get_mon(dt_ywd_t d)
 static dt_ymd_t
 __ywd_to_ymd(dt_ywd_t d)
 {
-	unsigned int y = d.y;
+	unsigned int y = __ywd_get_year(d);
 	struct __md_s md = __ywd_get_md(d);
 
-	if (d.c == 1) {
-		dt_dow_t f01 = __ywd_get_jan01_wday(d);
-
-		if (d.hang <= 0 && d.w >= DT_MONDAY && d.w < f01) {
-			y--;
-		}
-
-	} else if (d.c >= __get_z31wk(y)) {
-		dt_dow_t z31 = __ywd_get_dec31_wday(d);
-
-		if (z31 && (d.w > z31 || d.w == DT_SUNDAY)) {
-			y++;
-		}
-	}
 #if defined HAVE_ANON_STRUCTS_INIT
 	return (dt_ymd_t){.y = y, .m = md.m, .d = md.d};
 #else  /* !HAVE_ANON_STRUCTS_INIT */
@@ -671,25 +729,10 @@ __ywd_to_ymd(dt_ywd_t d)
 static dt_ymcw_t
 __ywd_to_ymcw(dt_ywd_t d)
 {
-	unsigned int y = d.y;
+	unsigned int y = __ywd_get_year(d);
 	struct __md_s md = __ywd_get_md(d);
 	unsigned int c;
 
-	if (d.c == 1) {
-		dt_dow_t f01 = __ywd_get_jan01_wday(d);
-
-		if (d.hang <= 0 && d.w >= DT_MONDAY && d.w < f01) {
-			y--;
-		}
-
-	} else if (d.c >= __get_z31wk(y)) {
-		dt_dow_t z31 = __ywd_get_dec31_wday(d);
-
-		if (z31 && (d.w > z31 || d.w == DT_SUNDAY)) {
-			y++;
-		}
-	}
-
 	/* we obtain C from weekifying the month */
 	c = (md.d - 1U) / GREG_DAYS_P_WEEK + 1U;
 
@@ -721,14 +764,24 @@ __ywd_to_daisy(dt_ywd_t d)
 static dt_yd_t
 __ywd_to_yd(dt_ywd_t d)
 {
+	unsigned int y = __ywd_get_year(d);
+	int x = __ywd_get_yday(d);
+
+	if (UNLIKELY(x <= 0)) {
+		/* gotta go for last years thing */
+		x += __get_ydays(d.y - 1);
+	} else if (x > (int)__get_ydays(d.y)) {
+		x -= __get_ydays(d.y);
+	}
+
 #if defined HAVE_ANON_STRUCTS_INIT
-	return (dt_yd_t){.y = d.y, .d = __ywd_get_yday(d)};
-#else
+	return (dt_yd_t){.y = y, .d = x};
+#else  /* !HAVE_ANON_STRUCTS_INIT */
 	dt_yd_t res;
-	res.y = d.y;
-	res.d = __ywd_get_yday(d);
+	res.y = y;
+	res.d = x;
 	return res;
-#endif
+#endif	/* HAVE_ANON_STRUCTS_INIT */
 }
 #endif	/* ASPECT_CONV */
 
@@ -737,44 +790,44 @@ __ywd_to_yd(dt_ywd_t d)
 #define YWD_ASPECT_ADD_
 
 static dt_ywd_t
-__ywd_fixup_w(unsigned int y, signed int w, dt_dow_t d)
+__ywd_fixup_w(unsigned int y, signed int w, dt_dow_t d, int hang)
 {
 	dt_ywd_t res = {0};
-	int new_hang_p = 0;
 
 	/* fixup q */
 	if (LIKELY(w >= 1 && w <= 52)) {
 		/* all years support this */
 		;
 	} else if (w < 1) {
-		int nw;
-
 		do {
-			nw = __get_isowk(--y);
-			w += nw;
+			if (UNLIKELY(__leapp(--y))) {
+				hang++;
+			}
+			if (++hang > 3) {
+				hang -= GREG_DAYS_P_WEEK;
+			}
+			w += __get_isowk(y);
 		} while (w < 1);
-		new_hang_p = 1;
 
 	} else {
 		int nw;
 
 		while (w > (nw = __get_isowk(y))) {
 			w -= nw;
-			y++;
-			new_hang_p = 1;
+			if (UNLIKELY(__leapp(y++))) {
+				hang--;
+			}
+			if (--hang < -3) {
+				hang += GREG_DAYS_P_WEEK;
+			}
 		}
 	}
 
-	if (new_hang_p) {
-		/* recompute hang */
-		dt_dow_t j01 = __get_jan01_wday(y);
-		res.hang = __ywd_get_jan01_hang(j01);
-	}
-
-	/* final assignment, what about the hang? */
+	/* final assignment */
 	res.y = y;
 	res.c = w;
 	res.w = d;
+	res.hang = hang;
 	return res;
 }
 
@@ -784,7 +837,7 @@ __ywd_add_w(dt_ywd_t d, int n)
 /* add N weeks to D */
 	signed int tgtc = d.c + n;
 
-	return __ywd_fixup_w(d.y, tgtc, (dt_dow_t)d.w);
+	return __ywd_fixup_w(d.y, tgtc, (dt_dow_t)d.w, d.hang);
 }
 
 static dt_ywd_t
@@ -795,21 +848,14 @@ __ywd_add_d(dt_ywd_t d, int n)
 	signed int aw = n / 7;
 	signed int ad = n % 7;
 
-	if ((ad += d.w) >= (signed int)GREG_DAYS_P_WEEK) {
+	if ((ad += d.w) > (signed int)GREG_DAYS_P_WEEK) {
 		ad -= GREG_DAYS_P_WEEK;
 		aw++;
-	} else if (ad < 0) {
+	} else if (ad <= 0) {
 		ad += GREG_DAYS_P_WEEK;
 		aw--;
 	}
 
-	/* fixup Sun/Mon wrap around */
-	if ((dt_dow_t)d.w < DT_MONDAY && (dt_dow_t)ad >= DT_MONDAY) {
-		aw++;
-	} else if ((dt_dow_t)d.w >= DT_MONDAY && (dt_dow_t)ad < DT_MONDAY) {
-		aw--;
-	}
-
 	d.w = (dt_dow_t)ad;
 	return __ywd_add_w(d, aw);
 }
@@ -831,7 +877,7 @@ __ywd_add_y(dt_ywd_t d, int n)
 	/* recompute hang */
 	j01 = __get_jan01_wday(d.y);
 	d.hang = __ywd_get_jan01_hang(j01);
-	return __ywd_fixup(d);
+	return d;
 }
 #endif	/* ASPECT_ADD */
 
@@ -839,11 +885,11 @@ __ywd_add_y(dt_ywd_t d, int n)
 #if defined ASPECT_DIFF && !defined YWD_ASPECT_DIFF_
 #define YWD_ASPECT_DIFF_
 
-static struct dt_d_s
+static struct dt_ddur_s
 __ywd_diff(dt_ywd_t d1, dt_ywd_t d2)
 {
 /* compute d2 - d1 entirely in terms of ymd but express the result as yd */
-	struct dt_d_s res = {.typ = DT_YWD, .dur = 1};
+	struct dt_ddur_s res = dt_make_ddur(DT_DURYWD, 0);
 	signed int tgtd;
 	signed int tgtw;
 	signed int tgty;
@@ -884,16 +930,17 @@ __ywd_diff(dt_ywd_t d1, dt_ywd_t d2)
 
 #if defined ASPECT_STRF && !defined YWD_ASPECT_STRF_
 #define YWD_ASPECT_STRF_
-static void
+DEFUN void
 __prep_strfd_ywd(struct strpd_s *tgt, dt_ywd_t d)
 {
 /* place ywd data of THIS into D for printing with FMT. */
-	if (d.c == 1 && d.w < __ywd_get_jan01_wday(d) && d.w) {
+	if (d.c == 1 && d.hang < 0 && d.w < __ywd_get_jan01_wday(d)) {
 		/* put gregorian year into y and real year into q */
 		tgt->y = d.y - 1;
 		tgt->q = d.y;
-	} else if (d.c >= 53) {
-		/* bit of service for the %Y printer */
+	} else if (d.c >= 53 && d.w >= __ywd_get_jan01_wday(d) +
+		   1U/*coz 365 = 1 mod 7*/ + __leapp(d.y)) {
+		/* put gregorian year into y and real year into q */
 		tgt->y = d.y + 1;
 		tgt->q = d.y;
 	} else {
