@@ -1,6 +1,6 @@
 /*** dadd.c -- perform simple date arithmetic, date plus duration
  *
- * Copyright (C) 2011-2012 Sebastian Freundt
+ * Copyright (C) 2011-2020 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -42,20 +42,21 @@
 #include <stdint.h>
 #include <sys/time.h>
 #include <time.h>
-#include <errno.h>
-#include <stdarg.h>
 
 #include "dt-core.h"
 #include "dt-io.h"
-#include "tzraw.h"
+#include "dt-core-tz-glue.h"
+#include "dt-locale.h"
 #include "prchunk.h"
 
+const char *prog = "dadd";
+
 
 static bool
-durs_only_d_p(struct dt_dt_s dur[], size_t ndur)
+durs_only_d_p(struct dt_dtdur_s dur[], size_t ndur)
 {
 	for (size_t i = 0; i < ndur; i++) {
-		if (dur[i].t.typ) {
+		if (dur[i].durtyp >= (dt_dtdurtyp_t)DT_NDURTYP) {
 			return false;
 		}
 	}
@@ -63,7 +64,7 @@ durs_only_d_p(struct dt_dt_s dur[], size_t ndur)
 }
 
 static struct dt_dt_s
-dadd_add(struct dt_dt_s d, struct dt_dt_s dur[], size_t ndur)
+dadd_add(struct dt_dt_s d, struct dt_dtdur_s dur[], size_t ndur)
 {
 	for (size_t i = 0; i < ndur; i++) {
 		d = dt_dtadd(d, dur[i]);
@@ -71,24 +72,6 @@ dadd_add(struct dt_dt_s d, struct dt_dt_s dur[], size_t ndur)
 	return d;
 }
 
-static void
-__attribute__((format(printf, 2, 3)))
-error(int eno, const char *fmt, ...)
-{
-	va_list vap;
-	va_start(vap, fmt);
-	fputs("dadd: ", stderr);
-	vfprintf(stderr, fmt, vap);
-	va_end(vap);
-	if (eno) {
-		fputc(':', stderr);
-		fputc(' ', stderr);
-		fputs(strerror(eno), stderr);
-	}
-	fputc('\n', stderr);
-	return;
-}
-
 
 struct mass_add_clo_s {
 	void *pctx;
@@ -103,18 +86,23 @@ struct mass_add_clo_s {
 	int quietp;
 };
 
-static void
+static int
 proc_line(const struct mass_add_clo_s *clo, char *line, size_t llen)
 {
 	struct dt_dt_s d;
 	char *sp = NULL;
 	char *ep = NULL;
+	int rc = 0;
 
 	do {
 		/* check if line matches, */
-		d = dt_io_find_strpdt2(line, clo->gra, &sp, &ep, clo->fromz);
+		d = dt_io_find_strpdt2(
+			line, llen, clo->gra, &sp, &ep, clo->fromz);
 
 		if (!dt_unk_p(d)) {
+			if (UNLIKELY(d.fix) && !clo->quietp) {
+				rc = 2;
+			}
 			/* perform addition now */
 			d = dadd_add(d, clo->st.durs, clo->st.ndurs);
 
@@ -140,14 +128,15 @@ proc_line(const struct mass_add_clo_s *clo, char *line, size_t llen)
 			/* obviously unmatched, warn about it in non -q mode */
 			if (!clo->quietp) {
 				dt_io_warn_strpdt(line);
+				rc = 2;
 			}
 			break;
 		}
 	} while (1);
-	return;
+	return rc;
 }
 
-static void
+static int
 mass_add_dur(const struct mass_add_clo_s *clo)
 {
 /* read lines from stdin
@@ -155,16 +144,17 @@ mass_add_dur(const struct mass_add_clo_s *clo)
  * add to reference duration
  * output */
 	size_t lno = 0;
+	int rc = 0;
 
 	for (char *line; prchunk_haslinep(clo->pctx); lno++) {
 		size_t llen = prchunk_getline(clo->pctx, &line);
 
-		proc_line(clo, line, llen);
+		rc |= proc_line(clo, line, llen);
 	}
-	return;
+	return rc;
 }
 
-static void
+static int
 mass_add_d(const struct mass_add_clo_s *clo)
 {
 /* read lines from stdin
@@ -173,7 +163,8 @@ mass_add_d(const struct mass_add_clo_s *clo)
  * output */
 	size_t lno = 0;
 	struct dt_dt_s d;
-	struct __strpdtdur_st_s st = __strpdtdur_st_initialiser();
+	struct __strpdtdur_st_s st = {0};
+	int rc = 0;
 
 	for (char *line; prchunk_haslinep(clo->pctx); lno++) {
 		size_t llen;
@@ -192,6 +183,9 @@ mass_add_d(const struct mass_add_clo_s *clo)
 		line[llen] = '\n';
 
 		if (has_dur_p) {
+			if (UNLIKELY(clo->rd.fix) && !clo->quietp) {
+				rc = 2;
+			}
 			/* perform addition now */
 			d = dadd_add(clo->rd, st.durs, st.ndurs);
 
@@ -207,128 +201,171 @@ mass_add_d(const struct mass_add_clo_s *clo)
 		} else if (!clo->quietp) {
 			line[llen] = '\0';
 			dt_io_warn_strpdt(line);
+			rc = 2;
 		}
 		/* just reset the ndurs slot */
 		st.ndurs = 0;
 	}
 	/* free associated duration resources */
 	__strpdtdur_free(&st);
-	return;
+	return rc;
 }
 
 
-#if defined __INTEL_COMPILER
-# pragma warning (disable:593)
-# pragma warning (disable:181)
-#elif defined __GNUC__
-# pragma GCC diagnostic ignored "-Wswitch-enum"
-# pragma GCC diagnostic ignored "-Wunused-function"
-#endif	/* __INTEL_COMPILER */
-#include "dadd.xh"
-#include "dadd.x"
-#if defined __INTEL_COMPILER
-# pragma warning (default:593)
-# pragma warning (default:181)
-#elif defined __GNUC__
-# pragma GCC diagnostic warning "-Wswitch-enum"
-# pragma GCC diagnostic warning "-Wunused-function"
-#endif	/* __INTEL_COMPILER */
+#include "dadd.yucc"
 
 int
 main(int argc, char *argv[])
 {
-	struct gengetopt_args_info argi[1];
+	yuck_t argi[1U];
 	struct dt_dt_s d;
-	struct __strpdtdur_st_s st = __strpdtdur_st_initialiser();
-	char *inp;
+	struct __strpdtdur_st_s st = {0};
 	const char *ofmt;
 	char **fmt;
 	size_t nfmt;
-	int res = 0;
+	int rc = 0;
 	bool dt_given_p = false;
 	zif_t fromz = NULL;
 	zif_t z = NULL;
 	zif_t hackz = NULL;
 
-	/* fixup negative numbers, A -1 B for dates A and B */
-	fixup_argv(argc, argv, "Pp+");
-	if (cmdline_parser(argc, argv, argi)) {
-		res = 1;
+	if (yuck_parse(argi, argc, argv)) {
+		rc = 1;
 		goto out;
-	} else if (argi->inputs_num == 0) {
-		error(0, "Error: DATE or DURATION must be specified\n");
-		cmdline_parser_print_help();
-		res = 1;
+	} else if (argi->nargs == 0) {
+		error("Error: DATE or DURATION must be specified\n");
+		yuck_auto_help(argi);
+		rc = 1;
 		goto out;
 	}
 	/* init and unescape sequences, maybe */
 	ofmt = argi->format_arg;
-	fmt = argi->input_format_arg;
-	nfmt = argi->input_format_given;
-	if (argi->backslash_escapes_given) {
+	fmt = argi->input_format_args;
+	nfmt = argi->input_format_nargs;
+	if (argi->backslash_escapes_flag) {
 		dt_io_unescape(argi->format_arg);
 		for (size_t i = 0; i < nfmt; i++) {
 			dt_io_unescape(fmt[i]);
 		}
 	}
 
-	/* try and read the from and to time zones */
-	if (argi->from_zone_given) {
-		fromz = zif_open(argi->from_zone_arg);
+	if (argi->from_locale_arg) {
+		setilocale(argi->from_locale_arg);
 	}
-	if (argi->zone_given) {
-		z = zif_open(argi->zone_arg);
+	if (argi->locale_arg) {
+		setflocale(argi->locale_arg);
+	}
+
+	/* try and read the from and to time zones */
+	if (argi->from_zone_arg) {
+		fromz = dt_io_zone(argi->from_zone_arg);
+	}
+	if (argi->zone_arg) {
+		z = dt_io_zone(argi->zone_arg);
+	}
+	if (argi->base_arg) {
+		struct dt_dt_s base = dt_strpdt(argi->base_arg, NULL, NULL);
+		dt_set_base(base);
+	}
+
+	/* sanity checks, decide whether we're a mass date adder
+	 * or a mass duration adder, or both, a date and durations are
+	 * present on the command line */
+	with (const char *inp = argi->args[0U]) {
+		/* date parsing needed postponing as we need to find out
+		 * about the durations */
+		if (!dt_unk_p(dt_io_strpdt(inp, fmt, nfmt, NULL))) {
+			dt_given_p = true;
+		}
 	}
 
 	/* check first arg, if it's a date the rest of the arguments are
 	 * durations, if not, dates must be read from stdin */
-	for (size_t i = 0; i < argi->inputs_num; i++) {
-		inp = unfixup_arg(argi->inputs[i]);
+	for (size_t i = dt_given_p; i < argi->nargs; i++) {
+		const char *inp = argi->args[i];
 		do {
 			if (dt_io_strpdtdur(&st, inp) < 0) {
-				if (UNLIKELY(i == 0)) {
-					/* that's ok, must be a date then */
-					dt_given_p = true;
-				} else {
-					error(errno, "Error: \
+				serror("Error: \
 cannot parse duration string `%s'", st.istr);
-					res = 1;
-					goto dur_out;
-				}
+				rc = 1;
+				goto clear;
 			}
 		} while (__strpdtdur_more_p(&st));
 	}
 	/* check if there's only d durations */
 	hackz = durs_only_d_p(st.durs, st.ndurs) ? NULL : fromz;
 
-	/* sanity checks, decide whether we're a mass date adder
-	 * or a mass duration adder, or both, a date and durations are
-	 * present on the command line */
+	/* read the first argument again in light of a completely parsed
+	 * duration sequence */
 	if (dt_given_p) {
-		/* date parsing needed postponing as we need to find out
-		 * about the durations */
-		inp = argi->inputs[0];
+		const char *inp = argi->args[0U];
 		if (dt_unk_p(d = dt_io_strpdt(inp, fmt, nfmt, hackz))) {
-			error(0, "Error: \
-cannot interpret date/time string `%s'", argi->inputs[0]);
-			res = 1;
-			goto dur_out;
+			error("\
+Error: cannot interpret date/time string `%s'", inp);
+			rc = 1;
+			goto clear;
 		}
 	}
 
 	/* start the actual work */
 	if (dt_given_p && st.ndurs) {
 		if (!dt_unk_p(d = dadd_add(d, st.durs, st.ndurs))) {
+			if (UNLIKELY(d.fix) && !argi->quiet_flag) {
+				rc = 2;
+			}
 			if (hackz == NULL && fromz != NULL) {
 				/* fixup zone */
 				d = dtz_forgetz(d, fromz);
 			}
 			dt_io_write(d, ofmt, z, '\n');
-			res = 0;
 		} else {
-			res = 1;
+			rc = 1;
 		}
 
+	} else if (st.ndurs && argi->empty_mode_flag) {
+		size_t lno = 0U;
+		void *pctx;
+
+		/* no threads reading this stream */
+		__io_setlocking_bycaller(stdout);
+
+		/* using the prchunk reader now */
+		if ((pctx = init_prchunk(STDIN_FILENO)) == NULL) {
+			serror("could not open stdin");
+			goto clear;
+		}
+
+		while (prchunk_fill(pctx) >= 0) {
+			for (char *line; prchunk_haslinep(pctx); lno++) {
+				size_t llen = prchunk_getline(pctx, &line);
+				char *ep = NULL;
+
+				if (UNLIKELY(!llen)) {
+					goto empty;
+				}
+				/* try and parse the line */
+				d = dt_io_strpdt_ep(line, fmt, nfmt, &ep, fromz);
+				if (UNLIKELY(dt_unk_p(d))) {
+					goto empty;
+				} else if (ep && (unsigned)*ep >= ' ') {
+					goto empty;
+				}
+				/* do the adding */
+				d = dadd_add(d, st.durs, st.ndurs);
+				if (UNLIKELY(dt_unk_p(d))) {
+					goto empty;
+				}
+
+				if (hackz == NULL && fromz != NULL) {
+					/* fixup zone */
+					d = dtz_forgetz(d, fromz);
+				}
+				dt_io_write(d, ofmt, z, '\n');
+				continue;
+			empty:
+				__io_write("\n", 1U, stdout);
+			}
+		}
 	} else if (st.ndurs) {
 		/* read dates from stdin */
 		struct grep_atom_s __nstk[16], *needle = __nstk;
@@ -351,7 +388,7 @@ cannot interpret date/time string `%s'", argi->inputs[0]);
 
 		/* using the prchunk reader now */
 		if ((pctx = init_prchunk(STDIN_FILENO)) == NULL) {
-			error(errno, "could not open stdin");
+			serror("could not open stdin");
 			goto ndl_free;
 		}
 
@@ -363,10 +400,10 @@ cannot interpret date/time string `%s'", argi->inputs[0]);
 		clo->hackz = hackz;
 		clo->z = z;
 		clo->ofmt = ofmt;
-		clo->sed_mode_p = argi->sed_mode_given;
-		clo->quietp = argi->quiet_given;
+		clo->sed_mode_p = argi->sed_mode_flag;
+		clo->quietp = argi->quiet_flag;
 		while (prchunk_fill(pctx) >= 0) {
-			mass_add_dur(clo);
+			rc |= mass_add_dur(clo);
 		}
 		/* get rid of resources */
 		free_prchunk(pctx);
@@ -385,7 +422,8 @@ cannot interpret date/time string `%s'", argi->inputs[0]);
 
 		/* using the prchunk reader now */
 		if ((pctx = init_prchunk(STDIN_FILENO)) == NULL) {
-			error(errno, "could not open stdin");
+			serror("could not open stdin");
+			goto clear;
 		}
 
 		/* build the clo and then loop */
@@ -395,28 +433,29 @@ cannot interpret date/time string `%s'", argi->inputs[0]);
 		clo->hackz = hackz;
 		clo->z = z;
 		clo->ofmt = ofmt;
-		clo->sed_mode_p = argi->sed_mode_given;
-		clo->quietp = argi->quiet_given;
+		clo->sed_mode_p = argi->sed_mode_flag;
+		clo->quietp = argi->quiet_flag;
 		while (prchunk_fill(pctx) >= 0) {
-			mass_add_d(clo);
+			rc |= mass_add_d(clo);
 		}
 		/* get rid of resources */
 		free_prchunk(pctx);
 	}
-dur_out:
+clear:
 	/* free the strpdur status */
 	__strpdtdur_free(&st);
 
-	if (argi->from_zone_given) {
-		zif_close(fromz);
+	dt_io_clear_zones();
+	if (argi->from_locale_arg) {
+		setilocale(NULL);
 	}
-	if (argi->zone_given) {
-		zif_close(z);
+	if (argi->locale_arg) {
+		setflocale(NULL);
 	}
 
 out:
-	cmdline_parser_free(argi);
-	return res;
+	yuck_free(argi);
+	return rc;
 }
 
 /* dadd.c ends here */
