@@ -1,6 +1,6 @@
 /*** dconv.c -- convert calendrical and time stamp systems
  *
- * Copyright (C) 2011-2012 Sebastian Freundt
+ * Copyright (C) 2011-2020 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -42,33 +42,14 @@
 #include <stdint.h>
 #include <sys/time.h>
 #include <time.h>
-#include <errno.h>
-#include <stdarg.h>
 
 #include "dt-core.h"
 #include "dt-io.h"
-#include "tzraw.h"
+#include "dt-locale.h"
 #include "prchunk.h"
 
 
-/* error() impl */
-static void
-__attribute__((format(printf, 2, 3)))
-error(int eno, const char *fmt, ...)
-{
-	va_list vap;
-	va_start(vap, fmt);
-	fputs("dconv: ", stderr);
-	vfprintf(stderr, fmt, vap);
-	va_end(vap);
-	if (eno) {
-		fputc(':', stderr);
-		fputc(' ', stderr);
-		fputs(strerror(eno), stderr);
-	}
-	fputc('\n', stderr);
-	return;
-}
+const char *prog = "dconv";
 
 struct prln_ctx_s {
 	struct grep_atom_soa_s *ndl;
@@ -79,15 +60,17 @@ struct prln_ctx_s {
 	int quietp;
 };
 
-static void
+static int
 proc_line(struct prln_ctx_s ctx, char *line, size_t llen)
 {
 	struct dt_dt_s d;
 	char *sp = NULL;
 	char *ep = NULL;
+	int rc = 0;
 
 	do {
-		d = dt_io_find_strpdt2(line, ctx.ndl, &sp, &ep, ctx.fromz);
+		d = dt_io_find_strpdt2(
+			line, llen, ctx.ndl, &sp, &ep, ctx.fromz);
 
 		/* check if line matches */
 		if (!dt_unk_p(d) && ctx.sed_mode_p) {
@@ -96,6 +79,9 @@ proc_line(struct prln_ctx_s ctx, char *line, size_t llen)
 			llen -= (ep - line);
 			line = ep;
 		} else if (!dt_unk_p(d)) {
+			if (UNLIKELY(d.fix) && !ctx.quietp) {
+				rc = 2;
+			}
 			dt_io_write(d, ctx.ofmt, ctx.outz, '\n');
 			break;
 		} else if (ctx.sed_mode_p) {
@@ -106,80 +92,114 @@ proc_line(struct prln_ctx_s ctx, char *line, size_t llen)
 			/* obviously unmatched, warn about it in non -q mode */
 			if (!ctx.quietp) {
 				dt_io_warn_strpdt(line);
+				rc = 2;
 			}
 			break;
 		}
 	} while (1);
-	return;
+	return rc;
 }
 
 
-#if defined __INTEL_COMPILER
-# pragma warning (disable:593)
-# pragma warning (disable:181)
-#elif defined __GNUC__
-# pragma GCC diagnostic ignored "-Wswitch-enum"
-# pragma GCC diagnostic ignored "-Wunused-function"
-#endif	/* __INTEL_COMPILER */
-#include "dconv.xh"
-#include "dconv.x"
-#if defined __INTEL_COMPILER
-# pragma warning (default:593)
-# pragma warning (default:181)
-#elif defined __GNUC__
-# pragma GCC diagnostic warning "-Wswitch-enum"
-# pragma GCC diagnostic warning "-Wunused-function"
-#endif	/* __INTEL_COMPILER */
+#include "dconv.yucc"
 
 int
 main(int argc, char *argv[])
 {
-	struct gengetopt_args_info argi[1];
+	yuck_t argi[1U];
 	const char *ofmt;
 	char **fmt;
 	size_t nfmt;
-	int res = 0;
+	int rc = 0;
 	zif_t fromz = NULL;
 	zif_t z = NULL;
 
-	if (cmdline_parser(argc, argv, argi)) {
-		res = 1;
+	if (yuck_parse(argi, argc, argv)) {
+		rc = 1;
 		goto out;
 	}
 	/* init and unescape sequences, maybe */
 	ofmt = argi->format_arg;
-	fmt = argi->input_format_arg;
-	nfmt = argi->input_format_given;
-	if (argi->backslash_escapes_given) {
+	fmt = argi->input_format_args;
+	nfmt = argi->input_format_nargs;
+	if (argi->backslash_escapes_flag) {
 		dt_io_unescape(argi->format_arg);
 		for (size_t i = 0; i < nfmt; i++) {
 			dt_io_unescape(fmt[i]);
 		}
 	}
 
-	/* try and read the from and to time zones */
-	if (argi->from_zone_given) {
-		fromz = zif_open(argi->from_zone_arg);
+	if (argi->locale_arg) {
+		setflocale(argi->locale_arg);
 	}
-	if (argi->zone_given) {
-		z = zif_open(argi->zone_arg);
-	}
-	if (argi->default_given) {
-		struct dt_dt_s dflt = dt_strpdt(argi->default_arg, NULL, NULL);
-		dt_set_default(dflt);
+	if (argi->from_locale_arg) {
+		setilocale(argi->from_locale_arg);
 	}
 
-	if (argi->inputs_num) {
-		for (size_t i = 0; i < argi->inputs_num; i++) {
-			const char *inp = argi->inputs[i];
+	/* try and read the from and to time zones */
+	if (argi->from_zone_arg) {
+		fromz = dt_io_zone(argi->from_zone_arg);
+	}
+	if (argi->zone_arg) {
+		z = dt_io_zone(argi->zone_arg);
+	}
+	if (argi->base_arg) {
+		struct dt_dt_s base = dt_strpdt(argi->base_arg, NULL, NULL);
+		dt_set_base(base);
+	}
+
+	if (argi->nargs) {
+		for (size_t i = 0; i < argi->nargs; i++) {
+			const char *inp = argi->args[i];
 			struct dt_dt_s d = dt_io_strpdt(inp, fmt, nfmt, fromz);
 
 			if (!dt_unk_p(d)) {
+				if (UNLIKELY(d.fix) && !argi->quiet_flag) {
+					rc = 2;
+				}
 				dt_io_write(d, ofmt, z, '\n');
-			} else if (!argi->quiet_given) {
+			} else if (!argi->quiet_flag) {
+				rc = 2;
 				dt_io_warn_strpdt(inp);
 			}
 		}
+	} else if (argi->empty_mode_flag) {
+		/* read from stdin */
+		size_t lno = 0;
+		void *pctx;
+
+		/* no threads reading this stream */
+		__io_setlocking_bycaller(stdout);
+
+		/* using the prchunk reader now */
+		if ((pctx = init_prchunk(STDIN_FILENO)) == NULL) {
+			serror("Error: could not open stdin");
+			goto clear;
+		}
+		while (prchunk_fill(pctx) >= 0) {
+			for (char *line; prchunk_haslinep(pctx); lno++) {
+				size_t llen = prchunk_getline(pctx, &line);
+				struct dt_dt_s d;
+				char *ep = NULL;
+
+				if (UNLIKELY(!llen)) {
+					goto empty;
+				}
+				/* try and parse the line */
+				d = dt_io_strpdt_ep(line, fmt, nfmt, &ep, fromz);
+				if (UNLIKELY(dt_unk_p(d))) {
+					goto empty;
+				} else if (ep && (unsigned)*ep >= ' ') {
+					goto empty;
+				}
+				dt_io_write(d, ofmt, z, '\n');
+				continue;
+			empty:
+				__io_write("\n", 1U, stdout);
+			}
+		}
+		/* get rid of resources */
+		free_prchunk(pctx);
 	} else {
 		/* read from stdin */
 		size_t lno = 0;
@@ -192,8 +212,8 @@ main(int argc, char *argv[])
 			.ofmt = ofmt,
 			.fromz = fromz,
 			.outz = z,
-			.sed_mode_p = argi->sed_mode_given,
-			.quietp = argi->quiet_given,
+			.sed_mode_p = argi->sed_mode_flag,
+			.quietp = argi->quiet_flag,
 		};
 
 		/* no threads reading this stream */
@@ -210,14 +230,14 @@ main(int argc, char *argv[])
 
 		/* using the prchunk reader now */
 		if ((pctx = init_prchunk(STDIN_FILENO)) == NULL) {
-			error(errno, "Error: could not open stdin");
+			serror("Error: could not open stdin");
 			goto ndl_free;
 		}
 		while (prchunk_fill(pctx) >= 0) {
 			for (char *line; prchunk_haslinep(pctx); lno++) {
 				size_t llen = prchunk_getline(pctx, &line);
 
-				proc_line(prln, line, llen);
+				rc |= proc_line(prln, line, llen);
 			}
 		}
 		/* get rid of resources */
@@ -228,16 +248,18 @@ main(int argc, char *argv[])
 		}
 	}
 
-	if (argi->from_zone_given) {
-		zif_close(fromz);
+clear:
+	dt_io_clear_zones();
+	if (argi->from_locale_arg) {
+		setilocale(NULL);
 	}
-	if (argi->zone_given) {
-		zif_close(z);
+	if (argi->locale_arg) {
+		setflocale(NULL);
 	}
 
 out:
-	cmdline_parser_free(argi);
-	return res;
+	yuck_free(argi);
+	return rc;
 }
 
 /* dconv.c ends here */
