@@ -1,6 +1,6 @@
 /*** ddiff.c -- perform simple date arithmetic, date minus date
  *
- * Copyright (C) 2011-2012 Sebastian Freundt
+ * Copyright (C) 2011-2020 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -42,11 +42,11 @@
 #include <stdint.h>
 #include <sys/time.h>
 #include <time.h>
-#include <stdarg.h>
-#include <errno.h>
 
-#include "dt-core.h"
+#include "date-core-private.h"
+#include "dt-core-private.h"
 #include "dt-io.h"
+#include "dt-locale.h"
 #include "prchunk.h"
 
 #if !defined UNUSED
@@ -57,6 +57,7 @@ typedef union {
 	unsigned int flags;
 	struct {
 		unsigned int has_year:1;
+		unsigned int has_qtr:1;
 		unsigned int has_mon:1;
 		unsigned int has_week:1;
 		unsigned int has_day:1;
@@ -71,6 +72,8 @@ typedef union {
 	};
 } durfmt_t;
 
+const char *prog = "ddiff";
+
 
 static durfmt_t
 determine_durfmt(const char *fmt)
@@ -103,6 +106,9 @@ determine_durfmt(const char *fmt)
 			res.has_day = 1;
 			res.has_biz = 1;
 			break;
+		case DT_DAISY:
+			res.has_day = 1;
+			break;
 		case DT_BIZSI:
 			res.has_day = 1;
 			res.has_biz = 1;
@@ -136,6 +142,9 @@ determine_durfmt(const char *fmt)
 			case DT_SPFL_N_YEAR:
 				res.has_year = 1;
 				break;
+			case DT_SPFL_N_QTR:
+				res.has_qtr = 1;
+				break;
 			case DT_SPFL_N_MON:
 			case DT_SPFL_S_MON:
 				res.has_mon = 1;
@@ -177,7 +186,7 @@ determine_durfmt(const char *fmt)
 	return res;
 }
 
-static dt_dttyp_t
+static dt_dtdurtyp_t
 determine_durtype(struct dt_dt_s d1, struct dt_dt_s d2, durfmt_t f)
 {
 	/* the type-multiplication table looks like:
@@ -194,49 +203,31 @@ determine_durtype(struct dt_dt_s d1, struct dt_dt_s d2, durfmt_t f)
 		;
 	} else if (dt_sandwich_only_t_p(d1) || dt_sandwich_only_t_p(d2)) {
 		/* isn't defined */
-		return (dt_dttyp_t)DT_UNK;
-	} else if (f.has_week && f.has_mon) {
-		return (dt_dttyp_t)DT_YMCW;
+		return (dt_dtdurtyp_t)DT_DURUNK;
+	} else if (f.has_week && (f.has_mon || f.has_qtr)) {
+		return (dt_dtdurtyp_t)DT_DURYMD;
 	} else if (f.has_week && f.has_year) {
-		return (dt_dttyp_t)DT_YWD;
-	} else if (f.has_mon) {
-		return (dt_dttyp_t)DT_YMD;
+		return (dt_dtdurtyp_t)DT_DURYWD;
+	} else if (f.has_mon || f.has_qtr) {
+		return (dt_dtdurtyp_t)DT_DURYMD;
 	} else if (f.has_year && f.has_day) {
-		return (dt_dttyp_t)DT_YD;
+		return (dt_dtdurtyp_t)DT_DURYD;
 	} else if (f.has_day && f.has_biz) {
-		return (dt_dttyp_t)DT_BIZSI;
+		return (dt_dtdurtyp_t)DT_DURBD;
 	} else if (f.has_year) {
-		return (dt_dttyp_t)DT_MD;
+		return (dt_dtdurtyp_t)DT_DURYMD;
 	} else if (dt_sandwich_only_d_p(d1) || dt_sandwich_only_d_p(d2)) {
 		/* default date-only type */
-		return (dt_dttyp_t)DT_DAISY;
+		return (dt_dtdurtyp_t)DT_DURD;
 	} else if (UNLIKELY(f.has_tai)) {
 		/* we has tais */
-		return (dt_dttyp_t)DT_SEXYTAI;
+		return (dt_dtdurtyp_t)0xffU;
 	}
-	return (dt_dttyp_t)DT_SEXY;
+	/* otherwise */
+	return (dt_dtdurtyp_t)(DT_DURS + (f.has_nano));
 }
 
 
-/* printers */
-static void
-__attribute__((format(printf, 2, 3)))
-error(int eno, const char *fmt, ...)
-{
-	va_list vap;
-	va_start(vap, fmt);
-	fputs("ddiff: ", stderr);
-	vfprintf(stderr, fmt, vap);
-	va_end(vap);
-	if (eno) {
-		fputc(':', stderr);
-		fputc(' ', stderr);
-		fputs(strerror(eno), stderr);
-	}
-	fputc('\n', stderr);
-	return;
-}
-
 static size_t
 ltostr(char *restrict buf, size_t bsz, long int v,
        int range, unsigned int pad)
@@ -286,119 +277,128 @@ ltostr(char *restrict buf, size_t bsz, long int v,
 static inline void
 dt_io_warn_dur(const char *d1, const char *d2)
 {
-	error(0, "\
+	error("\
 duration between `%s' and `%s' is not defined", d1, d2);
 	return;
 }
 
-static long int
-__strf_tot_secs(struct dt_dt_s dur)
+static __attribute__((pure)) long int
+__strf_tot_secs(struct dt_dtdur_s dur)
 {
-	long int res;
+/* return time portion of duration in UTC seconds */
+	int64_t s = dur.dv;
 
-	if (dur.typ == DT_SEXY) {
-		res = dur.sexydur;
-	} else if (dur.typ == DT_SEXYTAI) {
-		res = dur.soft;
-	} else {
-		/* otherwise */
-		res = dur.t.sdur;
+	if (UNLIKELY(dur.tai) && dur.durtyp == DT_DURS) {
+		return dur.soft - dur.corr;
 	}
-	return res;
+
+	switch (dur.durtyp) {
+	default:
+		/* all the date types */
+		return dur.t.sdur;
+	case DT_DURH:
+		s *= MINS_PER_HOUR;
+		/*@fallthrough@*/
+	case DT_DURM:
+		s *= SECS_PER_MIN;
+		/*@fallthrough@*/
+	case DT_DURS:
+		break;
+	case DT_DURNANO:
+		s /= NANOS_PER_SEC;
+		break;
+	}
+	return s;
 }
 
-static long int
-__strf_tot_corr(struct dt_dt_s dur)
+static __attribute__((pure)) long int
+__strf_tot_corr(struct dt_dtdur_s dur)
 {
-	if (dur.typ == DT_SEXYTAI) {
+	if (dur.durtyp == DT_DURS && dur.tai) {
 		return dur.corr;
 	}
 	/* otherwise no corrections */
 	return 0;
 }
 
-static int
-__strf_tot_days(struct dt_dt_s dur)
+static __attribute__((pure)) int
+__strf_tot_days(struct dt_dtdur_s dur)
 {
-/* DUR expressed solely as the number of days */
-	int d = 0;
+/* return date portion of DURation in days */
+	int d;
 
-	switch (dur.d.typ) {
-	case DT_DAISY:
-		d = dur.d.daisydur;
+	switch (dur.d.durtyp) {
+	case DT_DURD:
+	case DT_DURBD:
+		d = dur.d.dv;
 		break;
-	case DT_BIZSI:
-		d = dur.d.bizsidur;
-		break;
-	case DT_BIZDA:
+	case DT_DURBIZDA:
 		d = dur.d.bizda.bd;
 		break;
-	case DT_YMD:
+	case DT_DURYMD:
 		d = dur.d.ymd.d;
 		break;
-	case DT_YD:
+	case DT_DURYD:
 		d = dur.d.yd.d;
 		break;
-	case DT_YMCW:
-		d = dur.d.ymcw.w + dur.d.ymcw.c * (int)GREG_DAYS_P_WEEK;;
+	case DT_DURYMCW:
+		d = dur.d.ymcw.w + dur.d.ymcw.c * (int)GREG_DAYS_P_WEEK;
 		break;
-	case DT_YWD:
+	case DT_DURYWD:
 		d = dur.d.ywd.w + dur.d.ywd.c * (int)GREG_DAYS_P_WEEK;
 		break;
-	case DT_MD:
-		d = dur.d.md.d;
-		break;
 	default:
+		d = 0;
 		break;
 	}
 	return d;
 }
 
-static int
-__strf_tot_mon(struct dt_dt_s dur)
+static __attribute__((pure)) int
+__strf_tot_mon(struct dt_dtdur_s dur)
 {
 /* DUR expressed as month and days */
-	int m = 0;
+	int m;
 
-	switch (dur.d.typ) {
-	case DT_BIZDA:
+	switch (dur.d.durtyp) {
+	case DT_DURBIZDA:
 		m = dur.d.bizda.m + dur.d.bizda.y * (int)GREG_MONTHS_P_YEAR;
 		break;
-	case DT_YMD:
+	case DT_DURYMD:
 		m = dur.d.ymd.m + dur.d.ymd.y * (int)GREG_MONTHS_P_YEAR;
 		break;
-	case DT_MD:
-		m = dur.d.md.m;
-		break;
-	case DT_YMCW:
+	case DT_DURYMCW:
 		m = dur.d.ymcw.m + dur.d.ymcw.y * (int)GREG_MONTHS_P_YEAR;
 		break;
-	case DT_YD:
+	case DT_DURYD:
 		m = dur.d.yd.y * (int)GREG_MONTHS_P_YEAR;
 		break;
-	case DT_YWD:
+	case DT_DURYWD:
 		m = dur.d.ywd.y * (int)GREG_MONTHS_P_YEAR;
 		break;
 	default:
+		m = 0;
 		break;
 	}
 	return m;
 }
 
-static int
-__strf_ym_mon(struct dt_dt_s dur)
+static __attribute__((pure)) int
+__strf_ym_mon(struct dt_dtdur_s dur)
 {
 	return __strf_tot_mon(dur) % (int)GREG_MONTHS_P_YEAR;
 }
 
-static int
-__strf_tot_years(struct dt_dt_s dur)
+static __attribute__((pure)) int
+__strf_tot_years(struct dt_dtdur_s dur)
 {
 	return __strf_tot_mon(dur) / (int)GREG_MONTHS_P_YEAR;
 }
 
 static struct precalc_s {
+	int neg;
 	int Y;
+	int q;
 	int m;
 	int w;
 	int d;
@@ -410,65 +410,67 @@ static struct precalc_s {
 	long int N;
 
 	long int rS;
-} precalc(durfmt_t f, struct dt_dt_s dur)
+} precalc(durfmt_t f, struct dt_dtdur_s dur)
 {
 #define MINS_PER_DAY	(MINS_PER_HOUR * HOURS_PER_DAY)
 #define SECS_PER_WEEK	(SECS_PER_DAY * GREG_DAYS_P_WEEK)
 #define MINS_PER_WEEK	(MINS_PER_DAY * GREG_DAYS_P_WEEK)
 #define HOURS_PER_WEEK	(HOURS_PER_DAY * GREG_DAYS_P_WEEK)
 	struct precalc_s res = {0};
+	long long int us;
 
 	/* date specs */
 	if (f.has_year) {
 		/* just years */
 		res.Y = __strf_tot_years(dur);
 	}
-	if (f.has_year && f.has_mon) {
+	if (f.has_year && (f.has_mon || f.has_qtr)) {
 		/* years and months */
 		res.m = __strf_ym_mon(dur);
-	} else if (f.has_mon) {
+	} else if (f.has_mon || f.has_qtr) {
 		/* just months */
 		res.m = __strf_tot_mon(dur);
 	}
-	if (f.has_day || f.has_week) {
-		res.d = __strf_tot_days(dur);
-		if (f.has_week) {
-			/* week shadows days in the hierarchy */
-			res.w = res.d / (int)GREG_DAYS_P_WEEK;
-			res.d %= (int)GREG_DAYS_P_WEEK;
-		}
+
+	if (f.has_qtr) {
+		/* split m slot */
+		res.q = res.m / 3;
+		res.m = res.m % 3;
 	}
 
-	/* time specs */
-	if (f.has_sec || f.has_min || f.has_hour) {
-		res.S = __strf_tot_secs(dur);
+	/* the other units are easily converted as their factors are fixed.
+	 * we operate on clean seconds and attribute leap seconds only
+	 * to the S slot, so 59 seconds plus a leap second != 1 minute */
+	with (int64_t S = __strf_tot_secs(dur), d = __strf_tot_days(dur)) {
+		us = d * (int)SECS_PER_DAY + S;
+		res.neg = dur.neg || us < 0;
+		us = us >= 0 ? us : -us;
+	}
 
-		/* carry from the date specs */
-		res.S += res.w * (long int)SECS_PER_WEEK;
-		res.S += res.d * (long int)SECS_PER_DAY;
-
-		if (f.has_week) {
-			res.w = res.S / (long int)SECS_PER_WEEK;
-			if ((res.S %= (long int)SECS_PER_WEEK) < 0) {
-				res.S += SECS_PER_WEEK;
-				res.w--;
-			}
-		}
-		if (f.has_day) {
-			res.d = res.S / (long int)SECS_PER_DAY;
-			if ((res.S %= (long int)SECS_PER_DAY) < 0) {
-				res.S += SECS_PER_DAY;
-				res.d--;
-			}
-		}
-		if (f.has_hour) {
-			res.H = res.S / (long int)SECS_PER_HOUR;
-			res.S %= (long int)SECS_PER_HOUR;
-		}
-		if (f.has_min) {
-			/* minutes and seconds */
-			res.M = res.S / (long int)SECS_PER_MIN;
-			res.S %= (long int)SECS_PER_MIN;
+	if (f.has_week) {
+		/* week shadows days in the hierarchy */
+		res.w = us / SECS_PER_WEEK;
+		us %= SECS_PER_WEEK;
+	}
+	if (f.has_day) {
+		res.d += us / SECS_PER_DAY;
+		us %= SECS_PER_DAY;
+	}
+	if (f.has_hour) {
+		res.H = us / SECS_PER_HOUR;
+		us %= SECS_PER_HOUR;
+	}
+	if (f.has_min) {
+		/* minutes and seconds */
+		res.M = us / SECS_PER_MIN;
+		us %= SECS_PER_MIN;
+	}
+	if (f.has_sec) {
+		res.S = us + __strf_tot_corr(dur);
+	}
+	if (f.has_nano) {
+		if (dur.durtyp == DT_DURNANO) {
+			res.N = dur.dv % NANOS_PER_SEC;
 		}
 	}
 	return res;
@@ -477,7 +479,7 @@ static struct precalc_s {
 static size_t
 __strfdtdur(
 	char *restrict buf, size_t bsz, const char *fmt,
-	struct dt_dt_s dur, durfmt_t f)
+	struct dt_dtdur_s dur, durfmt_t f, bool only_d_p)
 {
 /* like strfdtdur() but do some calculations based on F on the way there */
 	static const char sexy_dflt_dur[] = "%0T";
@@ -492,39 +494,16 @@ __strfdtdur(
 	}
 
 	/* translate high-level format names */
-	if (dur.typ == DT_SEXY) {
-		if (fmt == NULL) {
-			fmt = sexy_dflt_dur;
-			f.has_sec = 1U;
-		}
-	} else if (dur.typ == DT_SEXYTAI) {
-		if (fmt == NULL) {
-			fmt = sexy_dflt_dur;
-			f.has_sec = 1U;
-		}
-	} else if (dt_sandwich_p(dur)) {
-		if (fmt == NULL) {
-			fmt = sexy_dflt_dur;
-			f.has_sec = 1U;
-		} else {
-			__trans_dtdurfmt(&fmt);
-		}
-	} else if (dt_sandwich_only_d_p(dur)) {
-		if (fmt == NULL) {
-			fmt = ddur_dflt_dur;
-			f.has_day = 1U;
-		} else {
-			__trans_ddurfmt(&fmt);
-			f.has_day = 1U;
-		}
-	} else if (dt_sandwich_only_t_p(dur)) {
-		if (fmt == NULL) {
-			fmt = sexy_dflt_dur;
-			f.has_sec = 1U;
-		}
+	if (fmt == NULL && dur.durtyp >= (dt_dtdurtyp_t)DT_NDURTYP) {
+		fmt = sexy_dflt_dur;
+		f.has_sec = 1U;
+	} else if (fmt == NULL) {
+		fmt = ddur_dflt_dur;
+		f.has_day = 1U;
+	} else if (only_d_p) {
+		__trans_ddurfmt(&fmt);
 	} else {
-		bp = buf;
-		goto out;
+		__trans_dtdurfmt(&fmt);
 	}
 
 	/* precompute */
@@ -533,7 +512,7 @@ __strfdtdur(
 	/* assign and go */
 	bp = buf;
 	fp = fmt;
-	if (dur.neg) {
+	if (pre.neg) {
 		*bp++ = '-';
 	}
 	for (char *const eo = buf + bsz; *fp && bp < eo;) {
@@ -577,7 +556,10 @@ __strfdtdur(
 		bizda_suffix:
 			if (spec.bizda) {
 				/* don't print the b after an ordinal */
-				switch (__get_bizda_param(dur.d).ab) {
+				dt_bizda_param_t bprm;
+
+				bprm.bs = dur.d.param;
+				switch (bprm.ab) {
 				case BIZDA_AFTER:
 					*bp++ = 'b';
 					break;
@@ -597,6 +579,10 @@ __strfdtdur(
 			bp += ltostr(bp, eo - bp, pre.m, 2, spec.pad);
 			break;
 
+		case DT_SPFL_N_QTR:
+			bp += ltostr(bp, eo - bp, pre.q, 2, spec.pad);
+			break;
+
 		case DT_SPFL_N_YEAR:
 			bp += ltostr(bp, eo - bp, pre.Y, -1, DT_SPPAD_NONE);
 			break;
@@ -626,6 +612,10 @@ __strfdtdur(
 			bp += ltostr(bp, eo - bp, pre.H, 2, spec.pad);
 			break;
 
+		case DT_SPFL_N_NANO:
+			bp += ltostr(bp, eo - bp, pre.N, 9, DT_SPPAD_ZERO);
+			break;
+
 		default:
 			break;
 		}
@@ -638,11 +628,11 @@ out:
 }
 
 static int
-ddiff_prnt(struct dt_dt_s dur, const char *fmt, durfmt_t f)
+ddiff_prnt(struct dt_dtdur_s dur, const char *fmt, durfmt_t f, bool only_d_p)
 {
 /* this is mainly a better dt_strfdtdur() */
 	char buf[256];
-	size_t res = __strfdtdur(buf, sizeof(buf), fmt, dur, f);
+	size_t res = __strfdtdur(buf, sizeof(buf), fmt, dur, f, only_d_p);
 
 	if (res > 0 && buf[res - 1] != '\n') {
 		/* auto-newline */
@@ -655,143 +645,160 @@ ddiff_prnt(struct dt_dt_s dur, const char *fmt, durfmt_t f)
 }
 
 
-#if defined __INTEL_COMPILER
-# pragma warning (disable:593)
-# pragma warning (disable:181)
-#elif defined __GNUC__
-# pragma GCC diagnostic ignored "-Wswitch-enum"
-# pragma GCC diagnostic ignored "-Wunused-function"
-#endif	/* __INTEL_COMPILER */
-#include "ddiff.xh"
-#include "ddiff.x"
-#if defined __INTEL_COMPILER
-# pragma warning (default:593)
-# pragma warning (default:181)
-#elif defined __GNUC__
-# pragma GCC diagnostic warning "-Wswitch-enum"
-# pragma GCC diagnostic warning "-Wunused-function"
-#endif	/* __INTEL_COMPILER */
+#include "ddiff.yucc"
 
 int
 main(int argc, char *argv[])
 {
-	struct gengetopt_args_info argi[1];
+	yuck_t argi[1U];
 	struct dt_dt_s d;
 	const char *ofmt;
 	const char *refinp;
 	char **fmt;
 	size_t nfmt;
-	int res = 0;
+	int rc = 0;
 	durfmt_t dfmt;
-	dt_dttyp_t dtyp;
+	dt_dtdurtyp_t dtyp;
 	zif_t fromz = NULL;
 
-	if (cmdline_parser(argc, argv, argi)) {
-		res = 1;
+	if (yuck_parse(argi, argc, argv)) {
+		rc = 1;
 		goto out;
 	}
 	/* unescape sequences, maybe */
-	if (argi->backslash_escapes_given) {
+	if (argi->backslash_escapes_flag) {
 		dt_io_unescape(argi->format_arg);
 	}
 
+	if (argi->from_locale_arg) {
+		setilocale(argi->from_locale_arg);
+	}
+
 	/* try and read the from and to time zones */
-	if (argi->from_zone_given) {
-		fromz = zif_open(argi->from_zone_arg);
+	if (argi->from_zone_arg) {
+		fromz = dt_io_zone(argi->from_zone_arg);
+	}
+	if (argi->base_arg) {
+		struct dt_dt_s base = dt_strpdt(argi->base_arg, NULL, NULL);
+		dt_set_base(base);
 	}
 
 	ofmt = argi->format_arg;
-	fmt = argi->input_format_arg;
-	nfmt = argi->input_format_given;
+	fmt = argi->input_format_args;
+	nfmt = argi->input_format_nargs;
 
-	if (argi->inputs_num == 0 ||
-	    (refinp = argi->inputs[0],
+	if (argi->nargs == 0 ||
+	    (refinp = argi->args[0U],
 	     dt_unk_p(d = dt_io_strpdt(refinp, fmt, nfmt, fromz)) &&
 	     dt_unk_p(d = dt_io_strpdt(refinp, NULL, 0U, fromz)))) {
-		error(0, "Error: reference DATE must be specified\n");
-		cmdline_parser_print_help();
-		res = 1;
+		error("Error: reference DATE must be specified\n");
+		yuck_auto_help(argi);
+		rc = 1;
 		goto out;
+	} else if (UNLIKELY(d.fix) && !argi->quiet_flag) {
+		rc = 2;
 	}
 
 	/* try and guess the diff tgttype most suitable for user's FMT */
 	dfmt = determine_durfmt(ofmt);
 
-	if (argi->inputs_num > 1) {
-		for (size_t i = 1; i < argi->inputs_num; i++) {
+	if (argi->nargs > 1) {
+		for (size_t i = 1; i < argi->nargs; i++) {
 			struct dt_dt_s d2;
-			struct dt_dt_s dur;
-			const char *inp = argi->inputs[i];
+			struct dt_dtdur_s dur;
+			const char *inp = argi->args[i];
+			bool onlydp;
 
 			d2 = dt_io_strpdt(inp, fmt, nfmt, fromz);
 			if (dt_unk_p(d2)) {
-				if (!argi->quiet_given) {
+				if (!argi->quiet_flag) {
 					dt_io_warn_strpdt(inp);
+					rc = 2;
 				}
 				continue;
+			} else if (UNLIKELY(d2.fix) && !argi->quiet_flag) {
+				rc = 2;
 			}
 			/* guess the diff type */
-			if ((dtyp = determine_durtype(d, d2, dfmt)) == DT_UNK) {
-				if (!argi->quiet_given) {
+			onlydp = dt_sandwich_only_d_p(d) ||
+				dt_sandwich_only_d_p(d2);
+			if (!(dtyp = determine_durtype(d, d2, dfmt))) {
+				if (!argi->quiet_flag) {
 				        dt_io_warn_dur(refinp, inp);
+					rc = 2;
 				}
 				continue;
 			}
 			/* subtraction and print */
 			dur = dt_dtdiff(dtyp, d, d2);
-			ddiff_prnt(dur, ofmt, dfmt);
+			ddiff_prnt(dur, ofmt, dfmt, onlydp);
 		}
 	} else {
 		/* read from stdin */
 		size_t lno = 0;
 		void *pctx;
 
+		/* convert deprecated -S|--skip-illegal */
+		argi->empty_mode_flag += argi->skip_illegal_flag;
+
 		/* no threads reading this stream */
 		__io_setlocking_bycaller(stdout);
 
 		/* using the prchunk reader now */
 		if ((pctx = init_prchunk(STDIN_FILENO)) == NULL) {
-			error(errno, "Error: could not open stdin");
+			serror("Error: could not open stdin");
 			goto out;
 		}
 		while (prchunk_fill(pctx) >= 0) {
 			for (char *line; prchunk_haslinep(pctx); lno++) {
 				struct dt_dt_s d2;
-				struct dt_dt_s dur;
+				struct dt_dtdur_s dur;
+				bool onlydp;
 
 				(void)prchunk_getline(pctx, &line);
 				d2 = dt_io_strpdt(line, fmt, nfmt, fromz);
 
 				if (dt_unk_p(d2)) {
-					if (!argi->quiet_given) {
+					if (!argi->quiet_flag) {
 						dt_io_warn_strpdt(line);
+						rc = 2;
+					}
+					if (argi->empty_mode_flag) {
+						/* empty line */
+						__io_write("\n", 1U, stdout);
 					}
 					continue;
+				} else if (UNLIKELY(d2.fix) &&
+					   !argi->quiet_flag) {
+					rc = 2;
 				}
 				/* guess the diff type */
-				dtyp = determine_durtype(d, d2, dfmt);
-				if (dtyp == DT_UNK) {
-					if (!argi->quiet_given) {
+				onlydp = dt_sandwich_only_d_p(d) ||
+					dt_sandwich_only_d_p(d2);
+				if (!(dtyp = determine_durtype(d, d2, dfmt))) {
+					if (!argi->quiet_flag) {
 						dt_io_warn_dur(refinp, line);
+						rc = 2;
 					}
 					continue;
 				}
 				/* perform subtraction now */
 				dur = dt_dtdiff(dtyp, d, d2);
-				ddiff_prnt(dur, ofmt, dfmt);
+				ddiff_prnt(dur, ofmt, dfmt, onlydp);
 			}
 		}
 		/* get rid of resources */
 		free_prchunk(pctx);
 	}
 
-	if (argi->from_zone_given) {
-		zif_close(fromz);
+	dt_io_clear_zones();
+	if (argi->from_locale_arg) {
+		setilocale(NULL);
 	}
 
 out:
-	cmdline_parser_free(argi);
-	return res;
+	yuck_free(argi);
+	return rc;
 }
 
 /* ddiff.c ends here */
