@@ -1,6 +1,6 @@
 /*** dgrep.c -- grep for lines with dates
  *
- * Copyright (C) 2011-2012 Sebastian Freundt
+ * Copyright (C) 2011-2020 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -42,41 +42,25 @@
 #include <stdint.h>
 #include <sys/time.h>
 #include <time.h>
-#include <errno.h>
-#include <stdarg.h>
 
 #include "dt-core.h"
+#include "dt-core-tz-glue.h"
 #include "dt-io.h"
-#include "tzraw.h"
 #include "dexpr.h"
+#include "dt-locale.h"
 #include "prchunk.h"
 
-
-/* error impl */
-static void
-__attribute__((format(printf, 2, 3)))
-error(int eno, const char *fmt, ...)
-{
-	va_list vap;
-	va_start(vap, fmt);
-	fputs("dgrep: ", stderr);
-	vfprintf(stderr, fmt, vap);
-	va_end(vap);
-	if (eno) {
-		fputc(':', stderr);
-		fputc(' ', stderr);
-		fputs(strerror(eno), stderr);
-	}
-	fputc('\n', stderr);
-	return;
-}
+const char *prog = "dgrep";
 
+
 /* dexpr subsystem */
 #include "dexpr.c"
 
 struct prln_ctx_s {
 	struct grep_atom_soa_s *ndl;
 	dexpr_t root;
+	zif_t fromz;
+	zif_t z;
 	unsigned int only_matching_p:1U;
 	unsigned int invert_match_p:1U;
 };
@@ -84,19 +68,24 @@ struct prln_ctx_s {
 static void
 proc_line(struct prln_ctx_s ctx, char *line, size_t llen)
 {
-	char *osp;
-	char *oep;
+	char *osp = NULL;
+	char *oep = NULL;
 
 	/* check if line matches,
 	 * there's currently no way to specify NEEDLE */
-	for (char *lp = line, *sp, *ep; ; lp = ep, osp = sp, oep = ep) {
+	for (char *lp = line, *const zp = line + llen, *sp, *ep;
+	     /*no check*/; lp = ep, osp = sp, oep = ep) {
 		struct dt_dt_s d =
-			dt_io_find_strpdt2(lp, ctx.ndl, &sp, &ep, NULL);
+			dt_io_find_strpdt2(
+				lp, zp - lp, ctx.ndl, &sp, &ep, ctx.fromz);
 		bool unkp = dt_unk_p(d);
 
 		if (unkp) {
 			/* just plain nothing */
 			break;
+		} else if (ctx.z != NULL) {
+			/* promote to zone ctx.z */
+			d = dtz_enrichz(d, ctx.z);
 		}
 		/* otherwise */
 		if (dexpr_matches_p(ctx.root, d)) {
@@ -118,6 +107,10 @@ proc_line(struct prln_ctx_s ctx, char *line, size_t llen)
 		if (!ctx.only_matching_p) {
 			osp = line;
 			oep = line + llen;
+		} else if (osp == NULL || oep == NULL) {
+			/* no date in line and only-matching is active
+			 * bugger off */
+			return;
 		}
 		/* finish the line and bugger off */
 		*oep++ = '\n';
@@ -127,71 +120,60 @@ proc_line(struct prln_ctx_s ctx, char *line, size_t llen)
 }
 
 
-#if defined __INTEL_COMPILER
-# pragma warning (disable:593)
-# pragma warning (disable:181)
-#elif defined __GNUC__
-# pragma GCC diagnostic ignored "-Wswitch-enum"
-# pragma GCC diagnostic ignored "-Wunused-function"
-#endif	/* __INTEL_COMPILER */
-#include "dgrep.xh"
-#include "dgrep.x"
-#if defined __INTEL_COMPILER
-# pragma warning (default:593)
-# pragma warning (default:181)
-#elif defined __GNUC__
-# pragma GCC diagnostic warning "-Wswitch-enum"
-# pragma GCC diagnostic warning "-Wunused-function"
-#endif	/* __INTEL_COMPILER */
+#include "dgrep.yucc"
 
 int
 main(int argc, char *argv[])
 {
-	struct gengetopt_args_info argi[1];
+	yuck_t argi[1U];
 	char **fmt;
 	size_t nfmt;
 	dexpr_t root;
 	oper_t o = OP_UNK;
 	int res = 0;
 
-	if (cmdline_parser(argc, argv, argi)) {
+	if (yuck_parse(argi, argc, argv)) {
 		res = 1;
 		goto out;
 	}
 
 	/* init and unescape sequences, maybe */
-	ckv_fmt = fmt = argi->input_format_arg;
-	ckv_nfmt = nfmt = argi->input_format_given;
-	if (argi->backslash_escapes_given) {
+	ckv_fmt = fmt = argi->input_format_args;
+	ckv_nfmt = nfmt = argi->input_format_nargs;
+	if (argi->backslash_escapes_flag) {
 		for (size_t i = 0; i < nfmt; i++) {
 			dt_io_unescape(fmt[i]);
 		}
 	}
+	if (argi->base_arg) {
+		struct dt_dt_s base = dt_strpdt(argi->base_arg, NULL, NULL);
+		dt_set_base(base);
+	}
 
-	if (argi->eq_given) {
+	if (argi->eq_flag) {
 		o = OP_EQ;
-	} else if (argi->ne_given) {
+	} else if (argi->ne_flag) {
 		o = OP_NE;
-	} else if (argi->lt_given || argi->ot_given) {
+	} else if (argi->lt_flag || argi->ot_flag) {
 		o = OP_LT;
-	} else if (argi->le_given) {
+	} else if (argi->le_flag) {
 		o = OP_LE;
-	} else if (argi->gt_given || argi->nt_given) {
+	} else if (argi->gt_flag || argi->nt_flag) {
 		o = OP_GT;
-	} else if (argi->ge_given) {
+	} else if (argi->ge_flag) {
 		o = OP_GE;
 	}
 	/* parse the expression */
-	if (argi->inputs_num == 0 || 
-	    dexpr_parse(&root, argi->inputs[0], strlen(argi->inputs[0])) < 0) {
+	if (argi->nargs == 0U || 
+	    dexpr_parse(&root, argi->args[0U], strlen(argi->args[0U])) < 0) {
 		res = 1;
-		error(0, "Error: need an expression to grep");
+		error("Error: need an expression to grep");
 		goto out;
 	}
 	/* fixup o, default is OP_EQ */
 	if (o != OP_UNK && root->type != DEX_VAL) {
 		res = 1;
-		error(0, "\
+		error("\
 long opt operators (--lt, --gt, ...) cannot be used in conjunction \n\
 with complex expressions");
 		goto out;
@@ -200,6 +182,10 @@ with complex expressions");
 		root->kv->op = o;
 	}
 
+	if (argi->from_locale_arg) {
+		setilocale(argi->from_locale_arg);
+	}
+
 	/* otherwise bring dexpr to normal form */
 	dexpr_simplify(root);
 	/* beef */
@@ -213,8 +199,10 @@ with complex expressions");
 		struct prln_ctx_s prln = {
 			.ndl = &ndlsoa,
 			.root = root,
-			.only_matching_p = argi->only_matching_given,
-			.invert_match_p = argi->invert_match_given,
+			.fromz = dt_io_zone(argi->from_zone_arg),
+			.z = dt_io_zone(argi->zone_arg),
+			.only_matching_p = argi->only_matching_flag,
+			.invert_match_p = argi->invert_match_flag,
 		};
 
 		/* no threads reading this stream */
@@ -231,7 +219,7 @@ with complex expressions");
 
 		/* using the prchunk reader now */
 		if ((pctx = init_prchunk(STDIN_FILENO)) == NULL) {
-			error(errno, "Error: could not open stdin");
+			serror("Error: could not open stdin");
 			goto ndl_free;
 		}
 		while (prchunk_fill(pctx) >= 0) {
@@ -250,8 +238,12 @@ with complex expressions");
 	}
 	/* resource freeing */
 	free_dexpr(root);
+	dt_io_clear_zones();
+	if (argi->from_locale_arg) {
+		setilocale(NULL);
+	}
 out:
-	cmdline_parser_free(argi);
+	yuck_free(argi);
 	return res;
 }
 
