@@ -1,6 +1,6 @@
 /*** dseq.c -- like seq(1) but for dates
  *
- * Copyright (C) 2009 - 2011 Sebastian Freundt
+ * Copyright (C) 2009 - 2020 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -44,11 +44,10 @@
 #include <stdbool.h>
 #include <time.h>
 #include <string.h>
-#include <stdarg.h>
-#include <errno.h>
 
 #include "dt-core.h"
 #include "dt-io.h"
+#include "dt-locale.h"
 #include "tzraw.h"
 
 typedef uint8_t __skipspec_t;
@@ -57,9 +56,9 @@ typedef uint8_t __skipspec_t;
 struct dseq_clo_s {
 	struct dt_dt_s fst;
 	struct dt_dt_s lst;
-	struct dt_dt_s *ite;
+	struct dt_dtdur_s *ite;
 	size_t nite;
-	struct dt_dt_s *altite;
+	struct dt_dtdur_s *altite;
 	__skipspec_t ss;
 	size_t naltite;
 	/* direction, >0 if increasing, <0 if decreasing, 0 if undefined */
@@ -68,25 +67,7 @@ struct dseq_clo_s {
 #define CLO_FL_FREE_ITE		(1)
 };
 
-
-/* impl of error() */
-static void
-__attribute__((format(printf, 2, 3)))
-error(int eno, const char *fmt, ...)
-{
-	va_list vap;
-	va_start(vap, fmt);
-	fputs("dseq: ", stderr);
-	vfprintf(stderr, fmt, vap);
-	va_end(vap);
-	if (eno) {
-		fputc(':', stderr);
-		fputc(' ', stderr);
-		fputs(strerror(eno), stderr);
-	}
-	fputc('\n', stderr);
-	return;
-}
+const char *prog = "dseq";
 
 
 /* skip system */
@@ -257,16 +238,23 @@ set_skip(__skipspec_t ss, char *spec)
 }
 
 static struct dt_dt_s
-date_add(struct dt_dt_s d, struct dt_dt_s dur[], size_t ndur)
+date_add(struct dt_dt_s d, struct dt_dtdur_s dur[], size_t ndur)
 {
+	int32_t carries = d.d.u;
+
 	for (size_t i = 0; i < ndur; i++) {
 		d = dt_dtadd(d, dur[i]);
+		/* keep track of carries */
+		carries += d.t.carry;
+	}
+	if (UNLIKELY(dt_sandwich_only_t_p(d))) {
+		d.d.u = carries;
 	}
 	return d;
 }
 
 static void
-date_neg_dur(struct dt_dt_s dur[], size_t ndur)
+date_neg_dur(struct dt_dtdur_s dur[], size_t ndur)
 {
 	for (size_t i = 0; i < ndur; i++) {
 		dur[i] = dt_neg_dtdur(dur[i]);
@@ -275,32 +263,44 @@ date_neg_dur(struct dt_dt_s dur[], size_t ndur)
 }
 
 static bool
-__daisy_feasible_p(struct dt_dt_s dur[], size_t ndur)
+__daisy_feasible_p(struct dt_dtdur_s dur[], size_t ndur)
 {
 	if (ndur != 1) {
 		return false;
-	} else if (dur->typ == (dt_dttyp_t)DT_YMD) {
-		if (dur->d.ymd.y || dur->d.ymd.m) {
-			return false;
-		}
-	} else if (dur->typ == (dt_dttyp_t)DT_MD) {
-		if (dur->d.md.m) {
-			return false;
-		}
-	} else if (dur->typ == (dt_dttyp_t)DT_BIZDA && (dur->d.bizda.bd)) {
+	}
+
+	switch (dur->d.durtyp) {
+	case DT_DURYMD:
+		return !(dur->d.ymd.y || dur->d.ymd.m);
+	case DT_DURBIZDA:
+		return !dur->d.bizda.bd;
+	case DT_DURD:
+	case DT_DURBD:
+		/* definitley, they're daisy already */
+		return true;
+	case DT_DURWK:
+		/* borderline, could be less efficient for ywd dates */
+		return true;
+	case DT_DURMO:
+	case DT_DURQU:
+	case DT_DURYR:
+		/* most definitely not */
+	default:
+		/* all the time durs make it infeasible as well */
 		return false;
 	}
-	return true;
+	/* not reached */
 }
 
 static bool
-__dur_naught_p(struct dt_dt_s dur)
+__dur_naught_p(struct dt_dtdur_s dur)
 {
-	return dur.d.u == 0 && dur.t.sdur == 0;
+	/* we use the fact that dur.dv overlaps dur.d.dv */
+	return dur.dv == 0;
 }
 
 static bool
-__durstack_naught_p(struct dt_dt_s dur[], size_t ndur)
+__durstack_naught_p(struct dt_dtdur_s dur[], size_t ndur)
 {
 	if (ndur == 0) {
 		return true;
@@ -316,31 +316,42 @@ __durstack_naught_p(struct dt_dt_s dur[], size_t ndur)
 }
 
 static bool
-__in_range_p(struct dt_dt_s now, struct dseq_clo_s *clo)
+__in_range_p(struct dt_dt_s now, const struct dseq_clo_s *clo)
 {
 	if (!dt_sandwich_only_t_p(now)) {
-		return (dt_dt_in_range_p(now, clo->fst, clo->lst) ||
-			dt_dt_in_range_p(now, clo->lst, clo->fst));
+		if (clo->dir > 0) {
+			return dt_dt_in_range_p(now, clo->fst, clo->lst) == 1;
+		} else if (clo->dir < 0) {
+			return dt_dt_in_range_p(now, clo->lst, clo->fst) == 1;
+		}
 	}
 	/* otherwise perform a simple range check */
 	if (clo->dir > 0) {
-		if (now.t.u >= clo->fst.t.u && now.t.u <= clo->lst.t.u) {
-			return true;
-		} else if (clo->fst.t.u >= clo->lst.t.u) {
-			return now.t.u <= clo->lst.t.u || now.d.daisydur == 0;
+		if (clo->fst.t.u < clo->lst.t.u) {
+			/* dseq A B  with A < B */
+			return now.t.u >= clo->fst.t.u &&
+				now.t.u <= clo->lst.t.u;
+		} else {
+			/* dseq A B  with A > B and wrap-around,
+			 * carries have kindly been stored in d.u */
+			return now.t.u <= clo->lst.t.u || now.d.u == 0U;
 		}
 	} else if (clo->dir < 0) {
-		if (now.t.u <= clo->fst.t.u && now.t.u >= clo->lst.t.u) {
-			return true;
-		} else if (clo->fst.t.u <= clo->lst.t.u) {
-			return now.t.u >= clo->lst.t.u || now.d.daisydur == 0;
+		if (clo->fst.t.u > clo->lst.t.u) {
+			/* counting down from A to B */
+			return now.t.u <= clo->fst.t.u &&
+				now.t.u >= clo->lst.t.u;
+		} else {
+			/* count down from A to B with wrap around,
+			 * carries have kindly been stored in d.u */
+			return now.t.u >= clo->lst.t.u || now.d.u == 0U;
 		}
 	}
 	return false;
 }
 
 static struct dt_dt_s
-__seq_altnext(struct dt_dt_s now, struct dseq_clo_s *clo)
+__seq_altnext(struct dt_dt_s now, const struct dseq_clo_s *clo)
 {
 	do {
 		now = date_add(now, clo->altite, clo->naltite);
@@ -349,7 +360,7 @@ __seq_altnext(struct dt_dt_s now, struct dseq_clo_s *clo)
 }
 
 static struct dt_dt_s
-__seq_this(struct dt_dt_s now, struct dseq_clo_s *clo)
+__seq_this(struct dt_dt_s now, const struct dseq_clo_s *clo)
 {
 /* if NOW is on a skip date, find the next date according to ALTITE, then ITE */
 	if (!skipp(clo->ss, now) && __in_range_p(now, clo)) {
@@ -369,7 +380,7 @@ __seq_this(struct dt_dt_s now, struct dseq_clo_s *clo)
 }
 
 static struct dt_dt_s
-__seq_next(struct dt_dt_s now, struct dseq_clo_s *clo)
+__seq_next(struct dt_dt_s now, const struct dseq_clo_s *clo)
 {
 /* advance NOW, then fix it */
 	struct dt_dt_s tmp = date_add(now, clo->ite, clo->nite);
@@ -377,18 +388,16 @@ __seq_next(struct dt_dt_s now, struct dseq_clo_s *clo)
 }
 
 static int
-__get_dir(struct dt_dt_s d, struct dseq_clo_s *clo)
+__get_dir(struct dt_dt_s d, const struct dseq_clo_s *clo)
 {
-	struct dt_dt_s tmp;
-
 	if (!dt_sandwich_only_t_p(d)) {
 		/* trial addition to to see where it goes */
-		tmp = __seq_next(d, clo);
+		struct dt_dt_s tmp = __seq_next(d, clo);
 		return dt_dtcmp(tmp, d);
 	}
-	if (clo->ite->t.sdur && !clo->ite->t.neg) {
+	if (clo->ite->dv > 0) {
 		return 1;
-	} else if (clo->ite->t.sdur && clo->ite->t.neg) {
+	} else if (clo->ite->dv < 0) {
 		return -1;
 	}
 	return 0;
@@ -415,65 +424,41 @@ __fixup_fst(struct dseq_clo_s *clo)
 	return old;
 }
 
-static struct dt_t_s
+static struct dt_dtdur_s
 tseq_guess_ite(struct dt_t_s beg, struct dt_t_s end)
 {
-	struct dt_t_s res;
+	struct dt_dtdur_s res = {(dt_dtdurtyp_t)DT_DURUNK};
 
 	if (beg.hms.h != end.hms.h &&
-	    beg.hms.m == 0 && end.hms.m == 0&&
+	    beg.hms.m == 0 && end.hms.m == 0 &&
 	    beg.hms.s == 0 && end.hms.s == 0) {
-		if (beg.u < end.u) {
-			res.sdur = SECS_PER_HOUR;
-		} else {
-			res.sdur = -SECS_PER_HOUR;
-		}
+		res.durtyp = DT_DURH;
+		res.dv = (beg.u < end.u) ? 1 : -1;
 	} else if (beg.hms.m != end.hms.m &&
 		   beg.hms.s == 0 && end.hms.s == 0) {
-		if (beg.u < end.u) {
-			res.sdur = SECS_PER_MIN;
-		} else {
-			res.sdur = -SECS_PER_MIN;
-		}
+		res.durtyp = DT_DURM;
+		res.dv = (beg.u < end.u) ? 1 : -1;
 	} else {
-		if (beg.u < end.u) {
-			res.sdur = 1L;
-		} else {
-			res.sdur = -1L;
-		}
+		res.durtyp = DT_DURS;
+		res.dv = (beg.u < end.u) ? 1 : -1;
 	}
-	res.dur = 1;
 	return res;
 }
 
 
-#if defined __INTEL_COMPILER
-# pragma warning (disable:593)
-# pragma warning (disable:181)
-#elif defined __GNUC__
-# pragma GCC diagnostic ignored "-Wswitch-enum"
-# pragma GCC diagnostic ignored "-Wunused-function"
-#endif	/* __INTEL_COMPILER */
-#include "dseq.xh"
-#include "dseq.x"
-#if defined __INTEL_COMPILER
-# pragma warning (default:593)
-# pragma warning (default:181)
-#elif defined __GNUC__
-# pragma GCC diagnostic warning "-Wswitch-enum"
-# pragma GCC diagnostic warning "-Wunused-function"
-#endif	/* __INTEL_COMPILER */
+#include "dseq.yucc"
 
 int
 main(int argc, char *argv[])
 {
-	static struct dt_dt_s ite_p1;
-	struct gengetopt_args_info argi[1];
+	static struct dt_dtdur_s ite_p1;
+	yuck_t argi[1U];
 	struct dt_dt_s tmp;
 	char **ifmt;
 	size_t nifmt;
 	char *ofmt;
-	int res = 0;
+	dt_dttyp_t tgttyp;
+	int rc = 0;
 	struct dseq_clo_s clo = {
 		.ite = &ite_p1,
 		.nite = 1,
@@ -484,35 +469,44 @@ main(int argc, char *argv[])
 		.flags = 0,
 	};
 
-	/* fixup negative numbers, A -1 B for dates A and B */
-	fixup_argv(argc, argv, NULL);
-	if (cmdline_parser(argc, argv, argi)) {
-		res = 1;
+	if (yuck_parse(argi, argc, argv)) {
+		rc = 1;
 		goto out;
 	}
 	/* assign ofmt/ifmt */
 	ofmt = argi->format_arg;
-	if (argi->backslash_escapes_given) {
+	if (argi->backslash_escapes_flag) {
 		dt_io_unescape(ofmt);
 	}
-	nifmt = argi->input_format_given;
-	ifmt = argi->input_format_arg;
+	nifmt = argi->input_format_nargs;
+	ifmt = argi->input_format_args;
 
-	for (size_t i = 0; i < argi->skip_given; i++) {
-		clo.ss = set_skip(clo.ss, argi->skip_arg[i]);
+	if (argi->from_locale_arg) {
+		setilocale(argi->from_locale_arg);
+	}
+	if (argi->locale_arg) {
+		setflocale(argi->locale_arg);
 	}
 
-	if (argi->alt_inc_given) {
-		struct __strpdtdur_st_s st = __strpdtdur_st_initialiser();
+	if (argi->base_arg) {
+		struct dt_dt_s base = dt_strpdt(argi->base_arg, NULL, NULL);
+		dt_set_base(base);
+	}
+
+	for (size_t i = 0; i < argi->skip_nargs; i++) {
+		clo.ss = set_skip(clo.ss, argi->skip_args[i]);
+	}
+
+	if (argi->alt_inc_arg) {
+		struct __strpdtdur_st_s st = {0};
 
-		unfixup_arg(argi->alt_inc_arg);
 		do {
 			if (dt_io_strpdtdur(&st, argi->alt_inc_arg) < 0) {
-				if (!argi->quiet_given) {
-					error(errno, "Error: \
+				if (!argi->quiet_flag) {
+					error("Error: \
 cannot parse duration string `%s'", argi->alt_inc_arg);
 				}
-				res = 1;
+				rc = 1;
 				goto out;
 			}
 		} while (__strpdtdur_more_p(&st));
@@ -521,93 +515,89 @@ cannot parse duration string `%s'", argi->alt_inc_arg);
 		clo.naltite = st.ndurs;
 	}
 
-	switch (argi->inputs_num) {
+	switch (argi->nargs) {
 		struct dt_dt_s fst, lst;
 	default:
-		cmdline_parser_print_help();
-		res = 1;
+		yuck_auto_help(argi);
+		rc = 1;
 		goto out;
 
 	case 2:
-		lst = dt_io_strpdt(argi->inputs[1], ifmt, nifmt, NULL);
+		lst = dt_io_strpdt(argi->args[1U], ifmt, nifmt, NULL);
 		if (dt_unk_p(lst)) {
-			if (!argi->quiet_given) {
-				dt_io_warn_strpdt(argi->inputs[1]);
+			if (!argi->quiet_flag) {
+				dt_io_warn_strpdt(argi->args[1U]);
 			}
-			res = 1;
+			rc = 1;
 			goto out;
+		} else if (UNLIKELY(lst.fix) && !argi->quiet_flag) {
+			rc = 2;
 		}
 		/* fallthrough */
 	case 1:
-		fst = dt_io_strpdt(argi->inputs[0], ifmt, nifmt, NULL);
+		fst = dt_io_strpdt(argi->args[0U], ifmt, nifmt, NULL);
 		if (dt_unk_p(fst)) {
-			if (!argi->quiet_given) {
-				dt_io_warn_strpdt(argi->inputs[0]);
+			if (!argi->quiet_flag) {
+				dt_io_warn_strpdt(argi->args[0U]);
 			}
-			res = 1;
+			rc = 1;
 			goto out;
+		} else if (UNLIKELY(fst.fix) && !argi->quiet_flag) {
+			rc = 2;
 		}
 
 		/* check the input arguments and do the sane thing now
-		 * if it's all dates, use daisy iterator
-		 * if it's all times, use sdur iterator
+		 * if it's all dates, use DURD iterator
+		 * if it's all times, use DURS/DURM/DURH iterators
 		 * if one of them is a dt, promote the other */
 		if (dt_sandwich_only_d_p(fst)) {
 			/* emulates old dseq(1) */
-			if (argi->inputs_num == 1) {
-				lst.d = dt_date(DT_YMD);
-				dt_make_d_only(&lst, DT_YMD);
+			if (argi->nargs == 1U) {
+				lst.d = dt_date(fst.d.typ);
+				dt_make_d_only(&lst, fst.d.typ);
 			}
-
-			dt_make_d_only(clo.ite, DT_DAISY);
-			clo.ite->d.daisy = 1;
+			clo.ite->d = dt_make_ddur(DT_DURD, 1);
 		} else if (dt_sandwich_only_t_p(fst)) {
 			/* emulates old tseq(1) */
-			if (argi->inputs_num == 1) {
+			if (argi->nargs == 1U) {
 				lst.t = dt_time();
 				dt_make_t_only(&lst, DT_HMS);
 			}
-			/* let the guesser do the work */
-			clo.ite->t.sdur = 0;
 		} else if (dt_sandwich_p(fst)) {
-			if (argi->inputs_num == 1) {
-				lst = dt_datetime((dt_dttyp_t)DT_YMD);
-				dt_make_sandwich(&lst, DT_YMD, DT_HMS);
+			if (argi->nargs == 1U) {
+				lst = dt_datetime(fst.typ);
+				dt_make_sandwich(&lst, fst.d.typ, DT_HMS);
 			}
-
-			dt_make_sandwich(clo.ite, DT_DAISY, DT_TUNK);
-			clo.ite->d.daisy = 1;
+			clo.ite->d = dt_make_ddur(DT_DURD, 1);
 		} else {
-			error(0, "\
+			error("\
 don't know how to handle single argument case");
-			res = 1;
+			rc = 1;
 			goto out;
 		}
+		goto make_compat;
 
-		clo.fst = fst;
-		clo.lst = lst;
-		clo.ite->dur = 1;
-		break;
 	case 3: {
-		struct __strpdtdur_st_s st = __strpdtdur_st_initialiser();
+		struct __strpdtdur_st_s st = {0};
 
 		/* get lower bound */
-		fst = dt_io_strpdt(argi->inputs[0], ifmt, nifmt, NULL);
+		fst = dt_io_strpdt(argi->args[0U], ifmt, nifmt, NULL);
 		if (dt_unk_p(fst)) {
-			if (!argi->quiet_given) {
-				dt_io_warn_strpdt(argi->inputs[0]);
+			if (!argi->quiet_flag) {
+				dt_io_warn_strpdt(argi->args[0U]);
 			}
-			res = 1;
+			rc = 1;
 			goto out;
+		} else if (UNLIKELY(fst.fix) && !argi->quiet_flag) {
+			rc = 2;
 		}
 
 		/* get increment */
-		unfixup_arg(argi->inputs[1]);
 		do {
-			if (dt_io_strpdtdur(&st, argi->inputs[1]) < 0) {
-				error(0, "Error: \
-cannot parse duration string `%s'", argi->inputs[1]);
-				res = 1;
+			if (dt_io_strpdtdur(&st, argi->args[1U]) < 0) {
+				error("Error: \
+cannot parse duration string `%s'", argi->args[1U]);
+				rc = 1;
 				goto out;
 			}
 		} while (__strpdtdur_more_p(&st));
@@ -617,26 +607,36 @@ cannot parse duration string `%s'", argi->inputs[1]);
 		clo.flags |= CLO_FL_FREE_ITE;
 
 		/* get upper bound */
-		lst = dt_io_strpdt(argi->inputs[2], ifmt, nifmt, NULL);
+		lst = dt_io_strpdt(argi->args[2U], ifmt, nifmt, NULL);
 		if (dt_unk_p(lst)) {
-			if (!argi->quiet_given) {
-				dt_io_warn_strpdt(argi->inputs[2]);
+			if (!argi->quiet_flag) {
+				dt_io_warn_strpdt(argi->args[2U]);
 			}
-			res = 1;
+			rc = 1;
 			goto out;
+		} else if (UNLIKELY(lst.fix) && !argi->quiet_flag) {
+			rc = 2;
 		}
-		clo.fst = fst;
-		clo.lst = lst;
-		break;
+		goto make_compat;
 	}
+
+	make_compat:
+		if (LIKELY(fst.typ == lst.typ)) {
+			clo.fst = fst;
+			clo.lst = lst;
+		} else {
+			clo.fst = fst;
+			clo.lst = dt_dtconv(fst.typ, lst);
+		}
+		break;
 	}
 
 	/* promote the args maybe */
 	if ((dt_sandwich_only_d_p(clo.fst) && dt_sandwich_only_t_p(clo.lst)) ||
 	    (dt_sandwich_only_t_p(clo.fst) && dt_sandwich_only_d_p(clo.lst))) {
-		error(0, "\
+		error("\
 cannot mix dates and times as arguments");
-		res = 1;
+		rc = 1;
 		goto out;
 	} else if (dt_sandwich_only_d_p(clo.fst) && dt_sandwich_p(clo.lst)) {
 		/* promote clo.fst */
@@ -656,38 +656,55 @@ cannot mix dates and times as arguments");
 		dt_make_sandwich(&clo.lst, clo.lst.d.typ, clo.fst.t.typ);
 	}
 
-#define x_DAISY	((dt_dttyp_t)DT_DAISY)
-	/* convert to daisies */
-	if (dt_sandwich_only_d_p(clo.fst) &&
-	    __daisy_feasible_p(clo.ite, clo.nite) &&
-	    ((clo.fst = dt_dtconv(x_DAISY, clo.fst)).d.typ != DT_DAISY ||
-	     (clo.lst = dt_dtconv(x_DAISY, clo.lst)).d.typ != DT_DAISY)) {
-		if (!argi->quiet_given) {
-			error(0, "\
+#define _DAISY	((dt_dttyp_t)DT_DAISY)
+	tgttyp = clo.fst.typ;
+	if ((dt_sandwich_p(clo.fst) || dt_sandwich_only_d_p(clo.fst)) &&
+	    clo.fst.d.typ == DT_YMD && clo.fst.d.ymd.m == 0 &&
+	    argi->nargs < 3U) {
+		/* iterate year-wise */
+		clo.ite->d = dt_make_ddur(DT_DURYR, 1);
+	} else if ((dt_sandwich_p(clo.fst) || dt_sandwich_only_d_p(clo.fst)) &&
+		   clo.fst.d.typ == DT_YMD && clo.fst.d.ymd.d == 0 &&
+		   argi->nargs < 3U) {
+		/* iterate month-wise */
+		clo.ite->d = dt_make_ddur(DT_DURMO, 1);
+	} else if (dt_sandwich_only_d_p(clo.fst) &&
+		   __daisy_feasible_p(clo.ite, clo.nite) &&
+		   clo.fst.d.typ == DT_YMD &&
+		   /* convert to daisies */
+		   ((clo.fst = dt_dtconv(_DAISY, clo.fst)).d.typ != DT_DAISY ||
+		    (clo.lst = dt_dtconv(_DAISY, clo.lst)).d.typ != DT_DAISY)) {
+		if (!argi->quiet_flag) {
+			error("\
 cannot convert calendric system internally");
 		}
-		res = 1;
+		rc = 1;
 		goto out;
-	} else if (dt_sandwich_only_t_p(clo.fst) && clo.ite->t.sdur == 0) {
-		clo.ite->t = tseq_guess_ite(clo.fst.t, clo.lst.t);
+	} else if (dt_sandwich_only_t_p(clo.fst) && clo.ite->dv == 0) {
+		*clo.ite = tseq_guess_ite(clo.fst.t, clo.lst.t);
 	}
 
 	if (__durstack_naught_p(clo.ite, clo.nite) ||
-	    (clo.dir = __get_dir(clo.fst, &clo)) == 0) {
-		if (!argi->quiet_given) {
-			error(0, "\
+	    !(clo.dir = __get_dir(clo.fst, &clo))) {
+		if (!argi->quiet_flag) {
+			error("\
 increment must not be naught");
 		}
-		res = 1;
+		rc = 1;
 		goto out;
-	} else if (argi->compute_from_last_given) {
+	} else if (argi->compute_from_last_flag) {
 		tmp = __fixup_fst(&clo);
 	} else {
 		tmp = __seq_this(clo.fst, &clo);
 	}
 
-	for (; __in_range_p(tmp, &clo); tmp = __seq_next(tmp, &clo)) {
-		dt_io_write(tmp, ofmt, NULL, '\n');
+	for (; __in_range_p(dt_fixup(tmp), &clo); tmp = __seq_next(tmp, &clo)) {
+		struct dt_dt_s tgt = tmp;
+
+		if (LIKELY(ofmt == NULL)) {
+			tgt = dt_dtconv(tgttyp, tmp);
+		}
+		dt_io_write(tgt, ofmt, NULL, '\n');
 	}
 
 out:
@@ -698,8 +715,14 @@ out:
 	if (clo.altite != NULL) {
 		free(clo.altite);
 	}
-	cmdline_parser_free(argi);
-	return res;
+	if (argi->from_locale_arg) {
+		setilocale(NULL);
+	}
+	if (argi->locale_arg) {
+		setflocale(NULL);
+	}
+	yuck_free(argi);
+	return rc;
 }
 
 /* dseq.c ends here */
