@@ -1,6 +1,6 @@
 /*** prchunk.c -- guessing line oriented data formats
  *
- * Copyright (C) 2010-2012 Sebastian Freundt
+ * Copyright (C) 2010-2020 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -46,7 +46,6 @@
 #endif	/* MAP_ANON_NEEDS_ALL_SOURCE */
 #include <stddef.h>
 #include <stdlib.h>
-#include <stdio.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <string.h>
@@ -63,6 +62,8 @@
 
 #if !defined MAP_ANONYMOUS && defined MAP_ANON
 # define MAP_ANONYMOUS	(MAP_ANON)
+#elif !defined MAP_ANON
+# define MAP_ANON	(0x1000U)
 #endif	/* MAP_ANON->MAP_ANONYMOUS */
 
 #if defined __INTEL_COMPILER
@@ -78,40 +79,21 @@ struct prch_ctx_s {
 	/* buffer */
 	char *buf;
 	/* number of lines in the buffer */
-	uint32_t lno;
+	uint32_t tot_lno;
 	/* number of columns per line */
-	uint32_t cno;
+	uint32_t tot_cno;
 	/* number of bytes in the buffer */
 	size_t bno;
 	/* last known offset */
 	size_t off;
 	/* offsets */
 	off32_t loff[MAX_NLINES];
-	off32_t lno_cur;
+	off32_t cur_lno;
 	/* delimiter offsets */
 	off16_t *soff;
 };
 
 
-/* error() impl */
-static void
-__attribute__((format(printf, 2, 3)))
-error(int eno, const char *fmt, ...)
-{
-	va_list vap;
-	va_start(vap, fmt);
-	fputs("prchunk: ", stderr);
-	vfprintf(stderr, fmt, vap);
-	va_end(vap);
-	if (eno) {
-		fputc(':', stderr);
-		fputc(' ', stderr);
-		fputs(strerror(eno), stderr);
-	}
-	fputc('\n', stderr);
-	return;
-}
-
 static inline void
 set_loff(prch_ctx_t ctx, uint32_t lno, off32_t off)
 {
@@ -162,13 +144,12 @@ prchunk_fill(prch_ctx_t ctx)
  * bytes read */
 #define CHUNK_SIZE	(4096)
 #define YIELD(x)	goto yield##x
-	int res = 0;
 	char *off = ctx->buf + 0;
 	char *bno = ctx->buf + ctx->bno;
 	ssize_t nrd;
 
 	/* initial work, reset the line counters et al */
-	ctx->lno = 0;
+	ctx->tot_lno = 0;
 	/* we just memcpy() the left over stuff to the front and restart
 	 * from there, someone left us a note in __ctx with the left
 	 * over offset */
@@ -203,10 +184,18 @@ yield1:
 	 * has been called, then off would be 0 and __ctx->bno would be
 	 * the buffer filled so far, if no more bytes could be read then
 	 * we'd proceed processing them (off < __ctx->bno + nrd */
-	if (UNLIKELY(nrd <= 0 && off == ctx->buf)) {
+	if (UNLIKELY(!nrd && off < bno && ctx->cur_lno <= ctx->tot_lno)) {
+		/* last line then, unyielded :| */
+		set_loff(ctx, 0, bno - ctx->buf);
+		YIELD(4);
+	} else if (UNLIKELY(nrd <= 0 && off == ctx->buf)) {
 		/* special case, we worked our arses off and nothing's
 		 * in the pipe line so just fuck off here */
-		return -1;
+		if (!ctx->bno) {
+			return -1;
+		}
+		/* go to drain mode */
+		YIELD(2);
 	} else if (LIKELY(off < bno || off == ctx->buf)) {
 		YIELD(2);
 	}
@@ -220,21 +209,20 @@ yield2:
 			if (LIKELY(nrd > 0)) {
 				break;
 			}
-			/* fucking idiots didnt conclude with a \n */
-			error(0, "ID:10T error");
-			p = bno;
+			/* not concluded with \n, let's hope we're in drain mode */
+			return -1;
 		}
 		/* massage our status structures */
-		set_loff(ctx, ctx->lno, p - ctx->buf);
+		set_loff(ctx, ctx->tot_lno, p - ctx->buf);
 		if (UNLIKELY(p[-1] == '\r')) {
 			/* oh god, when is this nightmare gonna end */
 			p[-1] = '\0';
-			set_lftermd(ctx, ctx->lno);
+			set_lftermd(ctx, ctx->tot_lno);
 		}
 		*p = '\0';
 		off = ++p;
 		/* count it as line and check if we need more */
-		if (++ctx->lno >= MAX_NLINES) {
+		if (++ctx->tot_lno >= MAX_NLINES) {
 			YIELD(3);
 		}
 	}
@@ -242,12 +230,13 @@ yield2:
 yield3:
 	/* need clean up, something like unread(),
 	 * in particular leave a note in __ctx with the left over offset */
-	ctx->lno_cur = 0;
+	ctx->cur_lno = 0;
+yield4:
 	ctx->off = off - ctx->buf;
 	ctx->bno = bno - ctx->buf;
 #undef YIELD
 #undef CHUNK_SIZE
-	return res;
+	return 0;
 }
 
 
@@ -255,7 +244,7 @@ yield3:
 FDEFU prch_ctx_t
 init_prchunk(int fd)
 {
-#define MAP_MEM		(MAP_ANONYMOUS | MAP_PRIVATE)
+#define MAP_MEM		(MAP_ANON | MAP_PRIVATE)
 #define PROT_MEM	(PROT_READ | PROT_WRITE)
 #define MAP_LEN		(MAX_NLINES * MAX_LLEN)
 	static struct prch_ctx_s __ctx;
@@ -274,7 +263,12 @@ init_prchunk(int fd)
 	if ((__ctx.fd = fd) > STDIN_FILENO) {
 #if defined POSIX_FADV_SEQUENTIAL
 		/* give advice about our read pattern */
-		posix_fadvise(fd, 0, 0, POSIX_FADV_SEQUENTIAL);
+		int rc = posix_fadvise(fd, 0, 0, POSIX_FADV_SEQUENTIAL);
+
+		if (UNLIKELY(rc < 0)) {
+			munmap(__ctx.soff, MAP_LEN);
+			return NULL;
+		}
 #endif	/* POSIX_FADV_SEQUENTIAL */
 	}
 	return &__ctx;
@@ -295,7 +289,7 @@ free_prchunk(prch_ctx_t ctx)
 FDEFU size_t
 prchunk_get_nlines(prch_ctx_t ctx)
 {
-	return ctx->lno;
+	return ctx->tot_lno;
 }
 
 FDEFU size_t
@@ -316,34 +310,35 @@ prchunk_getlineno(prch_ctx_t ctx, char **p, int lno)
 FDEFU size_t
 prchunk_getline(prch_ctx_t ctx, char **p)
 {
-	return prchunk_getlineno(ctx, p, ctx->lno_cur++);
+	return prchunk_getlineno(ctx, p, ctx->cur_lno++);
 }
 
 FDEFU void
 prchunk_reset(prch_ctx_t ctx)
 {
-	ctx->lno_cur = 0;
+	ctx->cur_lno = 0;
 	return;
 }
 
 FDEFU int
 prchunk_haslinep(prch_ctx_t ctx)
 {
-	return ctx->lno_cur < ctx->lno;
+/* the second condition is to allow unterminated last lines */
+	return ctx->cur_lno < ctx->tot_lno || ctx->cur_lno == 0U;
 }
 
 
 static inline void
 set_ncols(prch_ctx_t ctx, size_t ncols)
 {
-	ctx->cno = ncols;
+	ctx->tot_cno = ncols;
 	return;
 }
 
 FDEFU size_t
 prchunk_get_ncols(prch_ctx_t ctx)
 {
-	return ctx->cno;
+	return ctx->tot_cno;
 }
 
 static inline void
@@ -425,6 +420,8 @@ prchunk_getcolno(prch_ctx_t ctx, char **p, int lno, int cno)
 
 
 #if defined STANDALONE
+#include <stdio.h>
+
 int
 main(int argc, char *argv[])
 {
