@@ -1,6 +1,6 @@
 /*** strptime.c -- a shell interface to strptime(3)
  *
- * Copyright (C) 2011-2012 Sebastian Freundt
+ * Copyright (C) 2011-2020 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -45,38 +45,19 @@
 #include <stdio.h>
 #include <sys/time.h>
 #include <time.h>
-#include <stdarg.h>
-#include <errno.h>
+#include <locale.h>
 
 #include "dt-io.h"
 #include "prchunk.h"
 
-
-/* error() impl */
-static void
-__attribute__((format(printf, 2, 3)))
-error(int eno, const char *fmt, ...)
-{
-	va_list vap;
-	va_start(vap, fmt);
-	fputs("strptime: ", stderr);
-	vfprintf(stderr, fmt, vap);
-	va_end(vap);
-	if (eno) {
-		fputc(':', stderr);
-		fputc(' ', stderr);
-		fputs(strerror(eno), stderr);
-	}
-	fputc('\n', stderr);
-	return;
-}
+const char *prog = "strptime";
 
 
 static int
 pars_line(struct tm *tm, const char *const *fmt, size_t nfmt, const char *line)
 {
 	for (size_t i = 0; i < nfmt; i++) {
-		if (strptime(line, fmt[i], tm) != NULL) {
+		if (fmt[i] && strptime(line, fmt[i], tm) != NULL) {
 			return 0;
 		}
 	}
@@ -92,133 +73,124 @@ prnt_line(const char *ofmt, struct tm *tm)
 	return;
 }
 
-static inline __attribute__((pure, const)) struct tm
-__tm_initialiser(void)
-{
-#if defined HAVE_SLOPPY_STRUCTS_INIT
-	static const struct tm res = {};
-#else
-	static const struct tm res;
-#endif	/* HAVE_SLOPPY_STRUCTS_INIT */
-	return res;
-}
-
-static void
+static int
 proc_line(
 	const char *ln, const char *const *fmt, size_t nfmt,
 	const char *ofmt,
 	int quietp)
 {
-	struct tm tm = __tm_initialiser();
+	struct tm tm = {0};
+	int rc = 0;
 
 	if (pars_line(&tm, fmt, nfmt, ln) < 0) {
 		if (!quietp) {
 			dt_io_warn_strpdt(ln);
+			rc = 2;
 		}
 	} else {
 		prnt_line(ofmt, &tm);
 	}
-	return;
+	return rc;
 }
 
-static void
+static int
 proc_lines(const char *const *fmt, size_t nfmt, const char *ofmt, int quietp)
 {
 	size_t lno = 0;
+	int rc = 0;
 	void *pctx;
 
 	/* using the prchunk reader now */
 	if ((pctx = init_prchunk(STDIN_FILENO)) == NULL) {
-		error(errno, "Error: could not open stdin");
-		return;
+		serror("Error: could not open stdin");
+		return 1;
 	}
 	while (prchunk_fill(pctx) >= 0) {
 		for (char *line; prchunk_haslinep(pctx); lno++) {
 			(void)prchunk_getline(pctx, &line);
 			/* check if line matches */
-			proc_line(line, fmt, nfmt, ofmt, quietp);
+			rc |= proc_line(line, fmt, nfmt, ofmt, quietp);
 		}
 	}
 	/* get rid of resources */
 	free_prchunk(pctx);
-	return;
+	return rc;
 }
 
 
-#if defined __INTEL_COMPILER
-# pragma warning (disable:593)
-# pragma warning (disable:181)
-#elif defined __GNUC__
-# pragma GCC diagnostic ignored "-Wswitch-enum"
-# pragma GCC diagnostic ignored "-Wunused-function"
-#endif	/* __INTEL_COMPILER */
-#include "strptime.xh"
-#include "strptime.x"
-#if defined __INTEL_COMPILER
-# pragma warning (default:593)
-# pragma warning (default:181)
-#elif defined __GNUC__
-# pragma GCC diagnostic warning "-Wswitch-enum"
-# pragma GCC diagnostic warning "-Wunused-function"
-#endif	/* __INTEL_COMPILER */
+#include "strptime.yucc"
 
 int
 main(int argc, char *argv[])
 {
 	static char dflt_fmt[] = "%Y-%m-%d\n\0H:%M:%S %Z\n";
-	struct gengetopt_args_info argi[1];
+	yuck_t argi[1U];
 	char *outfmt = dflt_fmt;
 	char **infmt;
 	size_t ninfmt;
 	char **input;
 	size_t ninput;
 	int quietp;
-	int res = 0;
+	int rc = 0;
 
-	if (cmdline_parser(argc, argv, argi)) {
-		res = 1;
+	if (yuck_parse(argi, argc, argv)) {
+		rc = 1;
 		goto out;
 	}
 
-	if (argi->format_given) {
+	if (argi->format_arg) {
 		outfmt = argi->format_arg;
 		/* unescape sequences, maybe */
-		if (argi->backslash_escapes_given) {
+		if (argi->backslash_escapes_flag) {
 			dt_io_unescape(outfmt);
 		}
-	} else if (argi->time_given) {
+	} else if (argi->time_flag) {
 		outfmt[8] = ' ';
 		outfmt[9] = '%';
 	}
 
-	if (!argi->input_format_given) {
-		infmt = argi->inputs;
-		ninfmt = argi->inputs_num;
+	if (!argi->input_format_nargs) {
+		infmt = argi->args;
+		ninfmt = argi->nargs;
 		input = NULL;
 		ninput = 0;
 	} else {
-		infmt = argi->input_format_arg;
-		ninfmt = argi->input_format_given;
-		input = argi->inputs;
-		ninput = argi->inputs_num;
+		infmt = argi->input_format_args;
+		ninfmt = argi->input_format_nargs;
+		input = argi->args;
+		ninput = argi->nargs;
 	}
 	/* get quiet predicate */
-	quietp = argi->quiet_given;
+	quietp = argi->quiet_flag;
+
+	/* set locale specific/independent behaviour */
+	with (const char *loc) {
+		if (!argi->locale_flag) {
+			loc = "C";
+			/* we need to null out TZ for UTC */
+			setenv("TZ", "", 1);
+		} else {
+			loc = "";
+		}
+		/* actually set our findings in stone */
+		setlocale(LC_TIME, loc);
+		tzset();
+	}
 
 	/* get lines one by one, apply format string and print date/time */
 	if (ninput == 0) {
 		/* read from stdin */
-		proc_lines((const char*const*)infmt, ninfmt, outfmt, quietp);
+		rc |= proc_lines((const char*const*)infmt, ninfmt, outfmt, quietp);
 	} else {
 		const char *const *cinfmt = (const char*const*)infmt;
 		for (size_t i = 0; i < ninput; i++) {
-			proc_line(input[i], cinfmt, ninfmt, outfmt, quietp);
+			rc |= proc_line(input[i], cinfmt, ninfmt, outfmt, quietp);
 		}
 	}
 
 out:
-	cmdline_parser_free(argi);
-	return res;
+	yuck_free(argi);
+	return rc;
 }
 
 /* strptime.c ends here */
